/*
    Copyright (C) 2011 - 2012 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/


#include "efilesystemplugin.h"
#include "efilesystemplugin.moc"

#include <efileinfo.h>
#include <eopenfileexternal.h>

#include <QtPlugin>
#include <QFileInfo>
#include <QFile>
#include <QMessageBox>

QString EFileSystemPlugin::name() const
{
	return tr("Local Filesystem");
}

QString EFileSystemPlugin::serviceId() const
{
	return "efilesystem";
}

QString EFileSystemPlugin::description() const
{
	return tr("Handles files on the local filesystem.");
}

EFileInfo EFileSystemPlugin::info(QString uniqueId) const
{
	EFileInfo result;
	QFileInfo info(uniqueId);
	if(!info.isAbsolute())
		return result;

	result.setId(uniqueId);
	result.setName(uniqueId);
	if(info.exists())
		result.setSize(info.size());
	return result;
}

EFileInfo EFileSystemPlugin::add(QFileInfo info, bool create) const
{
	EFileInfo result = this->info(info.absoluteFilePath());
	if(!result.isValid())
		return EFileInfo();
	
	if(!info.exists()){
		if(!create)
			return EFileInfo();
		
		// create it
		QFile file(info.absoluteFilePath());
		if(!file.open(QFile::WriteOnly))
			return EFileInfo();
		file.close();
	}
	return result;
}

bool EFileSystemPlugin::move(EFileInfo source, QString target) const
{
	bool result = copy(source, target);
	if(!result)
		return false;

	result = QFile::remove(source.id());
	if(result)
		return true;

	// cleaning up copied stuff, because we cannot delete source
	QFile::remove(target);
	return false;
}

bool EFileSystemPlugin::copy(EFileInfo source, QString target) const
{
	if(!source.isValid())
		return false;

	QFileInfo sourceInfo(source.id());

	// we can only handle files right now!
	if(!sourceInfo.isFile())
		return false;

	QFile sourceFile(source.id());
	if(!sourceFile.exists())
		return false;

	QFileInfo targetFile(target);
	// Ask for override existing file
	if(targetFile.exists()){
		QMessageBox::StandardButton result = QMessageBox::question(
			NULL,
			tr("Overwrite existing file?"),
					tr("File\n%1\nalready exists, overwrite it?").arg(target),
			QMessageBox::Yes | QMessageBox::No,
			QMessageBox::No
		);
		if(result == QMessageBox::No)
			return false;

		// we must remove the existing file, because QFile does
		// not overwriteit.
		if(!QFile::remove(target))
			return false;
	}
	return sourceFile.copy(target);
}

bool EFileSystemPlugin::deleteFile(EFileInfo file) const
{
	if(!file.isValid())
		return false;
	return QFile::remove(file.id());
}

bool EFileSystemPlugin::open(EFileInfo file)
{
	if(!file.isValid())
		return false;
	return EOpenFileExternal::openFile(file.id(), "", this);
}

ESettingsWidget* EFileSystemPlugin::settingsWidget() const
{
	return NULL;
}

Q_EXPORT_PLUGIN2 ( LibEngsas_EFileSystemPlugin, EFileSystemPlugin )
