/*
    Copyright (C) 2011 - 2012 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/

#include "testefilesystemplugin.h"
#include "testefilesystemplugin.moc"

#include "efilesystemplugintestconfig.h"

#include <epluginloader.h>
#include <efileserviceplugin.h>
#include <efileinfo.h>

#include <QTest>
#include <QTemporaryFile>
#include <QDir>

void TestEFileSystemPlugin::initTestCase()
{
	plugin = EPluginLoader<EFileServicePlugin>::loadPlugin("efilesystemplugin", QString(LIBOUTDIR));
	if(!plugin)
		QSKIP("Plugin not found", SkipAll);
}

void TestEFileSystemPlugin::cleanupTestCase()
{
	foreach(QString dir, removeTempDirs)
		QVERIFY(QDir::temp().rmdir(dir));

	foreach(QString file, removeTempFiles)
		QVERIFY(QFile::remove(file));
}

void TestEFileSystemPlugin::testBasicQuality()
{
	testPlugin(plugin, false);
}

void TestEFileSystemPlugin::testInfo()
{
	EFileInfo info = plugin->info("");
	QCOMPARE(info.isValid(), false);

	QTemporaryFile temp;
	QVERIFY(temp.open());

	temp.write(QByteArray("This is a greate test"));
	temp.flush();

	info = plugin->info(temp.fileName());
	QCOMPARE(info.isValid(), true);
	QCOMPARE(info.id(), temp.fileName());
	QCOMPARE(info.name(), temp.fileName());
	QCOMPARE(info.size(), (quint64)temp.size());
}

void TestEFileSystemPlugin::testAdd()
{
	// adding bullshit
	QCOMPARE(plugin->add(QFileInfo(), false).isValid(), false);
	QCOMPARE(plugin->add(QFileInfo("asdasd"), false).isValid(), false);

	// opening existing files
	QTemporaryFile temp;
	QVERIFY(temp.open());
	QCOMPARE(plugin->add(QFileInfo(temp.fileName()), false).isValid(), true);
	QCOMPARE(plugin->add(QFileInfo(temp.fileName()), true).isValid(), true);

	// adding non existent files
	QFileInfo info(QDir::tempPath() +"/11111111");
	QCOMPARE(info.exists(), false);
	QCOMPARE(plugin->add(info, false).isValid(), false);
	QCOMPARE(info.exists(), false);
	QCOMPARE(plugin->add(info, true).isValid(), true);
	info.refresh();
	QCOMPARE(info.exists(), true);
	QVERIFY(QFile::remove(info.absoluteFilePath()));
}

void TestEFileSystemPlugin::testDeleteFile()
{
	// bullshit
	QCOMPARE(plugin->deleteFile(EFileInfo()), false);
	EFileInfo test;
	test.setId("asdasd");
	test.setName("adsdasd");
	QCOMPARE(plugin->deleteFile(test), false);

	// deliting existent
	QFileInfo info(QDir::tempPath() +"/222222222");
	QCOMPARE(info.exists(), false);
	EFileInfo eInfo = plugin->add(info, true);
	QCOMPARE(eInfo.isValid(), true);
	info.refresh();
	QCOMPARE(info.exists(), true);
	QCOMPARE(plugin->deleteFile(eInfo), true);
	info.refresh();
	QCOMPARE(info.exists(), false);
}

void TestEFileSystemPlugin::testCopy()
{
	EFileInfo malformed;
	malformed.setId("asdasd");
	malformed.setName("asdasd");

	// bullshit copy
	QCOMPARE(plugin->copy(EFileInfo(), ""), false);
	QCOMPARE(plugin->copy(EFileInfo(), "asdasdasd"), false);
	QCOMPARE(plugin->copy(malformed, "asdasdasdasdasd"), false);

	QString target = QDir::tempPath() +"/333333333";
	QTemporaryFile temp;
	QVERIFY(temp.open());
	EFileInfo info = plugin->add(temp.fileName(), true);
	QCOMPARE(plugin->copy(info, target), true);
	QCOMPARE(QFileInfo(info.id()).exists(), true);
	QCOMPARE(QFileInfo(target).exists(), true);
	QVERIFY(QFile::remove(target));

	// copy dir
	info.setId(QDir::tempPath());
	info.setName(info.id());
	bool result = plugin->copy(info, target);
	QEXPECT_FAIL("", "Directories are not supported right now", Continue);
	QCOMPARE(result, true);
	if(result)
		QVERIFY(QDir(QDir::tempPath()).remove(target));
}

void TestEFileSystemPlugin::testMove()
{
	EFileInfo malformed;
	malformed.setId("asdasd");
	malformed.setName("asdasd");

	// bullshit move
	QCOMPARE(plugin->move(EFileInfo(), ""), false);
	QCOMPARE(plugin->move(EFileInfo(), "asdasdasd"), false);
	QCOMPARE(plugin->move(malformed, "asdasdasdasdasd"), false);

	QString source = QDir::tempPath() +"/4444444444";
	QString target = QDir::tempPath() +"/5555555555";
	EFileInfo info = plugin->add(source, true);
	QCOMPARE(info.isValid(), true);
	QCOMPARE(plugin->move(info, target), true);
	QCOMPARE(QFileInfo(info.id()).exists(), false);
	QCOMPARE(QFileInfo(target).exists(), true);
	QVERIFY(QFile::remove(target));

	// copy dir
	source = "4444444444";
	QDir tempDir(QDir::tempPath());
	QVERIFY(tempDir.mkdir(source));

	info.setId(QDir::tempPath() +"/"+ source);
	info.setName(info.id());

	bool result = plugin->move(info, target);
	if(!result)
		removeTempDirs << source;
	QEXPECT_FAIL("", "Directories are not supported right now", Continue);
	QCOMPARE(result, true);
	if(result)
		QVERIFY(tempDir.remove(target));
}

QTEST_MAIN(TestEFileSystemPlugin)
