/*
    Copyright (C) 2010 - 2012 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/

#include "epartexeconfig.h"
#include <epart.h>
#include <elibrary.h>
#include <epartmainwindow.h>
#include <ecmdlineargs.h>

#include <QApplication>
#include <QMessageBox>
#include <QObject>

#ifdef BINDINGS_KDE
	#include <KAboutData>
	#include <KApplication>
	#include <KCmdLineArgs>
	#include <KCmdLineOptions>
#endif

#ifdef _MSC_VER
	#include <windows.h>
#endif

/**
	* @brief This Application is used to execute EPart's in standalonoe mode.
	*
	* To load an EPart, call
	* @code
	* epartexe [options of the epart] EPartName
	* # or
	* epartexe [options of the epart] /absoulute/path/to/epartname
	* @endcode
	*
  * @author Joachim Langenbach
  */
int main ( int argc, char** argv )
{
	// find out, which EPart should be used.
	// The EPart should be the last option given
	QString ePartName = QString(argv[argc - 1]);
	QString error;

	// setup the command line options
	EPart *ePart = ELibrary::loadEPartFirstTime(ePartName, error);
	if(!error.isEmpty() || !ePart){
		// we use a QMessageBox here, because EPartExe may be launched
		// without a terminal.
		// The QApplication is needed by the QMessageBox
		QApplication app ( argc, argv );
		QMessageBox::critical(0, "EPartExe", "Loading of the EPart failed\n\n" + error);
		return 1;
	}

	ECmdLineArgs *args = new ECmdLineArgs(ePart);
	foreach(ECmdLineOption *option, ePart->cmdLineArgs())
		args->add(option);

	// if --help or --version is used, init returns false
	// and outputs the help
	if(!args->init(argc, argv))
		return 1;

	// instanciate the QApplication or KApplication
#ifdef BINDINGS_KDE
	// the first argument is needed, otherwise a message box
	// with "Please contact your system administrator"
	// appears
	// Also KAboutData execpts only KlocalizedStrings, but there is no direct way
	// to convert QStrings into KlocalizedStrings. Therefore the hack with toUtf8()
	// and ki18n ist used.
	KAboutData aboutData(
		ePartName.toUtf8(), QByteArray(), ki18n(ePart->name().toUtf8()),
		ePart->versionString().toAscii(), ki18n(ePart->description().toUtf8()),
		KAboutData::License_GPL, ki18n(""),
		ki18n(""), ePart->website().toString().toAscii(),
		QByteArray()
	);
	
	KCmdLineArgs::init( argc, argv, &aboutData);
  KCmdLineOptions epart;
	args->addToKCmdLineOptions(&epart);
  epart.add("+epartname", ki18n(QObject::tr("The name of the EPart, which should be loaded").toUtf8()));
  KCmdLineArgs::addCmdLineOptions(epart);
	KApplication app;
#else
  QApplication app ( argc, argv );
	// handle EPart standard arguments
#endif
	ePart = ELibrary::loadEPart(ePart, ePartName, error);
	if(!error.isEmpty() || !ePart){
		QMessageBox::critical(0, "EPartExe", "Loading of the EPart failed\n\n" + error);
		return 1;
	}

	EPartMainWindow *mw = new EPartMainWindow();
	EPart::InitState result =  mw->init(ePart, args);
	switch(result){
    case EPart::Exit:
			return 0;
		case EPart::Failed:
			return 1;
		// this handles mainly the Successfull return value ???
		default:
			break;
	}
	mw->show();
  return app.exec();
}

#ifdef _MSC_VER
int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nShowCmd)
{
	return main(__argc, __argv);
}
#endif
