/*
    Copyright (C) 2012 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/

#include "echartmarkers.h"

#include "echartmarkersconfig.h"

#include <QPainter>
#include <QMatrix>

#include <cmath>

QPointer<EChartMarkersConfig> EChartMarkers::markersConfig = NULL;

EChartMarkers::EChartMarkers()
{
	config();
}

EChartMarkersConfig *EChartMarkers::config()
{
	if(!markersConfig)
		markersConfig = new EChartMarkersConfig();
	return markersConfig;
}

void EChartMarkers::circle(QPainter *painter, QPoint point)
{
	if(!painter)
		return;

	// make sure, markersConfig exists
	EChartMarkers markers;

	QRect rect(point.x() - markers.halfSize(), point.y() - markers.halfSize(), markers.size(), markers.size());
	painter->drawEllipse(rect);
}

void EChartMarkers::rhomb(QPainter *painter, QPoint point)
{
	if(!painter)
		return;

	// make sure, markersConfig exists
	EChartMarkers markers;

	QPolygon polygon;
	polygon << QPoint(point.x() - markers.halfSize(), point.y());
	polygon << QPoint(point.x(), point.y() + markers.halfSize());
	polygon << QPoint(point.x() + markers.halfSize(), point.y());
	polygon << QPoint(point.x(), point.y() - markers.halfSize());
	painter->drawPolygon(polygon);
}

void EChartMarkers::square(QPainter *painter, QPoint point)
{
	if(!painter)
		return;

	// make sure, markersConfig exists
	EChartMarkers markers;

	QRect rect(point.x() - markers.halfSize(), point.y() - markers.halfSize(), markers.size(), markers.size());
	painter->drawRect(rect);
}

void EChartMarkers::triangleUp(QPainter *painter, QPoint point)
{
	EChartMarkers marker;
	marker.triangle(painter, point, 0);
}

void EChartMarkers::triangleLeft(QPainter *painter, QPoint point)
{
	EChartMarkers marker;
	marker.triangle(painter, point, 90);
}

void EChartMarkers::triangleBottom(QPainter *painter, QPoint point)
{
	EChartMarkers marker;
	marker.triangle(painter, point, 180);
}

void EChartMarkers::triangleRight(QPainter *painter, QPoint point)
{
	EChartMarkers marker;
	marker.triangle(painter, point, 270);
}

uint EChartMarkers::size() const
{
	 return config()->size();
}

int EChartMarkers::halfSize() const
{
	return floor(float(size())/2.0);
}

void EChartMarkers::triangle(QPainter *painter, QPoint point, int angle)
{
	if(!painter)
		return;

	EChartMarkers markers;
	qreal half = qreal(markers.halfSize());
	// distance between middle and top
	qreal a = half / cos(0.523598776);
	// distance between middle and bottom
	qreal b = pow(3, 0.5) * half - a;

	QPolygonF polygon;
	polygon << QPointF(0, a) << QPointF(half, -b) << QPointF(-half, -b);

	QMatrix matrix1, matrix2;
	polygon = matrix1.rotate(angle).map(polygon);
	polygon = matrix2.translate(point.x(), point.y()).map(polygon);
	painter->drawPolygon(polygon);
}
