/*
    Copyright (C) 2012 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/

#ifndef ECHARTPIE_H
#define ECHARTPIE_H

#include "echartbase.h"
#include "echarts.h"

/**
	* @brief Class to draw pie charts.
	*
	* Just add one data set per piece (with one x-value). The x-Values
	* should be the values used to calculate the pie.
	* If the x-values are already percents, call also setPercents(true).
	* Otherwise EChartPie calculates the percentages automatically.
	*
	* To set the number format and precision, use setNumberFormat() and
	* setNumberPrecision().
	*
	*/
class E_CHARTS_EXPORT EChartPie : public EChartBase
{
	Q_OBJECT
	Q_PROPERTY(bool percents READ percents WRITE setPercents)
	public:
		EChartPie(QWidget *parent = NULL);
		void setPercents(bool percent);
		inline bool percents() const { return arePercents; }

	protected:
		/**
			* @todo Calculate percentages once and not everytime.
			*/
		void drawChart(QPainter *painter, QRect viewPort);

	private:
		QString percentLabel(float percentage);

		bool arePercents;
		double PI;
	
};

#endif // ECHARTPIE_H
