/*
    Copyright (C) 2010 - 2012 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/

#include "eaboutdialog.h"
#include "eaboutdialog.moc"
#include "ui_eaboutdialog.h"

#include "epushbutton.h"
#include "elibrary.h"
#include "eopenfileexternal.h"
#include "eaboutmodel.h"

#include <QUrl>

EAboutDialog::EAboutDialog(QPixmap logo, QString name, QString version,
													 QUrl url, EAboutModel *model,
													 QWidget* parent, Qt::WindowFlags flags)
	: EDialog("eAboutDialog", parent, flags), ui(new Ui::EAboutDialog)
{
	ui->setupUi(this);
	connect(ui->rightLabel, SIGNAL(linkActivated(QString)), this, SLOT(openLink(QString)));
	connect(ui->closeButton, SIGNAL(clicked()), this, SLOT(close()));
	setLogo(logo);
	setName(name);
	setVersion(version);
	setApplicationUrl(url);
	setModel(model);
}

void EAboutDialog::setLogo(QPixmap logo)
{
	if(logo.isNull())
		return;
	ui->logoLabel->setMinimumSize(logo.size());
	ui->logoLabel->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
	ui->logoLabel->setPixmap(logo);
}

void EAboutDialog::setName(QString name)
{
	myName = name;
	setupTextLabel();
}

void EAboutDialog::setVersion(QString version)
{
	myVersion = version;
	setupTextLabel();
}

void EAboutDialog::setApplicationUrl(QUrl url)
{
	myUrl = url;
	if(url.isRelative())
		myUrl = ELibrary::website().resolved(url);
	setupTextLabel();
}

void EAboutDialog::setModel(EAboutModel* model)
{
	if(!model)
		return;
	model->connectByPass(this, model);
	ui->detailsEdit->setHtml(model->toHtml());
	delete model;
}

void EAboutDialog::openLink(QString url)
{
	EOpenFileExternal::openFile(url, "", this);
}

void EAboutDialog::setupTextLabel()
{
		ui->rightLabel->setText(QString(
									"<table>"
									"	<tr><th colspan=\"2\"><h3>%1</h3></th></tr>"
									"	<tr><td>"+ tr("Version:") +"</td><td>%2</td></tr>"
									"	<tr><td>"+ tr("Homepage:") +"</td><td><a href=\"%3\">%3</a></td></tr>"
									" <tr><th colspan=\"2\" height=\"0.1em\">&nbsp;</td></tr>"
									"	<tr><td>"+ tr("Manufacturer:") +"</td><td>%4</td></tr>"
									"	<tr><td>"+ tr("Homepage:") +"</td><td><a href=\"%5\">%5</a></td></tr>"
									"</table>"
									)
									.arg(myName)
									.arg(myVersion)
									.arg(myUrl.toString())
									.arg(ELibrary::manufacturer())
									.arg(ELibrary::website().toString())
									);
}
