/*
    Copyright (C) 2010 - 2012 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/

#include "eaboutmodel.h"
#include "eaboutmodel.moc"

#include <QMetaObject>
#include <QMetaEnum>

EAboutModel::EAboutModel(QObject* parent)
	: EAbstractItemModel(parent)
{
	
}

EAboutModel::~EAboutModel()
{
	qDeleteAll(items);
}

QString EAboutModel::toHtml() const
{
	if(rowCount() < 1)
		return QString();
	
	QString result = "<table width=100%>";
	
	// getting all EAboutPerson::Category to walk through
	QMetaObject meta = EAboutPerson::staticMetaObject;
	QMetaEnum catEnum = meta.enumerator(meta.indexOfEnumerator("Category"));
	QMetaEnum catFlags = meta.enumerator(meta.indexOfEnumerator("Categories"));
	if(!catEnum.isValid() || !catFlags.isValid())
		return QString();
	for(int i = 0; i < catEnum.keyCount(); i++){
		QStringList lines;
		for(int j = 0; j < rowCount(); j++){
			EAboutPerson *person = itemFromIndex(index(j, 0));
			if(person){
				// test whether current person is listed
				// at the actual category
				EAboutPerson::Categories cats = person->categories();
				if(cats.testFlag(EAboutPerson::Category(catEnum.value(i)))){
					lines << QString("  <tr><td>%1 %2 %3</td></tr>")
											.arg(person->givenName())
											.arg(person->name())
											.arg(person->email());
				}
			}
		}
		if(lines.size() > 0){
			result += QString("  <tr><th align=left>%1</th></tr><tr><th /></tr>").arg(catEnum.valueToKey(catEnum.value(i)));
			result += lines.join("\n");
			result += "  <tr><th></th></tr>";
		}
	}
	result += "</table>";
	return result;
}

void EAboutModel::addPerson(EAboutPerson* person)
{
	if(!person)
		return;
	connectByPass(this, person);
	emit layoutAboutToBeChanged();
	items << person;
	emit layoutChanged();
}

void EAboutModel::addPerson(QString name, QString givenName, EAboutPerson::Categories categories)
{
	addPerson(new EAboutPerson(name, givenName, categories));
}

void EAboutModel::addPerson(QString name, QString givenName,
														QString email, EAboutPerson::Categories categories)
{
	addPerson(new EAboutPerson(name, givenName, email, categories));
}

EAboutPerson* EAboutModel::itemFromIndex(QModelIndex index) const
{
	if(!index.isValid())
		return 0;
	return static_cast<EAboutPerson*>(index.internalPointer());
}

QVariant EAboutModel::data(const QModelIndex& index, int role) const
{
	if(role != Qt::DisplayRole && role != Qt::EditRole)
		return QVariant();
	
	EAboutPerson *person = itemFromIndex(index);
	if(!person)
		return QVariant();
	
	switch(index.column()){
		case 0:
			return person->name();
		case 1:
			return person->givenName();
		case 2:
			return person->email();
	}
	return QVariant();
}

int EAboutModel::columnCount(const QModelIndex& parent) const
{
	if(parent != QModelIndex())
		return 0;
	return 3;
}

int EAboutModel::rowCount(const QModelIndex& parent) const
{
	if(parent != QModelIndex())
		return 0;
	return items.size();
}

QModelIndex EAboutModel::parent(const QModelIndex& child) const
{
	Q_UNUSED(child);
	return QModelIndex();
}

QModelIndex EAboutModel::index(int row, int column, const QModelIndex& parent) const
{
	if(parent != QModelIndex())
		return QModelIndex();
	
	if(row < 0 || row >= rowCount(parent))
		return QModelIndex();
	
	if(column < 0 || column >= columnCount(parent))
		return QModelIndex();
	
	return createIndex(row, column, items[row]);
}

QVariant EAboutModel::headerData(int section, Qt::Orientation orientation, int role) const
{
	if(role != Qt::DisplayRole)
		return QVariant();
	
	if(orientation != Qt::Horizontal)
		return section + 1;
	
	switch(section){
		case 0:
			return tr("Name");
		case 1:
			return tr("Given Name");
		case 2:
			return tr("E-Mail");
	}
	return QVariant();
}
