/*
    Copyright (C) 2010 - 2012 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/

#include "eabstractitemmodel.h"
#include "eabstractitemmodel.moc"

EAbstractItemModel::EAbstractItemModel(QObject* parent)
	: QAbstractItemModel(parent)
{
	connect(this, SIGNAL(rowsInserted(const QModelIndex &, int, int)),
					this, SLOT(rowsInsertedByPass(const QModelIndex &, int, int)));
	connect(this, SIGNAL(rowsRemoved(const QModelIndex &, int, int)),
					this, SLOT(rowsRemovedByPass(const QModelIndex &, int, int)));
}

void EAbstractItemModel::connectByPass(QObject* parent, EAbstractItemModel* child)
{
	if(!parent || !child)
		return;
	
	connect(child, SIGNAL(error(QString)), parent, SLOT(errorByPass(QString)));
	connect(child, SIGNAL(warning(QString)), parent, SLOT(warningByPass(QString)));
	connect(child, SIGNAL(information(QString)), parent, SLOT(informationByPass(QString)));
}

void EAbstractItemModel::connectByPass(EAbstractItemModel* parent, QObject* child)
{
	connect(child, SIGNAL(error(QString)), parent, SLOT(errorByPass(QString)));
	connect(child, SIGNAL(warning(QString)), parent, SLOT(warningByPass(QString)));
	connect(child, SIGNAL(information(QString)), parent, SLOT(informationByPass(QString)));
}

void EAbstractItemModel::rowsInsertedByPass(const QModelIndex &parent, int start, int end)
{
	emit eRowsInserted(parent, start, end);
}

void EAbstractItemModel::rowsRemovedByPass(const QModelIndex &parent, int start, int end)
{
	emit eRowsRemoved(parent, start, end);
}
