/*
    Copyright (C) 2010 - 2012 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/

#include "eaction.h"
#include "eaction.moc"

#include <QSettings>
#include <QIcon>

EAction::EAction ( QString standardIconName, QString iconPath, const QString& text, QObject* parent, bool enableText )
	: QAction ( parent )
{
	init(enableText);
	setIcon(standardIconName, iconPath);
	setText(text);
}

EAction::EAction(const QString& text, QObject* parent)
	: QAction(parent)
{
	init(false);
	setText(text);
}

EAction::~EAction()
{
	if(myActionWithText)
		delete myActionWithText;
}

QAction* EAction::actionWithText()
{
	// return ourself, if the text is not hidden
	if(text() == myText)
		return this;
	if(!myActionWithText){
		myActionWithText = new QAction(icon(), myText, this);
		connect(myActionWithText, SIGNAL(triggered()), this, SLOT(triggeredByPass()));
	}
	return myActionWithText;	
}

void EAction::setText(const QString& text)
{
	myText = text;
	if(toolTip().isEmpty())
		setToolTip(myText);
	
	if(iconsAndText || icon().isNull() || showText())
		QAction::setText(myText);
	else
		QAction::setText("");

	if(myActionWithText)
		myActionWithText->setText(myText);
}

void EAction::setIcon(const QIcon& icon)
{
	QAction::setIcon(icon);
}

void EAction::setIcon(QString standardIconName, QString iconPath )
{
#if QT_VERSION < 0x040600
	setIcon(QIcon(iconPath));
#else
	setIcon(QIcon::fromTheme(standardIconName, QIcon(iconPath)));
#endif
}

void EAction::setShowText(bool anytime)
{
	myShowText = anytime;
	setText(myText);
}

bool EAction::showText() const
{
	return myShowText;
}

void EAction::setEnabled ( bool status )
{
	if(myActionWithText)
		myActionWithText->setEnabled(status);
	QAction::setEnabled(status);
}

void EAction::setDisabled ( bool status )
{
	setEnabled(!status);
}

void EAction::triggeredByPass()
{
	emit triggered();
}

void EAction::init(bool enableText)
{
	myActionWithText = 0;
	QSettings qSettings;
  iconsAndText = qSettings.value ( "iconsAndText", false ).toBool();
	setShowText(enableText);
}
