/*
    Copyright (C) 2010 - 2012 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/

#include "edbbackupdialog.h"
#include "edbbackupdialog.moc"

#include "edbbackup.h"
#include "epushbutton.h"

#include <QVBoxLayout>
#include <QHBoxLayout>

EDBBackupDialog::EDBBackupDialog(QWidget* parent, Qt::WindowFlags flags)
	: EDialog("eDBBackupDialog", parent, flags)
{
	inProgress = false;
	
	setObjectName("DBBackupDialog");
	setWindowTitle(tr("Database Backups"));
	
	QVBoxLayout *ml = new QVBoxLayout(this);
	setLayout(ml);
	
	dbBackup = new EDBBackup(0, this);
	ml->addWidget(dbBackup);
	connectByPass(dbBackup);
	connect(dbBackup, SIGNAL(exportStarted()), this, SLOT(exportStarted()));
	connect(dbBackup, SIGNAL(exportFinished(bool)), this, SLOT(exportFinished(bool)));
	
	QHBoxLayout *bl = new QHBoxLayout();
	ml->addLayout(bl);
	
	EPushButton *button = new EPushButton(this);
	bl->addWidget(button);
	setHelp("keyword:DBBackup", button);
	
	bl->addStretch();
	
	exportButton = new EPushButton(tr("Export"), this);
	bl->addWidget(exportButton);
	connect(exportButton, SIGNAL(clicked()), this, SLOT(finish()));
	
	closeButton = new EPushButton(this);
	bl->addWidget(closeButton);
	connect(closeButton, SIGNAL(clicked()), this, SLOT(abort()));
	exportFinished(false);
}

void EDBBackupDialog::finish()
{
	dbBackup->finish();
}

void EDBBackupDialog::abort()
{
	if(inProgress){
		dbBackup->abort();
	}
	else{
		if(doneSuccessfully){
			accept();
		}
		else{
			reject();
		}
	}
}

void EDBBackupDialog::exportStarted()
{
	inProgress = true;
	exportButton->setEnabled(false);
	closeButton->setText(tr("Cancel"));
}

void EDBBackupDialog::exportFinished(bool result)
{
	inProgress = false;
	exportButton->setEnabled(true);
	closeButton->setText(tr("Close"));
	doneSuccessfully = result;
}
