/*
    Copyright (C) 2010 - 2012 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/

#include "edbsettings.h"
#include "edbsettings.moc"
#include "ui_edbsettings.h"

#include "esqllogin.h"
#include "esqldatabase.h"

#include <QSettings>
#include <QSqlQuery>
#include <QSqlError>

EDBSettings::EDBSettings ( QStringList drivers, ESetupWizardObject *newObject, QWidget *parent, Qt::WindowFlags fl )
    : ESetupWizardWidget ( newObject, parent, fl ), ui(new Ui::EDBSettings)
{
	setObjectName("eDBSettings");
  ui->setupUi ( this );
  setName ( tr ( "Database Settings" ) );

	if(!newObject)
		ui->createDatabaseBox->setVisible(false);

	ui->eDBSettingsWidget->setAvailableDrivers(drivers);
}

EDBSettings::~EDBSettings()
{
	delete ui;
}

bool EDBSettings::finish()
{
	QString msg;
  if ( !saveSettings(msg) )
    return false;

  if ( !ESqlLogin::checkSettings() ) {
    emit error ( tr ( "The database settings are wrong!" ) );
    return false;
  }

	// if create user or create database is selected,
	// we must use an admin connection
	QString userName = ui->createUserNameEdit->text();
	QString password = ui->createUserPasswordEdit->text();
	if(ui->createDBEdit->isChecked() || ui->createUserBox->isChecked()){
		QString dbName = ui->eDBSettingsWidget->database();

		QString adminConnName = "admin";
		ESqlLogin adminLogin(tr("Administration login"), adminConnName, this);
		connectByPass(&adminLogin);
		if( adminLogin.exec() != QDialog::Accepted){
			emit error(tr("Could not connect to database server with administrator privileges."));
			return false;
		}
		ESqlDatabase db = ESqlDatabase::database(adminConnName);
		if(!db.open()){
			emit error(tr("Could not connect to database server with administrator privileges."));
			return false;
		}
		QSqlQuery query(db);
		// first create user if needed
		if(ui->createUserBox->isChecked()){
			QString host = "%";
			if(userName.isEmpty()){
				emit error(tr("Empty Username is not allowed."));
				return false;
			}
			if(password.isEmpty()){
				emit error(tr("Empty password is not allowed."));
				return false;
			}

			// check wether user already exists
			// this query can not be used again (I think because of queryNotReuseable.first() later on)
			QSqlQuery queryNotReuseable(db);
			queryNotReuseable.prepare("SELECT User FROM mysql.user WHERE Host=:Host AND User=:User");
			queryNotReuseable.bindValue(":Host", host);
			queryNotReuseable.bindValue(":User", userName);
			if(!queryNotReuseable.exec()){
				emit error(tr("Could not check wether user exist."));
				db.close();
				return false;
			}
			if(queryNotReuseable.first()){
				// update password only, user exists already
				query.prepare("SET PASSWORD FOR ?@? = PASSWORD(?)");
				query.addBindValue(userName);
				query.addBindValue(host);
				query.addBindValue(password);
				if(!query.exec()){
					emit error(tr("Could not change password."));
					return false;
				}
			}
			else{
				// create new user
				query.prepare("CREATE USER ?@? IDENTIFIED BY ?");
				query.addBindValue(userName);
				query.addBindValue(host);
				query.addBindValue(password);
				if(!query.exec()){
					emit error(tr("Could not create user."));
					return false;
				}
			}
			// grant privileges
			query.prepare(QString("GRANT ALL ON %1.* TO ?@?").arg(dbName));
			query.addBindValue(userName);
			query.addBindValue(host);
			if(!query.exec()){
				emit error(tr("Could not grant privileges to chosen database."));
				return false;
			}
			if(!query.exec("FLUSH PRIVILEGES")){
				emit error(tr("Could not reload privileges."));
				return false;
			}
		}

		// create database
		if(ui->createDBEdit->isChecked()){
			// check wether database already exists
			if(!query.exec(QString("CREATE DATABASE IF NOT EXISTS %1").arg(dbName))){
				emit error(tr("Could not create database."));
				return false;
			}
		}
		db.close();
	}

  ESqlLogin sqlLogin(this);
	sqlLogin.loadAndCheckSettings(userName, "", "", "", password);
 	connectByPass ( &sqlLogin );

	bool result = false;
  if ( sqlLogin.exec() == QDialog::Accepted )
		result =  true;

  return result;
}

bool EDBSettings::saveSettings (QString& error)
{
	if(!ui->eDBSettingsWidget->saveSettings(error))
		return false;

	emit reloadSettingsRequest();
  return true;
}

void EDBSettings::reloadSettings()
{
	ESqlLogin sqlLogin(this);
	sqlLogin.loadAndCheckSettings();
 	connectByPass ( &sqlLogin );

	bool result = false;
  if ( sqlLogin.exec() == QDialog::Accepted )
		result =  true;
	EWidget::reloadSettings();
}
