/*
    Copyright (C) 2010 - 2012 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/

#include "edbsettingswidget.h"
#include "edbsettingswidget.moc"
#include "ui_edbsettingswidget.h"

#include "esqldatabase.h"

#include <QSettings>

EDBSettingsWidget::EDBSettingsWidget(QWidget *parent, Qt::WindowFlags flags) : ESettingsWidget(parent, flags),
		m_ui(new Ui::EDBSettingsWidget){

	myName = tr("Database Settings");
  m_ui->setupUi(this);
	connect ( m_ui->typeEdit, SIGNAL ( currentIndexChanged ( QString ) ), this, SLOT ( typeChanged ( QString ) ) );

	// database stuff
	m_ui->hostEdit->setText(host());
	setAvailableDrivers(sqlDrivers());
	m_ui->databaseEdit->setText(database());
	m_ui->sslEdit->setChecked( ssl() );
}

EDBSettingsWidget::~EDBSettingsWidget(){
    delete m_ui;
}

void EDBSettingsWidget::setAvailableDrivers(QStringList drivers)
{
	m_ui->typeEdit->clear();
	m_ui->typeEdit->addItems(drivers);
	m_ui->typeEdit->setCurrentIndex(m_ui->typeEdit->findText(driver()));
	typeChanged(m_ui->typeEdit->currentText());
}

QStringList EDBSettingsWidget::sqlDrivers()
{
	return ESqlDatabase::drivers();
}

QString EDBSettingsWidget::host()
{
	QSettings qSettings;
	qSettings.beginGroup("database");
	QString result = qSettings.value("host").toString();
	qSettings.endGroup();
	return result;
}

QString EDBSettingsWidget::driver()
{
	QSettings qSettings;
	qSettings.beginGroup("database");
	QString result = qSettings.value("driver").toString();
	qSettings.endGroup();
	return result;
}

QString EDBSettingsWidget::database()
{
	QSettings qSettings;
	qSettings.beginGroup("database");
	QString result = qSettings.value("database").toString();
	qSettings.endGroup();
	return result;
}

bool EDBSettingsWidget::ssl()
{
	QSettings qSettings;
	qSettings.beginGroup("database");
	bool result = qSettings.value( "ssl" ).toBool();
	qSettings.endGroup();
	return result;
}

bool EDBSettingsWidget::saveSettings(QString &errorString){
	if(m_ui->databaseEdit->text().contains(QRegExp("[;,\\s]+"))){
		errorString.append(tr("Database name contains not allowed strings"));
		return false;
	}

	QSettings qSettings;
	qSettings.beginGroup("database");
  qSettings.setValue("host", m_ui->hostEdit->text());
	qSettings.setValue("driver", m_ui->typeEdit->currentText());
  qSettings.setValue("database", m_ui->databaseEdit->text());
	qSettings.setValue ( "ssl", m_ui->sslEdit->isChecked() );
  qSettings.endGroup();

	return true;
}

void EDBSettingsWidget::typeChanged ( QString text )
{
	if ( text.contains ( "SQLITE" ) ) {
		m_ui->databaseLabel->setText ( tr ( "File:" ) );
		m_ui->hostEdit->setVisible ( false );
		m_ui->hostLabel->setVisible ( false );
		return;
	}
	m_ui->databaseLabel->setText ( tr ( "Database:" ) );
	m_ui->hostEdit->setVisible ( true );
	m_ui->hostLabel->setVisible ( true );
}
