/*
    Copyright (C) 2010 - 2012 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/

#include "edbupdatewidget.h"
#include "edbupdatewidget.moc"
#include "ui_edbupdatewidget.h"

#include "esqlinit.h"

EDBUpdateWidget::EDBUpdateWidget ( QString title,
																	 ESqlInit* newDBInit,
																	 ESetupWizardObject* newObject,
																	 QWidget* parent,
																	 Qt::WindowFlags flags )
    : ESetupWizardWidget ( newObject, parent, flags ), ui(new Ui::EDBUpdateWidget)
{
  ui->setupUi ( this );
  setName ( title );
  sqlInit = newDBInit;
}

EDBUpdateWidget::EDBUpdateWidget ( ESqlInit* newDBInit,
																	 ESetupWizardObject *newObject,
																	 QWidget *parent,
																	 Qt::WindowFlags flags )
    : ESetupWizardWidget ( newObject, parent, flags )
{
  ui->setupUi ( this );
  setName ( tr ( "Checking the database structure" ) );
	sqlInit = newDBInit;
}

EDBUpdateWidget::~EDBUpdateWidget()
{
	if(sqlInit)
		delete sqlInit;
}

EDBUpdateWidget::Needed EDBUpdateWidget::needed()
{
	if(!sqlInit)
		return Both;
	
	if ( sqlInit->login(true) == ESqlInit::LoggedIn ){
		if(sqlInit->login(false) == ESqlInit::NoUpdateNeeded)
			return Nothing;
		else
			return Update;
	}
  
  return Both;
}

bool EDBUpdateWidget::finish()
{
	if(!sqlInit)
		return false;
	
  connectByPass ( sqlInit );
  connect ( sqlInit, SIGNAL ( actualStep ( QString ) ), this, SLOT ( actualStep ( QString ) ) );
  connect ( sqlInit, SIGNAL ( finishedStep() ), this, SLOT ( finishedStep() ) );
  connect ( sqlInit, SIGNAL ( neededSteps ( int ) ), this, SLOT ( setRange ( int ) ) );
  if ( sqlInit->login ( false, true ) != ESqlInit::NoUpdateNeeded )
    return false;

  return true;
}

bool EDBUpdateWidget::saveSettings(QString& error)
{
	Q_UNUSED(error);
  return false;
}

void EDBUpdateWidget::errorByPass(const QString& string)
{
	log(string);
}

void EDBUpdateWidget::warningByPass(const QString& string)
{
	log(string);
}

void EDBUpdateWidget::informationByPass(const QString& string)
{
	log(string);
}

void EDBUpdateWidget::setRange ( int num )
{
  ui->progressBar->setRange ( 0, num );
}

void EDBUpdateWidget::finishedStep()
{
  ui->progressBar->setValue ( ui->progressBar->value() + 1 );
}

void EDBUpdateWidget::actualStep ( QString msg )
{
  ui->stepLabel->setText ( msg );
}

void EDBUpdateWidget::log ( QString msg )
{
  ui->logEdit->appendPlainText ( msg );
}
