/*
    Copyright (C) 2010 - 2012 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/

#include "edelegateforeignkey.h"
#include "edelegateforeignkey.moc"
#include "esqldatabase.h"

#include <QTimer>
#include <QComboBox>
#include <QLineEdit>
#include <QCompleter>
#include <QSettings>
#include <QAbstractItemView>
#include <QSqlRecord>
#include <QStyleOptionViewItemV2>
#include <QSqlQueryModel>
#include <QSqlQuery>

#include <QDebug>

EDelegateForeignKey::EDelegateForeignKey(QString table, QString matchColumn, QString resultColumn, QWidget *parent)
  : QStyledItemDelegate(parent)
{
	translator = NULL;
	init(table, matchColumn, resultColumn);
}

EDelegateForeignKey::EDelegateForeignKey(QString table, QString matchColumn, QString resultColumn, QString (*translator)(const QString), QWidget *parent)
{
	this->translator = translator;
	init(table, matchColumn, resultColumn);
}

void EDelegateForeignKey::setTranslator(QString (*translator)(const QString))
{
	this->translator = translator;
	reloadForeignKeys();
}

void EDelegateForeignKey::reloadForeignKeys()
{
	if(!resultModel)
		resultModel = new QSqlQueryModel();
	
  ESqlDatabase db = ESqlDatabase::database();
  bool closeDB = !db.isOpen();
	if(closeDB)
		db.open();
  if(db.isOpen()){
    resultModel->setQuery
    (
      QString("SELECT %1, %2 FROM %3 ORDER BY %2 LIMIT %4")
      .arg(matchCol)
      .arg(resCol)
      .arg(resTable)
			.arg(maxCompleteItems)
    );
    if(closeDB)
      db.close();
  }
  if(resultModel->rowCount() >= maxCompleteItems){
		useComboBox = false;
		useCompleter = false;
		useExtendedCompleter = true;
	}
	else{
		if(resultModel->rowCount() > maxComboItems){
			useComboBox = false;
			useCompleter = true;
			useExtendedCompleter = false;
		}
		if(resultModel->rowCount() < maxComboItems){
			useComboBox = true;
			useCompleter = false;
			useExtendedCompleter = false;
		}
		for(int i = 0; i < resultModel->rowCount(); i++){
			QString result = resultModel->record(i).value(resCol).toString();
			if(translator)
				result = translator(result);
			mapOutIn.insert
			(
				result,
				resultModel->record(i).value(matchCol).toUInt()
			);
		}
	}
}

void EDelegateForeignKey::paint(QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const{
  if(!index.data().canConvert(QVariant::String))
    // all other than booleans should be painted by the standard delegates
    return QStyledItemDelegate::paint(painter, option, index);
  
  unsigned int data = index.data().toUInt();

	QBrush brush = option.palette.base();
	QVariant modelBrush = index.data(Qt::BackgroundRole);
  if (modelBrush.canConvert<QBrush>()){
      brush = modelBrush.value<QBrush>();
	}
	else{
		const QStyleOptionViewItemV2 *style2 = qstyleoption_cast<const QStyleOptionViewItemV2*>(&option);
		if(style2)
			if(style2->features.testFlag(QStyleOptionViewItemV2::Alternate))
				brush = style2->palette.alternateBase();
	}
	QColor foregroundColor = option.palette.text().color();
	QVariant foregroundBrush = index.data(Qt::ForegroundRole);
	if (foregroundBrush.canConvert<QBrush>())
		foregroundColor = foregroundBrush.value<QBrush>().color();

	painter->save();
	painter->setFont(option.font);
	painter->setPen(foregroundColor);
	painter->fillRect(option.rect, brush);
	if(option.state & QStyle::State_Selected){
		// its selected
		painter->fillRect(option.rect, option.palette.highlight());
		painter->setPen(option.palette.highlightedText().color());
	}
	
  QTextOption textOptions;
  textOptions.setAlignment(Qt::AlignCenter);
  painter->drawText(option.rect, mapOutIn.key(data, QString()), textOptions);
	painter->restore();
}

QWidget *EDelegateForeignKey::createEditor(QWidget *parent, const QStyleOptionViewItem &option, const QModelIndex &index) const{
  // called from model if the user want to edit a field
//   if(!index.data().canConvert(QVariant::Bool)){
//     return QStyledItemDelegate::createEditor(parent, option, index);
//   }
  // the parent at QComboBox(parent) is important, otherwise the combobox comes up as a new window
//   unsigned int data = index.data().toUInt();
//   if(mapInOut.value(data, QString()).isEmpty())
//     return QStyledItemDelegate::createEditor(parent, option, index);
  if(useComboBox){
		QComboBox *editor = new QComboBox(parent);
		editor->addItems(mapOutIn.uniqueKeys());
		connect(editor, SIGNAL(currentIndexChanged(int)), this, SLOT(commitAndCloseEditor()));
		return editor;
	}
	else if(useCompleter){
		QLineEdit *editor = new QLineEdit(parent);
		QCompleter *completer = new QCompleter(mapOutIn.uniqueKeys(), editor);
		editor->setCompleter(completer);
		connect(editor, SIGNAL(editingFinished()), this, SLOT(commitAndCloseEditor()));
		return editor;
	}
	else{
		QLineEdit *editor = new QLineEdit(parent);
		connect(editor, SIGNAL(editingFinished()), this, SLOT(commitAndCloseEditor()));
		connect(editor, SIGNAL(textChanged(QString)), this, SLOT(updateCompleter()));
		return editor;
	}
}

void EDelegateForeignKey::setEditorData(QWidget *editor, const QModelIndex &index) const{
  // used to initialize the data of the editor
  // this is mainly used to get the editor showing the current data of the model
  unsigned int data = index.data().toUInt();
  if(mapOutIn.key(data, QString()).isEmpty())
    return QStyledItemDelegate::setEditorData(editor, index);
  
  QComboBox *box = qobject_cast<QComboBox *>(editor);
	if(box){    
		if(!box->findText(mapOutIn.key(data)))
			return QStyledItemDelegate::setEditorData(editor, index);
		box->setCurrentIndex(box->findText(mapOutIn.key(data)));
	}
	else {
		QLineEdit *edit = qobject_cast<QLineEdit*>(editor);
		if(!edit){
			return QStyledItemDelegate::setEditorData(editor, index);
		}
		edit->setText(mapOutIn.key(data));
	}
}

void EDelegateForeignKey::setModelData(QWidget *editor, QAbstractItemModel *model, const QModelIndex &index) const{
//  if(!index.data().canConvert(QVariant::String))
//    return QStyledItemDelegate::setModelData(editor, model, index);
  
  QComboBox *box = qobject_cast<QComboBox *>(editor);
	if(box)
		model->setData(index, mapOutIn.value(box->currentText()));
	else{
		QLineEdit *edit = qobject_cast<QLineEdit*>(editor);
		if(!edit)
			return QStyledItemDelegate::setModelData(editor, model, index);
		
		QCompleter *completer = edit->completer();
		if(!completer)
			return QStyledItemDelegate::setModelData(editor, model, index);
		
		if(mapOutIn.value(edit->text(), 0) != 0){
			model->setData(index, mapOutIn.value(edit->text()));
			return;
		}
		
// 		QModelIndex complIndex = completer->currentIndex();
// 		if(complIndex.isValid()){
// // 			emit addPair(resultModel->data(complIndex).toString(), resultModel->record(complIndex.row()).value(matchCol).toUInt());
// // 			mapOutIn.insert(resultModel->data(complIndex).toString(), resultModel->record(complIndex.row()).value(matchCol).toUInt());
// 			
// 			model->setData(index, mapOutIn.value(edit->text()));
// 		}
	}
}

void EDelegateForeignKey::commitAndCloseEditor(){
  // the user has changed data
  // so the editor can be closed (closeEditor()) and
  // the model is informed, that new data is present
  QComboBox *editor = qobject_cast<QComboBox *>(sender());
  if(editor){
		emit commitData(editor);
//     emit closeEditor(editor);
		timer->singleShot(1000, this, SLOT(doCloseEditor()));
  }
}

void EDelegateForeignKey::doCloseEditor()
{
	QComboBox *editor = qobject_cast<QComboBox *>(sender());
  if(editor)
		emit closeEditor(editor);
}

void EDelegateForeignKey::updateCompleter()
{
	QLineEdit *editor = qobject_cast<QLineEdit*>(sender());
	if(!editor)
		return;
	
	QCompleter *completer = editor->completer();
	if(!completer){
		completer = new QCompleter(editor);
		editor->setCompleter(completer);
	}
	
	QSqlQuery query;
	query.prepare(QString("SELECT %1, %2 FROM %3 WHERE %2 LIKE ?% ORDER BY %2 LIMIT %4")
								.arg(matchCol)
								.arg(resCol)
								.arg(resTable)
								.arg(maxCompleteItems));
	query.addBindValue(editor->text());
	resultModel->setQuery(query);
	completer->setModel(resultModel);
	completer->setModelSorting(QCompleter::CaseInsensitivelySortedModel);
	completer->setCompletionColumn(resultModel->record().indexOf(resCol));
}

void EDelegateForeignKey::init(QString table, QString matchColumn, QString resultColumn)
{
	maxComboItems = 25;
	maxCompleteItems = 99999;

	resultModel = 0;
	timer = new QTimer(this);
	resTable = table;
	matchCol = matchColumn;
	resCol = resultColumn;

// 	connect(this, SIGNAL(addPair(QString,uint)), this, SLOT(doAddPair(QString,uint)));

	reloadForeignKeys();
}
