/*
    Copyright (C) 2010 - 2012 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/

#include "edependenciesdialog.h"
#include "edependenciesdialog.moc"
#include "ui_edependenciesdialog.h"

#include "edependencymodel.h"
#include "eopenfileexternal.h"

#include <QMenu>

EDependenciesDialog::EDependenciesDialog(EDependencyModel *model, QWidget* parent, Qt::WindowFlags flags)
	: EDialog("eDependenciesDialog", parent, flags), ui(new Ui::EDependenciesDialog)
{
	ui->setupUi(this);
	setHelp("", ui->helpButton);
	connect(ui->closeButton, SIGNAL(clicked()), this, SLOT(close()));
	this->model = model;
	ui->view->setModel(model);
	ui->view->setContextMenuPolicy(Qt::CustomContextMenu);
	connect(ui->view, SIGNAL(customContextMenuRequested(QPoint)), this, SLOT(contextMenu(QPoint)));
	connect(ui->view, SIGNAL(doubleClicked(QModelIndex)), this, SLOT(doubleClicked(QModelIndex)));

	loadGeometry();
}

EDependenciesDialog::~EDependenciesDialog()
{
	close();
	delete ui;
	if(model)
		delete model;
}

void EDependenciesDialog::contextMenu(QPoint pos)
{
	bool showWebsite = false;
	curIndex = ui->view->indexAt(pos);
	if(curIndex.isValid()){
		QString website = model->data(curIndex).toString();
		if(curIndex.column() != 2)
			website = model->data(model->index(curIndex.row(), 2, curIndex.parent())).toString();
		if(!website.isEmpty())
			showWebsite = true;
	}
	
	QMenu menu;
	if(showWebsite)
		menu.addAction(tr("Open Website"), this, SLOT(openWebsite()));
	menu.addAction(tr("Expand all"), ui->view, SLOT(expandAll()));
	menu.addAction(tr("Collapse all"), ui->view, SLOT(collapseAll()));
	menu.exec(QCursor::pos());
}

void EDependenciesDialog::doubleClicked(QModelIndex index)
{
	curIndex = index;
	if(!index.isValid())
		return;
	if(index.column() != 2)
		return;
	openWebsite();
}

void EDependenciesDialog::openWebsite()
{
	if(!curIndex.isValid())
		return;
	if(curIndex.column() != 2)
		curIndex = model->index(curIndex.row(), 2, curIndex.parent());
	QString website = model->data(curIndex).toString();
	if(website.isEmpty())
		return;
	EOpenFileExternal::openFile(website, "", this);
}

