/*
    Copyright (C) 2010 - 2012 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/

#include "edependencyitem.h"

#include "exmlparseritem.h"

#include <QString>
#include <QStringList>

EXmlItem* EDependencyItem::create(EXmlModel *model)
{
	Q_UNUSED(model);
	return new EDependencyItem();
}

QVariant EDependencyItem::data(int column, int role) const
{
	if(role != Qt::DisplayRole)
		return QVariant();
	
	switch(column){
		case 0:
			return name();
		case 1:
			return version();
		case 2:
			return website();
		default:
			return QVariant();
	}
}

bool EDependencyItem::load(const QDomElement element, QMap< QString, EXmlParserItem* > parser)
{
	// basic checking, if given element is valid
	if(element.tagName().toLower() != tagName())
		return false;

	setName(element.attribute("name"));
	if(name().isEmpty())
		return false;

	// process child elements
	QDomElement child = element.firstChildElement();
	while(!child.isNull()){
		// processing the child elements, which are handled within this class
		if(child.tagName().toLower() == "version"){
			setVersion(child.text());
		}
		else if(child.tagName().toLower() == "website"){
			setWebsite(child.text());
		}
		else{
			// all other should be handled by others
			QStringList keys = parser.uniqueKeys();
			if(!keys.contains(child.tagName().toLower()))
				return false;

			if(!addChild(rowCount(), parser.value(child.tagName().toLower())->load(child, this, parser)))
				return false;
		}
		child = child.nextSiblingElement();
	}
	
	return true;
}

bool EDependencyItem::save(QDomElement& useElem)
{
	// we recieve segfaults, when using null elements
	// therefore we check for it
	if(useElem.isNull())
		return false;
	
	useElem.setTagName(tagName());
	useElem.setAttribute("name", name());
	QDomDocument doc = useElem.ownerDocument();
	// adding version
	if(!version().isEmpty()){
		QDomElement tmp = doc.createElement("version");
		tmp.appendChild(doc.createTextNode(version()));
		useElem.appendChild(tmp);
	}
	// adding website
	if(!website().isEmpty()){
		QDomElement tmp = doc.createElement("website");
		tmp.appendChild(doc.createTextNode(website()));
		useElem.appendChild(tmp);
	}
	
	return EXmlItem::save(useElem);
}

void EDependencyItem::setName(QString name)
{
	myName = name;
}

QString EDependencyItem::name() const
{
	return myName;
}

void EDependencyItem::setVersion(QString version)
{
	myVersion = version;
}

QString EDependencyItem::version() const
{
	return myVersion;
}

void EDependencyItem::setWebsite(QString url)
{
	myWebsite = url;
}

QString EDependencyItem::website() const
{
	return myWebsite;
}

EXmlItem* EDependencyItem::copy() const
{
	EDependencyItem *res = dynamic_cast<EDependencyItem*>(create(NULL));
	if(!res)
		return NULL;
	copyBase(res);
	res->setName(name());
	res->setVersion(version());
	res->setWebsite(website());
	return res;
}

