/*
    Copyright (C) 2010 - 2012 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/

#include "edialog.h"
#include "edialog.moc"

#include "epushbutton.h"

#include <QSettings>
#include <QApplication>

EDialog::EDialog ( QString objName, QWidget *parent, Qt::WindowFlags flags ) : QDialog ( parent, flags )
{
	setObjectName(objName);
	init();
}

EDialog::EDialog ( QWidget *parent, Qt::WindowFlags flags ) : QDialog ( parent, flags )
{
	init();
}

void EDialog::setWindowTitle ( const QString &title )
{
	if(QApplication::applicationName().isEmpty())
		QDialog::setWindowTitle(title);
	else
		QDialog::setWindowTitle ( QString("%1 - %2").arg(QApplication::applicationName()).arg(title) );
}

void EDialog::setObjectName(const QString& name)
{
	QDialog::setObjectName(name);
	loadGeometry();
}

void EDialog::loadGeometry()
{
	if(objectName().isEmpty())
		return;
	
	QSettings qSettings;
	qSettings.beginGroup("geometries");
	restoreGeometry(qSettings.value(objectName()).toByteArray());
	qSettings.endGroup();
}

void EDialog::setHelp ( QString page, EPushButton* helpButton )
{
	myHelpPage = page;
	if(helpButton){
		connect(helpButton, SIGNAL(clicked()), this, SLOT(help()));
		helpButton->setShortcut(QKeySequence(tr("ALT+H")));
		helpButton->setIcon("help-contents", ":/libengsas/help.png");
		helpButton->setText(tr("Help"));
	}
}

QString EDialog::helpPage()
{
	return myHelpPage;
}

void EDialog::closeEvent(QCloseEvent* event)
{
	if(!objectName().isEmpty()){
		QSettings qSettings;
		qSettings.beginGroup("geometries");
		qSettings.setValue(objectName(), saveGeometry());
		qSettings.endGroup();
	}
  QDialog::closeEvent(event);
}

void EDialog::init()
{
	setWindowIcon(QApplication::windowIcon());
	setWindowTitle(QString());
}
