/*
    Copyright (C) 2010 - 2012 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/

#include "efiledialog.h"
#include "efiledialog.moc"
#include "elibconfig.h"

#include "efiledialogprivate.h"

#ifdef BINDINGS_KDE
	#include <KFileDialog>
	#include <KUrl>
#else
	#include <QFileDialog>
#endif

#include <QSettings>

QString EFileDialog::getExistingDirectory(QWidget* parent, const QString& caption,
																					const QString& dir, QFileDialog::Options options)
{
	QString lastDir = dir;
	if(lastDir.isEmpty())
		lastDir = EFileDialogPrivate::getLastDirectory();
	
	QString result;
#ifdef BINDINGS_KDE
	Q_UNUSED(options);
	result = KFileDialog::getExistingDirectory(KUrl(lastDir), parent, caption);
#else
	result = QFileDialog::getExistingDirectory(parent, caption, lastDir, options);
#endif
	EFileDialogPrivate::saveLastDirectory(result);
	return result;
}

QString EFileDialog::getOpenFileName(QWidget* parent, const QString& caption,
																		 const QString& dir, const QString& filter,
																		 QString* selectedFilter, QFileDialog::Options options)
{
	QString lastDir = dir;
	if(lastDir.isEmpty())
		lastDir = EFileDialogPrivate::getLastDirectory();
	
	QString result;
	QString convertedFilter = EFileDialogPrivate::convertFilter(filter, selectedFilter);
#ifdef BINDINGS_KDE
	Q_UNUSED(options);
	result = KFileDialog::getOpenFileName(KUrl(lastDir), convertedFilter, parent, caption);
#else
	result = QFileDialog::getOpenFileName(parent, caption, lastDir, convertedFilter, selectedFilter, options);
#endif
	
	EFileDialogPrivate::saveLastDirectory(result);
	return result;
}

QStringList EFileDialog::getOpenFileNames(QWidget* parent, const QString& caption,
																					const QString& dir, const QString& filter,
																					QString* selectedFilter, QFileDialog::Options options)
{
	QString lastDir = dir;
	if(lastDir.isEmpty())
		lastDir = EFileDialogPrivate::getLastDirectory();
	
	QStringList results;
	QString convertedFilter = EFileDialogPrivate::convertFilter(filter, selectedFilter);
#ifdef BINDINGS_KDE
	Q_UNUSED(options);
	results = KFileDialog::getOpenFileNames(KUrl(lastDir), convertedFilter, parent, caption);
#else
	results = QFileDialog::getOpenFileNames(parent, caption, lastDir, convertedFilter, selectedFilter, options);
#endif
	
	if(results.size() > 0)
		EFileDialogPrivate::saveLastDirectory(results[0]);
	return results;
}

QString EFileDialog::getSaveFileName(QWidget* parent, const QString& caption,
																		 const QString& dir, const QString& filter,
																		 QString* selectedFilter, QFileDialog::Options options)
{
	QString lastDir = dir;
	if(lastDir.isEmpty())
		lastDir = EFileDialogPrivate::getLastDirectory();
	
	QString result;
	QString convertedFilter = EFileDialogPrivate::convertFilter(filter, selectedFilter);
#ifdef BINDINGS_KDE
	Q_UNUSED(options);
	result = KFileDialog::getSaveFileName(KUrl(lastDir), convertedFilter, parent, caption);
#else
	result = QFileDialog::getSaveFileName(parent, caption, lastDir, convertedFilter, selectedFilter, options);
#endif
	
	EFileDialogPrivate::saveLastDirectory(result);
	return result;
}
