/*
    Copyright (C) 2011 - 2012 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/


#include "efileservice.h"
#include "efileservice.moc"

#include "elibconfig.h"
#include "efileinfo.h"
#include "efileserviceplugin.h"
#include "epluginloader.h"
#include "efileservicesettings.h"
#include "esettingsmodelitem.h"

#include <QSettings>

EFileService::EFileService(QObject* parent)
	: EObject(parent)
{
	service = NULL;
	QSettings settings;
	QString serviceId = settings.value("fileServiceId").toString();
	service = loadService(serviceId);
}

EFileService::EFileService(QString serviceId, QObject* parent)
	: EObject(parent)
{
	service = NULL;
	service = loadService(serviceId);
}

QList< EFileServicePlugin* > EFileService::services()
{
	EngSaS::ModuleListWidgetItems protocol;
	QList<EFileServicePlugin*> plugins = EPluginLoader<EFileServicePlugin>::loadPlugins(
		QString( ENGSAS_FILE_SERVICE_PLUGIN_DIR ),
		protocol
	);
	return plugins;
}

ESettingsModelItem* EFileService::settings()
{
	EFileServiceSettings *settings = new EFileServiceSettings();
	return settings->toModelItem();
}

bool EFileService::serviceLoaded() const
{
	if(!service)
		return false;
	return true;
}

EFileInfo EFileService::info(QString uniqueId) const
{
	if(!serviceLoaded())
		return EFileInfo();
	return service->info(uniqueId);
}

EFileInfo EFileService::add(QFileInfo info, bool create) const
{
	if(!serviceLoaded())
		return EFileInfo();
	return service->add(info, create);
}

bool EFileService::move(EFileInfo source, QString target) const
{
	if(!serviceLoaded())
		return false;
	if(!source.isValid())
		return false;
	return service->move(source, target);
}

bool EFileService::copy(EFileInfo source, QString target) const
{
	if(!serviceLoaded())
		return false;
	if(!source.isValid())
		return false;
	return service->copy(source, target);
}

bool EFileService::deleteFile(EFileInfo file) const
{
	if(!serviceLoaded())
		return false;
	if(!file.isValid())
		return false;
	return service->deleteFile(file);
}

bool EFileService::open(EFileInfo file)
{
	if(!serviceLoaded())
		return false;
	if(!file.isValid())
		return false;
	return service->open(file);
}

bool EFileService::switchService(QString newServiceId)
{
	EFileServicePlugin *newService = loadService(newServiceId);
	
	return false;
}

EFileServicePlugin* EFileService::loadService(QString serviceId)
{
	if(serviceId.isEmpty())
		return NULL;
	
	QList<EFileServicePlugin*> plugins = services();
	foreach(EFileServicePlugin *plugin, plugins){
		if(plugin->serviceId() == serviceId)
			return plugin;
	}
	return NULL;
}
