#include "eheaderviewverticaltext.h"
#include "eheaderviewverticaltext.moc"

#include <QPainter>

EHeaderViewVerticalText::EHeaderViewVerticalText(Qt::Orientation orientation, QWidget *parent) :
	QHeaderView(orientation, parent)
{
}

QSize EHeaderViewVerticalText::sectionSizeFromContents(int logicalIndex) const
{
	if(logicalIndex < 0)
		return QHeaderView::sectionSizeFromContents(logicalIndex);
	QFontMetrics metrics(qvariant_cast<QFont>(model()->headerData(logicalIndex, orientation(), Qt::FontRole)));
	return QSize(40, metrics.width(model()->headerData(logicalIndex, orientation(), Qt::DisplayRole).toString())+20);
}

void EHeaderViewVerticalText::paintSection(QPainter *painter, const QRect &rect, int logicalIndex) const{
	if (!rect.isValid() || logicalIndex < 0)
		return QHeaderView::paintSection(painter, rect, logicalIndex);

	// get the state of the section
	QStyleOptionHeader opt;
	initStyleOption(&opt);
	opt.rect = rect;
	opt.section = logicalIndex;
	opt.sortIndicator = QStyleOptionHeader::None;

	QStyle::State state = QStyle::State_None;
	if (isEnabled())
			state |= QStyle::State_Enabled;
	if (window()->isActiveWindow())
			state |= QStyle::State_Active;
//	if (rect.contains(m_p))
//			state |= QStyle::State_MouseOver;
	if (sortIndicatorSection() == logicalIndex) {
		//state |= QStyle::State_Sunken;
		if (sortIndicatorOrder() == Qt::AscendingOrder) {
				opt.sortIndicator = QStyleOptionHeader::SortDown;
		} else {
				opt.sortIndicator = QStyleOptionHeader::SortUp;
		}
	}
	opt.state = state;

	// draw the section
	//reset the header text to no text
	opt.text = QString();
	//draw the control (unrotated!)
	style()->drawControl(QStyle::CE_HeaderSection, &opt, painter);

	QString headerText = model()->headerData(logicalIndex, orientation(), Qt::DisplayRole).toString();
	painter->save();
	painter->setPen(Qt::black);
	painter->setRenderHint(QPainter::Antialiasing);
	painter->translate(rect.x(), rect.bottom());
	painter->rotate(-90); // or 270
	QRect textRect = painter->matrix().inverted().mapRect(rect.adjusted(0, 0, 0, -10));
	painter->drawText(textRect, Qt::AlignLeft | Qt::AlignVCenter, headerText);
	painter->restore();
}
