/*
		Copyright (C) 2010 - 2013 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/

#include "elogobject.h"
#include "elogobject.moc"

#include "ewidget.h"
#include "elogobjectprivate.h"

#include <QMetaType>

Q_DECLARE_METATYPE(QtMsgType)

ELogObject *ELogObject::self = NULL;

ELogObject::ELogObject(bool installMessageHandler, QObject *parent)
	: EObject(parent)
{
	qRegisterMetaType<QtMsgType>();

	ELogObject::self = this;
	setObjectName("logObject");
	logger = new ELogObjectPrivate(this);
	connectByPass(logger);
	connect(this, SIGNAL(addMessage(QtMsgType,QString)), logger, SLOT(addMessage(QtMsgType,QString)));

	if(installMessageHandler)
		qInstallMsgHandler(messageHandler);
	installedAsMessageHandler = installMessageHandler;
}

ELogObject::~ELogObject()
{
	if(installedAsMessageHandler)
		qInstallMsgHandler(NULL);
	ELogObject::self = NULL;
}

void ELogObject::init(QString filePath)
{
	logger->init(filePath);
}

void ELogObject::setup(EObject *object)
{
	if(!object)
		return;

	logger->setup(object);
}

void ELogObject::setup(EWidget *object)
{
	if(!object)
		return;

	logger->setup(object);
}

void ELogObject::messageHandler(QtMsgType type, const char *msg)
{
	if(!msg)
		return;

	if(!self)
		return;

	QString message(msg);
	self->populateMessage(type, message);
}

void ELogObject::populateMessage(QtMsgType type, QString msg)
{
	emit addMessage(type, msg);
}
