/*
    Copyright (C) 2010 - 2012 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/

#include "emodulelistwidgetitem.h"

#include <QApplication>

EModuleListWidgetItem::EModuleListWidgetItem(QObject* plugin, QListWidget* parent, int type)
	: QListWidgetItem(parent, type)
{
	init();
	extractFromPlugin(plugin);
}

EModuleListWidgetItem::EModuleListWidgetItem ( QListWidget *parent, int type )
	: QListWidgetItem ( parent, type )
{
	init();
}

EModuleListWidgetItem::EModuleListWidgetItem ( QString text, QListWidget * parent, int type )
	: QListWidgetItem ( text, parent, type )
{
	init();
}

EModuleListWidgetItem::EModuleListWidgetItem ( QIcon icon, QString text, QListWidget *parent, int type )
	: QListWidgetItem ( icon, text, parent, type )
{
	init();
}

void EModuleListWidgetItem::setName ( QString name )
{
  setText ( name );
}

QString EModuleListWidgetItem::name() const
{
  return text();
}

void EModuleListWidgetItem::setFile ( QString file )
{
  myFile = QFileInfo ( file );
  if ( text().isEmpty() )
    setText ( myFile.baseName() );
}

QString EModuleListWidgetItem::file() const
{
  return myFile.absoluteFilePath();
}

void EModuleListWidgetItem::setStatus ( bool status )
{
	myStatus = status;
}

bool EModuleListWidgetItem::status() const
{
  return myStatus;
}

void EModuleListWidgetItem::setErrorMessage ( QString message )
{
  myErrorMessage = message;
}

QString EModuleListWidgetItem::errorMessage() const
{
  return myErrorMessage;
}

void EModuleListWidgetItem::setDescription ( QString description )
{
  myDescription = description;
}

QString EModuleListWidgetItem::description() const
{
  return myDescription;
}

void EModuleListWidgetItem::setModuleIcon ( QImage modIcon )
{
	myModuleIcon = modIcon;
}

QIcon EModuleListWidgetItem::moduleIcon() const
{
	return QIcon(QPixmap::fromImage(myModuleIcon));
}

void EModuleListWidgetItem::extractFromPlugin(QObject* plugin)
{
	if(!plugin)
		return;

	QVariant result = plugin->property("name");
	if(result.isValid())
		setName(result.toString());
	result = plugin->property("description");
	if(result.isValid())
		setDescription(result.toString());
	result = plugin->property("icon");
	if(result.canConvert<QImage>())
		setModuleIcon(result.value<QImage>());
}

QListWidgetItem* EModuleListWidgetItem::clone() const
{
	QListWidgetItem *result = QListWidgetItem::clone();
	if ( qApp && !myStatus )
    result->setIcon ( QIcon ( ":/icons/notok.png" ) );
  else
    result->setIcon ( QIcon ( ":/icons/ok.png" ) );

	return result;
}


bool EModuleListWidgetItem::operator==(const EModuleListWidgetItem& item) const
{
	if(!item.file().isEmpty() || file().isEmpty()){
		if(item.file() == file())
			return true;
	}
	
	return false;
}

bool EModuleListWidgetItem::operator!=(const EModuleListWidgetItem& item) const
{
	return !operator==(item);
}

void EModuleListWidgetItem::init()
{
  setName ( QString() );
  setFile ( QString() );
  setErrorMessage ( QString() );
  setDescription ( QString() );
  setStatus ( false );
}
