/*
    Copyright (C) 2010 - 2012 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/


#include "emodulestatedialog.h"
#include "emodulestatedialog.moc"
#include "ui_emodulestatedialog.h"

#include "emodulestatewidget.h"
#include <emodulelistwidgetitem.h>
#include <epushbutton.h>

EModuleStateDialog::EModuleStateDialog(QWidget* parent, Qt::WindowFlags flags)
	: EDialog("eModuleStateDialog", parent, flags), ui (new Ui::EModuleStateDialog)
{
	ui->setupUi(this);
	setHelp("keyword:Plugins", ui->helpButton);
	connect(ui->loadButton, SIGNAL(clicked()), this, SLOT(loadButton()));
	connect(ui->unloadButton, SIGNAL(clicked()), this, SLOT(unloadButton()));
	connect(ui->reloadAllButton, SIGNAL(clicked()), this, SLOT(reloadAllButton()));
	connect(ui->closeButton, SIGNAL(clicked()), this, SLOT(accept()));
	connect(ui->tabWidget, SIGNAL(currentChanged(int)), this, SLOT(tabChanged(int)));

	activeItem = 0;
}

EModuleStateWidget* EModuleStateDialog::addModuleState(EngSaS::ModuleListWidgetItems protocol,
																											 QString title, ActiveButtons enabledButtons)
{
	EModuleStateWidget *widget = new EModuleStateWidget(protocol);
	ui->tabWidget->addTab(widget, title);
	connectByPass(widget);
	connect(widget, SIGNAL(itemClicked(EModuleListWidgetItem*)), this, SLOT(itemClicked(EModuleListWidgetItem*)));
	activeButtons.insert(ui->tabWidget->count() - 1, enabledButtons);
	return widget;
}

void EModuleStateDialog::tabChanged(int index)
{
	Q_UNUSED(index);
	itemClicked(NULL);
}

void EModuleStateDialog::itemClicked(EModuleListWidgetItem* item)
{
	activeItem = item;

	ui->loadButton->setEnabled(false);
	ui->unloadButton->setEnabled(false);
	ui->reloadAllButton->setEnabled(false);
	if(!item)
		return;

	ActiveButtons buttons = activeButtons.value(ui->tabWidget->currentIndex());
	if(buttons.testFlag(Load) && buttons.testFlag(Unload))
		ui->reloadAllButton->setEnabled(true);
	if ( item->status() ){
		if (buttons.testFlag(Unload))
			ui->unloadButton->setEnabled ( true );
	}
	else{
		if(buttons.testFlag(Load))
			ui->loadButton->setEnabled ( true );
	}
}

void EModuleStateDialog::loadButton()
{
  if ( !activeItem )
    return;

  emit loadPlugin ( activeItem, true );
}

void EModuleStateDialog::unloadButton()
{
  if ( !activeItem )
    return;

  emit unload ( activeItem );
}

void EModuleStateDialog::reloadAllButton()
{
  emit reloadPlugins();
}
