/*
    Copyright (C) 2010 - 2012 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/


#include "emodulestatewidget.h"
#include "emodulestatewidget.moc"
#include "ui_emodulestatewidget.h"

#include "emodulelistwidgetitem.h"

EModuleStateWidget::EModuleStateWidget ( EngSaS::ModuleListWidgetItems protocol, QWidget* parent )
	: EWidget ( parent ), ui ( new Ui::EModuleStateWidget )
{
	ui->setupUi ( this );

	connect ( ui->moduleList, SIGNAL ( itemClicked ( QListWidgetItem* ) ), this, SLOT ( itemClickedByPass ( QListWidgetItem* ) ) );

	this->protocol = protocol;
	convertProtocol();
}

EModuleStateWidget::~EModuleStateWidget()
{
	delete ui;
	this->close();
}

void EModuleStateWidget::changeEvent ( QEvent *e )
{
	QWidget::changeEvent ( e );
	switch ( e->type() ) {
		case QEvent::LanguageChange:
			ui->retranslateUi ( this );
			break;
		default:
			break;
	}
}

void EModuleStateWidget::itemClickedByPass ( QListWidgetItem *item )
{
  // the dynamic cast isn't working, so we do a workaround
//  ModuleListWidgetItem *tmp = dynamic_cast<ModuleListWidgetItem *>(item);
//  if(!tmp){
//    ui->nameEdit->setText("test");
//    ui->fileEdit->setText("");
//    ui->descriptionEdit->setPlainText("");
//    ui->errorMessageEdit->setPlainText("");
//    return;
//  }

	foreach ( EModuleListWidgetItem *module, protocol ) {
		if ( module->name() == item->text() ) {
			ui->imageLabel->setPixmap(module->moduleIcon().pixmap(ui->imageLabel->size()));
			ui->nameEdit->setText ( module->name() );
			ui->fileEdit->setText ( module->file() );
			ui->descriptionEdit->setPlainText ( module->description() );
			ui->errorMessageEdit->setPlainText ( module->errorMessage() );
			emit itemClicked(module);
			return;
		}
	}
	ui->nameEdit->setText ( "" );
	ui->fileEdit->setText ( "" );
	ui->descriptionEdit->setPlainText ( "" );
	ui->errorMessageEdit->setPlainText ( "" );
	emit itemClicked(NULL);
}

void EModuleStateWidget::convertProtocol()
{
  ui->moduleList->clear();
  foreach ( EModuleListWidgetItem *module, protocol ) {
		QListWidgetItem *item = module->clone();
    ui->moduleList->addItem ( item );
  }
  
  // resize list
	if(ui->moduleList->count() < 1)
		return;
	
  int maxSize = 0;
  QFontMetrics fontMetric = ui->moduleList->fontMetrics();
  for(int i = 0; i < ui->moduleList->count(); i++){
    if(maxSize < fontMetric.width(ui->moduleList->item(i)->text())){
      maxSize = fontMetric.width(ui->moduleList->item(i)->text());
    }
  }
  
	maxSize += ui->moduleList->iconSize().width();
  maxSize += 50;
  ui->moduleList->setMinimumWidth(maxSize);
	ui->moduleList->setMaximumWidth(maxSize);
}
