/*
    Copyright (C) 2010 - 2012 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/

#include "eobject.h"
#include "eobject.moc"

#include <eaction.h>

#include <QSettings>
#include <QAction>
#include <QDockWidget>

EObject::~EObject()
{
	// delete added actions of managed dock widgets
	// (Those created within addManagedDockWidget())
	foreach(QPointer<QAction> dockAction, dockActions){
		dockActions.removeAll(dockAction);
		if(dockAction)
			delete dockAction;
	}
	// delete the managed dock widgets
	foreach(QPointer<QDockWidget> dockWidget, dockWidgets){
		dockWidgets.removeAll(dockWidget);
		if(dockWidget)
			delete dockWidget;
	}
}

void EObject::connectByPass ( QObject *parent, QObject *child )
{
	if(!parent || !child)
		return;
	
  connect ( child, SIGNAL ( error ( const QString& ) ), parent, SLOT ( errorByPass ( const QString& ) ) );
  connect ( child, SIGNAL ( warning ( const QString& ) ), parent, SLOT ( warningByPass ( const QString& ) ) );
  connect ( child, SIGNAL ( information ( const QString& ) ), parent, SLOT ( informationByPass ( const QString& ) ) );

	connect ( child, SIGNAL ( reloadEnableActions() ) , parent, SLOT ( reloadEnableActionsByPass () ) );
	connect ( child, SIGNAL ( addMenu ( QMenu*,QString,QString ) ), parent, SLOT ( addMenuByPass ( QMenu*,QString,QString ) ) );
  connect ( child, SIGNAL ( addAction ( QAction *, QString, QString ) ), parent, SLOT ( addActionByPass ( QAction*, QString, QString ) ) );
	connect ( child, SIGNAL ( addToolBar ( QToolBar *, Qt::ToolBarArea ) ), parent, SLOT ( addToolBarByPass ( QToolBar*, Qt::ToolBarArea ) ) );
  connect ( child, SIGNAL ( addNewDockWidget ( QDockWidget*, const Qt::DockWidgetArea& ) ), parent, SLOT ( addNewDockWidgetByPass ( QDockWidget*, const Qt::DockWidgetArea& ) ) );
  connect ( child, SIGNAL ( setCentralWidget ( QWidget* ) ), parent, SLOT ( setCentralWidgetByPass ( QWidget* ) ) );

  connect ( child, SIGNAL ( showSettings() ), parent, SLOT ( showSettingsByPass() ) );
  connect ( child, SIGNAL ( showHelp( QString ) ), parent, SLOT ( showHelpByPass( QString ) ) );
  connect ( child, SIGNAL ( showStatusWidget ( QWidget* ) ), parent, SLOT ( showStatusWidgetByPass ( QWidget* ) ) );

	connect ( child, SIGNAL ( reloadSettingsRequest() ), parent, SLOT ( reloadSettingsRequestByPass() ) );
	connect ( parent, SIGNAL ( doReloadSettings() ), child, SLOT ( reloadSettings() ) );
}

void EObject::addManagedDockWidget(QDockWidget *widget, const Qt::DockWidgetArea &area,
																	 QString actionLabel, QString actionParent)
{
	if(!widget)
		return;
	dockWidgets << widget;
	emit addNewDockWidget(widget, area);
	if(!actionLabel.isEmpty() && !actionParent.isEmpty()){
		EAction *action = new EAction(actionLabel);
		action->setCheckable(true);
		emit addAction(action, actionParent);
		connect(widget, SIGNAL(visibilityChanged(bool)), action, SLOT(setChecked(bool)));
		connect(action, SIGNAL(triggered(bool)), widget, SLOT(setVisible(bool)));
		dockActions << action;
	}
}
