/*
    Copyright (C) 2010 - 2012 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/

#include "elibconfig.h"

#include "eopenfileexternal.h"
#include "eopenfileexternal.moc"

#include "emimetypemodel.h"
#include "esearchexternalapp.h"
#include "emimetype.h"
#include "emimeprogram.h"
#include "esettings.h"

#include <QFileInfo>
#include <QStringList>
#include <QProcess>

#ifdef BINDINGS_KDE
	#include <KApplication>
	#include <KRun>
	#include <KUrl>
#endif

EOpenFileExternal::EOpenFileExternal(QObject *parent)
        : EObject(parent)
{
	model = 0;
	myType = 0;
	model = mimeTypes();
	isStream = false;
}

EOpenFileExternal::~EOpenFileExternal() {
	delete model;
}

EMimeTypeModel* EOpenFileExternal::mimeTypes() {
	EMimeTypeModel *myModel = new EMimeTypeModel();
	myModel->loadModel();
	return myModel;
}

QString EOpenFileExternal::group()
{
	return "EOpenFileExternal";
}

bool EOpenFileExternal::openFile(QString file, QString mimeType, QObject *parent)
{
	EOpenFileExternal openFile(parent);
	EObject::connectByPass(parent, &openFile);
	return openFile.openFile(file);
}

bool EOpenFileExternal::openFile(QFileInfo file, QString mimeType, QObject* parent)
{
	EOpenFileExternal openFile(parent);
	EObject::connectByPass(parent, &openFile);
	return openFile.openFile(file);
}

void EOpenFileExternal::setFile(QString file, QString mimeType) {
	myFile = file;
	myType = model->getMimeType(mimeType);

	if (myFile.contains("://")) {
			isStream = true;
	}
	else {
			myFile = QFileInfo(myFile).absoluteFilePath();
	}
}

void EOpenFileExternal::setFile(QFileInfo file, QString mimeType) {
	setFile(file.absoluteFilePath(), mimeType);
}

bool EOpenFileExternal::openFile(QString file, QString mimeType) {
	setFile(file, mimeType);
	return open();
}

bool EOpenFileExternal::openFile(QFileInfo file, QString mimeType) {
	setFile(file, mimeType);
	return open();
}

bool EOpenFileExternal::open() {
#ifdef BINDINGS_KDE
  // check wether main App is really a KApplication
	if(KApplication::kApplication()){
		new KRun(KUrl(myFile), 0);
		return true;
	}
#endif
	if (!isStream) {
			if (!QFileInfo(myFile).isReadable())
					return false;
	}

	// detect mime type
	QFileInfo app;
	if (!model)
			return false;
	EMimeType *mimeType = detectType();
	if (mimeType)
			if (mimeType->defaultProgram())
					app = mimeType->defaultProgram()->file();

	ESettings settings;
	settings.beginGroup(EOpenFileExternal::group());
	bool showAnytime = settings.value("showAnytime", false).toBool();
	settings.endGroup();

	// try to open file into app
	// first check wether app is executable
	if ((!app.isExecutable()) || (showAnytime)) {
			app.setFile(searchApp(mimeType));
			if (!app.isExecutable()) {
					emit information(tr("Don't have an executeable application: %1").arg(app.absoluteFilePath()));
					return false;
			}
	}
	// it's executable, so start it
	QStringList arguments;
	arguments.append(myFile);
	emit information(tr("Starting %1 %2").arg(app.absoluteFilePath()).arg(arguments.join(", ")));
	return QProcess::startDetached(app.absoluteFilePath(), arguments);
}

EMimeType *EOpenFileExternal::detectType() {
	if (!model)
		return false;

	if (myType)
		return myType;

	if (isStream) {
		// test prefix
		//   QString prefix = myFile.suffix().toLower();
		for (int i = 0; i < model->rowCount(); i++) {
			QModelIndex index = model->index(i, 2);
			QString prefix = model->data(index, Qt::DisplayRole).toString();
			if (prefix.isEmpty())
				continue;
			QRegExp reg("^("+ prefix +")");
			if (reg.indexIn(myFile) >= 0)
				return model->getMimeType(i);
		}
		return 0;
	}

	// test suffix
	QString suffix = QFileInfo(myFile).suffix().toLower();
	for (int i = 0; i < model->rowCount(); i++) {
		QModelIndex index = model->index(i, 1);
		if (model->data(index, Qt::DisplayRole).toString() == suffix) {
			return model->getMimeType(i);
		}
	}
	return 0;
}

QString EOpenFileExternal::searchApp(EMimeType* mimeType) {
	if (!model)
		return false;
	ESearchExternalApp *searchApp;
	searchApp = new ESearchExternalApp();
	QList<EMimeProgram*> progList;
	if (mimeType) {
		progList = mimeType->possiblePrograms();
	}
	connectByPass(searchApp);
	searchApp->init(progList);
	if (searchApp->exec() != QDialog::Accepted) {
		delete searchApp;
		return QString();
	}

	QString result = searchApp->selectedApp();
	bool setAsDefault = searchApp->setAsDefault();
	delete searchApp;
	if (!QFileInfo(result).isExecutable())
		return QString();

	if (setAsDefault) {
		if (mimeType) {
			QList<EMimeProgram*> apps = mimeType->possiblePrograms();
			bool found = false;
			foreach(EMimeProgram *app, apps){
				if(app->completePath() == result){
					found = true;
					mimeType->setDefaultProgram(app);
					break;
				}
			}
			if(!found)
				mimeType->setDefaultProgram(new EMimeProgram(result, mimeType));
			mimeType->saveMimeType();
		}
	}

	return result;
}
