/*
    Copyright (C) 2010 - 2012 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/

#include "epartmainwindow.h"
#include "epartmainwindow.moc"

#include "epart.h"
#include "eobject.h"
#include "elibrary.h"
#include "eaction.h"
#include "esettingsmodel.h"
#include "esettingsmodelitem.h"
#include "edependencymodel.h"
#include "edependencyitem.h"
#include "eaboutmodel.h"
#include "eaboutperson.h"

#include <QDir>
#include <QPluginLoader>
#include <QApplication>

EPartMainWindow::EPartMainWindow(QWidget* parent, Qt::WindowFlags flags)
	: EMainWindow(parent, flags)
{
	part = 0;
}

EPart::InitState EPartMainWindow::init(EPart* part, ECmdLineArgs *args)
{
	if(!part)
		return EPart::Failed;
	this->part = part;
  EObject::connectByPass(this, part);
	EPart::InitState result = part->init(args, true);
	if(result != EPart::Successful)
		return result;
	
	setObjectName("mw"+ part->shortName());
	setWindowIcon(QIcon(QPixmap::fromImage(part->icon())));
	// set the default window icon
	QApplication::setWindowIcon(QIcon(QPixmap::fromImage(part->icon())));
	setWindowTitle(part->name());
	setupHelpCenter(part->helpSetup());
	// disable settings button, if not needed
	ESettingsModelItem *item = part->settingsModelItem();
	if(!item)
		editSettingsAction->setVisible(false);
	else
		delete item;
	// disable dependencies button, if not needed
	EDependencyItem *dep = part->dependencies();
	if(!dep)
		helpDependenciesAction->setVisible(false);
	else
		delete dep;
	
	// Show ESetupWizard, if needed and do not inform
	// the user, if everything is alright
	EngSaS::SetupWizardObjects objects = part->setupWizardObjects();
	if(objects.size() < 1)
		editSetupWizardAction->setVisible(false);
	else
		showSetupWizard(objects);
	// connect this signal after the lines before,
	// so that emitted signals of this kind in init function
	// does not raise the setup wizard!
	connect(part, SIGNAL(showSetupWizard()), this, SLOT(editSetupWizard()));
	controleMenuStati();
	
	part->show();

	restoreWindowState();
	
	return result;
}

void EPartMainWindow::editSetupWizard()
{
	if(!part)
		return;
	
	showSetupWizard(part->setupWizardObjects(), true);
}

void EPartMainWindow::editSettings()
{
	ESettingsModelItem *item = part->settingsModelItem();
	if(!item)
		return;
	
	ESettingsModel *model = new ESettingsModel(this);
	model->connectByPass(this, model);
	model->addItem(item);
	showSettings(model);
}

void EPartMainWindow::helpDependencies()
{
	EDependencyModel *model = new EDependencyModel(this);
	model->connectByPass(this, model);
	model->setRootItem(part->dependencies());
	showDependencies(model);
}

void EPartMainWindow::helpAbout()
{
	EAboutModel *model = new EAboutModel(this);
	foreach(EAboutPerson *person, part->aboutPersons())
		model->addPerson(person);
		
	showAbout(QPixmap::fromImage(part->icon()).scaled(128, 128, Qt::KeepAspectRatio),
						part->name(), part->versionString(), part->website(), model);
}
