/*
    Copyright (C) 2010 - 2012 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/

#include "epushbutton.h"
#include "epushbutton.moc"

#include <QSettings>

EPushButton::EPushButton(QWidget* parent)
	: QPushButton(parent)
{
	init();
}

EPushButton::EPushButton(const QString& text, QWidget* parent)
	: QPushButton(parent)
{
	init(text);
}

EPushButton::EPushButton(QString standardIconName, QString iconPath, const QString& text, QWidget* parent)
	: QPushButton(parent)
{
	init(text);
	// must be used to strip out text
	setIcon(standardIconName, iconPath);
}

void EPushButton::setShowText(bool anytime)
{
	myShowText = anytime;
	setText(myText);
}

bool EPushButton::showText() const
{
	return myShowText;
}

void EPushButton::setIcon(const QIcon& icon)
{
	QPushButton::setIcon(icon);
	
	if(QPushButton::icon().pixmap(QPushButton::iconSize()).isNull())
		return;
	
	if(showText())
		return;
	
	QSettings settings;
	if(!settings.value("iconsAndText", false).toBool()){
		QPushButton::setText("");
		if(toolTip().isEmpty())
			setToolTip(myText);
	}
}

void EPushButton::setIcon(QString standardIconName, QString iconPath)
{
#if QT_VERSION < 0x040600
	setIcon(QIcon(iconPath));
#else
	setIcon(QIcon::fromTheme(standardIconName, QIcon(iconPath)));
#endif
}

void EPushButton::setText(const QString& text)
{
	myText = text;
	if(iconsAndText || icon().isNull() || showText())
		QPushButton::setText(myText);
	else
		QPushButton::setText("");
}

void EPushButton::init(QString text)
{
	QSettings qSettings;
  iconsAndText = qSettings.value ( "iconsAndText", false ).toBool();
	setText(text);
	setShowText(false);
}
