/*
    Copyright (C) 2010 - 2012 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/

#include "equerythread.h"
#include "equerythread.moc"

#include "equeryworker.h"

#include <QTime>
#include <QCoreApplication>
#include <QVariant>

EQueryThread::EQueryThread(QObject* parent) : QThread(parent)
{
	worker = NULL;
	finishedStartup = false;
// 	connect(this, SIGNAL(started()), this, SLOT(setStarted()));
	// since started() is emitted before run() is called, events between started() and the signal connections in run()
	// might not be queued (all events after the connections are queued until exec() is called
	connect(this, SIGNAL(allConnected()), this, SLOT(setStarted()));	
}

EQueryThread::~EQueryThread()
{
	if(worker)
		delete worker;
}

void EQueryThread::run()
{
	// create worker in new thread
	worker = new EQueryWorker();
	qRegisterMetaType< QList<QVariant> >("QList<QVariant>");
	qRegisterMetaType< QSqlRecord >("QSqlRecord");
	qRegisterMetaType<QVariant>("QVariant");
	connect(this, SIGNAL(addToQueue(QObject*, QVariant, QString, QString, QList<QVariant>)),
					worker, SLOT(addToQueue(QObject*, QVariant, QString, QString, QList<QVariant>)));
	connect(this, SIGNAL(configureDB(QString,QString,QString,QString,QString)),
					worker, SLOT(configureDB(QString,QString,QString,QString,QString)));
	connect(this, SIGNAL(cancel()), worker, SLOT(cancel()));
	connect(worker, SIGNAL(result(QObject*, QVariant, int, QSqlRecord)),
					this, SLOT(resultByPass(QObject*, QVariant, int, QSqlRecord)));
	connect(worker, SIGNAL(setRange(int,int)), this, SLOT(setRangeByPass(int,int)));
	connect(worker, SIGNAL(setValue(int)), this, SLOT(setValueByPass(int)));
	connect(worker, SIGNAL(finished(QObject*, QVariant, bool)), this, SLOT(finishedByPass(QObject*, QVariant, bool)));
	emit allConnected();
	
	// starting our own event loop (in our own thread)
	exec();
}

bool EQueryThread::isStarted()
{
	return finishedStartup;
}

bool EQueryThread::waitForStart()
{
	if(isStarted())
		return true;
	
	// wait for thread to be started
	QTime dieTime = QTime::currentTime().addSecs(2);
	// Userevents should be excluded see http://lists.trolltech.com/qt-interest/2007-01/thread00133-0.html
	while( QTime::currentTime() < dieTime ){
		QCoreApplication::processEvents(QEventLoop::ExcludeUserInputEvents, 100);
		if(isStarted())
			return true;
	}
	
	if(isStarted())
		return true;
	return false;
}

void EQueryThread::execQuery(QObject* client, QVariant requestId, QString queryString, QString errorMsg, QList< QVariant > bindValues)
{
	emit addToQueue(client, requestId, queryString, errorMsg, bindValues);
}

void EQueryThread::cancelQuery()
{
	if(isStarted())
		emit cancel();
}

void EQueryThread::dbSetup(QString dbname, QString username, QString password, QString host, QString driver)
{
	emit configureDB(dbname, username, password, host, driver);
}

void EQueryThread::resultByPass(QObject* client, QVariant requestId, int querySize, QSqlRecord record)
{
	emit result(client, requestId, querySize, record);
}

void EQueryThread::setRangeByPass(int min, int max)
{
	emit setRange(min, max);
}

void EQueryThread::setValueByPass(int value)
{
	emit setValue(value);
}

void EQueryThread::finishedByPass(QObject *client, QVariant requestId, bool status)
{
	emit finished(client, requestId, status);
}

void EQueryThread::setStarted()
{
	finishedStartup = true;
}
