/*
    Copyright (C) 2010 - 2012 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/


#include "esettingsdialog.h"
#include "esettingsdialog.moc"
#include "ui_esettingsdialog.h"

#include "esettingsmodel.h"
#include "esettingsmodelitem.h"
#include "epushbutton.h"
#include "esettingswidget.h"

#include <QHBoxLayout>
#include <QVBoxLayout>
#include <QSplitter>
#include <QTreeView>
#include <QStackedWidget>
#include <QLabel>
#include <QWidget>
#include <QSettings>

ESettingsDialog::ESettingsDialog (ESettingsModel *model, QWidget* parent, Qt::WindowFlags fl )
	: EDialog ( "eSettingsDialog", parent, fl ), m_ui ( new Ui::ESettingsDialog )
{
	m_ui->setupUi ( this );

	this->model = model;
	if(!model)
		return;
	model->connectByPass(this, model);
	connect(model, SIGNAL(reloadSettingsRequest()), this, SLOT(reloadSettingsRequestByPass()));
	connect(this, SIGNAL(doReloadSettings()), model, SLOT(reloadSettings()));
	if(model->rowCount() > 1 || model->hasChildren(model->index(0, 0))){
		m_ui->treeView->setModel(model);
		connect(m_ui->treeView, SIGNAL(clicked(QModelIndex)), this, SLOT(itemSelected(QModelIndex)));
	}
	else{
		ESettingsModelItem *item = model->itemFromIndex(model->index(0, 0));
		if(!item)
			return;
		ESettingsWidget *widget = item->widget();
		if(!widget)
			return;
		m_ui->stackedWidget->addWidget(widget);
		m_ui->stackedWidget->setCurrentWidget(widget);
		m_ui->treeView->setVisible(false);
	}

	setHelp("", m_ui->helpButton);
	connect(m_ui->cancelButton, SIGNAL(clicked()), this, SLOT(reject()));
	connect(m_ui->saveButton, SIGNAL(clicked()), this, SLOT(saveSettings()));
	connect(m_ui->saveAndQuitButton, SIGNAL(clicked()), this, SLOT(saveQuit()));

	loadGeometry();
	QSettings settings;
	settings.beginGroup ( "geometries" );
	m_ui->eSettingsDialogSplitter->restoreState (
		settings.value ( m_ui->eSettingsDialogSplitter->objectName() ).toByteArray()
	);
  settings.endGroup();
}

ESettingsDialog::~ESettingsDialog(){
	close();
	cancel();
	if(model)
		delete model;
	delete m_ui;
}

void ESettingsDialog::itemSelected(QModelIndex index)
{
	if(!model)
		return;
	ESettingsModelItem *item = model->itemFromIndex(index);
	if(!item)
		return;
	// leave empty QWidget (index 0) in stack
	if(m_ui->stackedWidget->count() > 1){
		m_ui->stackedWidget->setCurrentIndex(0);
		m_ui->stackedWidget->removeWidget(m_ui->stackedWidget->widget(1));
	}
	if(item->widget()){
		m_ui->stackedWidget->addWidget(item->widget());
		m_ui->stackedWidget->setCurrentIndex(1);
	}
}

int ESettingsDialog::exec()
{
	if(!model)
		reject();
	return EDialog::exec();
}

bool ESettingsDialog::saveSettings(){
  setError("");
	if(!model)
		return false;
	QString error;
  bool result = model->save(error);
	setError(error);
  return result;
}

void ESettingsDialog::saveQuit(){
  if(saveSettings())
    accept();
}

void ESettingsDialog::cancel(){
  reject();
}

void ESettingsDialog::setError(QString msg){
  m_ui->errorLabel->setText(msg);
}

void ESettingsDialog::closeEvent ( QCloseEvent* event )
{
  QSettings settings;
  settings.beginGroup ( "geometries" );
  settings.setValue (
		m_ui->eSettingsDialogSplitter->objectName(),
		m_ui->eSettingsDialogSplitter->saveState()
	);
  settings.endGroup();

  EDialog::closeEvent ( event );
}
