/*
    Copyright (C) 2010 - 2012 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/

#include "esettingsmodel.h"
#include "esettingsmodel.moc"

#include "esettingsmodelitem.h"

ESettingsModel::ESettingsModel(QObject* parent)
	: EAbstractItemModel(parent)
{
	Q_UNUSED(parent);
	root = new ESettingsModelItem("root", 0, 0);
	reloadRequested = false;
}

ESettingsModel::~ESettingsModel()
{
	if(root)
		delete root;
}

bool ESettingsModel::addItem(ESettingsModelItem* item)
{
	if(!item)
		return false;
	connectByPass(this, item);
	connect(item, SIGNAL(reloadSettingsRequest()), this, SLOT(reloadSettingsRequestByPass()));
	connect(this, SIGNAL(doReloadSettings()), item, SLOT(reloadSettings()));
	emit layoutAboutToBeChanged();
	item->setParent(root);
	root->addChild(item);
	emit layoutChanged();
	return true;
}

ESettingsModelItem* ESettingsModel::itemFromIndex(QModelIndex index) const
{
	if(!index.isValid())
		return 0;
	return static_cast<ESettingsModelItem*>(index.internalPointer());
}

bool ESettingsModel::save(QString& error)
{
	if(!root){
		error.append(tr("No pages given"));
		return false;
	}
	bool result = root->save(error);
	if(reloadRequested)
		emit reloadSettingsRequest();
	reloadRequested = false;
	return result;
}

QVariant ESettingsModel::data(const QModelIndex& index, int role) const
{
	if(role != Qt::DisplayRole)
		return QVariant();

	ESettingsModelItem *item = itemFromIndex(index);
	if(!item)
		return QVariant();

	switch(index.column()){
		case 0:
			return item->name();
	}
	return QVariant();
}

int ESettingsModel::columnCount(const QModelIndex& parent) const
{
	Q_UNUSED(parent);
	return 1;
}

int ESettingsModel::rowCount(const QModelIndex& parent) const
{
	// catch special case root item
	if(parent == QModelIndex()){
		if(!root)
			return 0;
		return root->rowCount();
	}
	ESettingsModelItem *item = itemFromIndex(parent);
	if(!item)
		return 0;
	return item->rowCount();
}

QModelIndex ESettingsModel::parent(const QModelIndex& child) const
{
	ESettingsModelItem *item = itemFromIndex(child);
	if(!item)
		return QModelIndex();
	ESettingsModelItem *parent = item->parent();
	if(!parent)
		return QModelIndex();
	if(parent == root)
		return QModelIndex();
	ESettingsModelItem *grandParent = parent->parent();
	if(!grandParent)
			return QModelIndex();

	int row = grandParent->childRow(parent);
	if(row < 0)
		return QModelIndex();
	return createIndex(row, 0, parent);
}

QModelIndex ESettingsModel::index(int row, int column, const QModelIndex& parent) const
{
  if ( ( row < 0 ) || ( column < 0 ) || (column >= columnCount()))
    return QModelIndex();

	ESettingsModelItem *parentItem = itemFromIndex ( parent );
  if ( !parentItem ){
		if(!root)
			return QModelIndex();
		parentItem = root;
	}

	if ( row < parentItem->rowCount() )
    return createIndex ( row, column, parentItem->child(row) );

  return QModelIndex();
}

QVariant ESettingsModel::headerData(int section, Qt::Orientation orientation, int role) const
{
	Q_UNUSED(section);
	Q_UNUSED(orientation);
	Q_UNUSED(role);
	return QVariant();
}
