/*
    Copyright (C) 2010 - 2012 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/

#include "esettingsmodelitem.h"
#include "esettingsmodelitem.moc"

#include "esettingswidget.h"

#include <qalgorithms.h>

ESettingsModelItem::ESettingsModelItem(QString name, ESettingsWidget* widget, ESettingsModelItem* parent)
	: EObject(parent)
{
	setParent(parent);
	setName(name);
	setWidget(widget);
	
}

ESettingsModelItem::ESettingsModelItem(ESettingsWidget* widget, ESettingsModelItem* parent)
{
	setParent(parent);
	setWidget(widget);
	setName(widget->name());
}

ESettingsModelItem::~ESettingsModelItem()
{
// 	qDeleteAll(myChilds);
	if(myWidget)
		delete myWidget;
}

void ESettingsModelItem::setName(QString name)
{
	myName = name;
}

QString ESettingsModelItem::name() const
{
	return myName;
}

void ESettingsModelItem::setWidget(ESettingsWidget* widget)
{
	connectByPass(widget);
	myWidget = widget;
}

ESettingsWidget* ESettingsModelItem::widget() const
{
	return myWidget;
}

void ESettingsModelItem::setParent(ESettingsModelItem* parent)
{
	myParent = parent;
}

ESettingsModelItem* ESettingsModelItem::parent() const
{
	return myParent;
}

void ESettingsModelItem::addChild(ESettingsModelItem* item)
{
	if(!item)
		return;

	// check if item is already a child
	if(childRow(item) > -1)
		return;
	
	item->setParent(this);
	connectByPass(item);
	myChilds << item;
}

void ESettingsModelItem::addChild(ESettingsWidget* widget)
{
	if(!widget)
		return;
	addChild(widget->toModelItem());
}

ESettingsModelItem* ESettingsModelItem::child(int row) const
{
	if(row < 0 || row >= rowCount())
		return 0;
	return myChilds[row];
}

int ESettingsModelItem::childRow(ESettingsModelItem* child) const
{
	int row = myChilds.indexOf(child);
	if(row < 0)
		return -1;
	return row;
}

int ESettingsModelItem::rowCount() const
{
	return myChilds.size();
}

bool ESettingsModelItem::save(QString& error)
{
	if(widget())
		if(!widget()->saveSettings(error))
			return false;

	foreach(ESettingsModelItem *item, myChilds){
		if(!item->save(error))
			return false;
	}
	return true;
}
