/*
    Copyright (C) 2010 - 2012 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/

#include "elibconfig.h"
#include "esqldatabase.h"

#include "esqllogin.h"

#include <QSqlQuery>
#include <QSqlRecord>
#include <QSqlField>
#include <QStringList>
#include <QSettings>

// #ifdef BINDINGS_KDE
// 	#include <KWallet/Wallet>
// #endif

ESqlDatabase::ESqlDatabase()
	: QSqlDatabase()
{
	init();
}

ESqlDatabase::ESqlDatabase(const QSqlDatabase& other)
	: QSqlDatabase(other)
{
	init();
}

ESqlDatabase::ESqlDatabase(const QString& useType)
	: QSqlDatabase(useType)
{

}

ESqlDatabase::~ESqlDatabase()
{
// #ifdef BINDINGS_KDE
// 	if(wallet)
// 		delete wallet;
// #endif
// 	QSqlDatabase::~QSqlDatabase();
}

// bool ESqlDatabase::open()
// {
// 	qDebug() << "ESqlOpen";
// 	if(!loadAndCheckSettings(QString(), QString(), QString(), QString(), QString()))
// 		return false;
// 	
// 	if(userName().isEmpty() || password().isEmpty()){
// 		SqlLogin login;
// 		login.setUsername(userName());
// 		if(login.exec() != QDialog::Accepted)
// 			return false;
// 		setUserName(login.username());
// 		setPassword(login.password());
// 	}
	
// // 	QString connName;
// // 	if ( !connName.isEmpty() ) {
// //     if ( connectionName() != connName ) {
// //       if ( connectionNames().contains ( connName ) ) {
// //         QSqlDatabase::removeDatabase ( connName );
// //       }
// // 
// //       db = ESqlDatabase::addDatabase ( driver, connName );
// //     }
// //   }
// //   else {
// //     if ( !db.isValid() )
// //       db = ESqlDatabase::addDatabase ( driver );
// //     }
// // 
// //     if ( db.driverName() != driver ) {
// //       QSqlDatabase::removeDatabase ( QLatin1String ( "defaultConnection" ) );
// //       db = QSqlDatabase::addDatabase ( driver );
// //     }
// //   }
// 	bool result = QSqlDatabase::open();
// 	if(!result){
// 		// try without database
// 		setDatabaseName("");
// 		result = QSqlDatabase::open();
// 	}
// 	
// #ifdef KDE
// 	if(result){
// 		if((!userName().isEmpty()) && (!password().isEmpty())){
// 			if(wallet){
// 				if(wallet->isOpen())
// 					wallet->writePassword(userName(), password());
// 			}
// 		}
// 	}
// #endif
// 	return result;
// }

QStringList ESqlDatabase::tables(QSql::TableType type) const
{
	QStringList result;
	
	if(databaseName().isEmpty())
		return result;
	
	switch(type){
		case QSql::Tables:
			result = tableList();
		break;
		case QSql::SystemTables:
			result = systemTableList();
			break;
		case QSql::Views:
			result = viewList();
			break;
		case QSql::AllTables:
			result = allList();
			break;
	}

	return result;
}

bool ESqlDatabase::loadAndCheckSettings(QString username, QString host, QString dbName,
																		QString newType, QString passwd)
{
  QSettings settings;
  settings.beginGroup ( "database" );
	if(!username.isEmpty()){
		setUserName(username);
#ifdef KDE
		// try to set password from kwallet the user
		if(wallet){
			if(wallet->isOpen()){
				wallet->readPassword(username, passwd);
				setPassword(passwd);
			}
		}
#endif
	}
	
	if(!passwd.isEmpty())
		setPassword(passwd);
	
	if(!host.isEmpty()){
		setHostName(host);
	}
	else{
		host = settings.value ( "host" ).toString();
		if(!host.isEmpty())
			setHostName(host);
	}
	if(!newType.isEmpty()){
		setType(newType);
	}
	else{
		newType = settings.value ( "type" ).toString();
		if(!newType.isEmpty())
			setType(newType);
	}
	if(!dbName.isEmpty()){
		setDatabaseName(dbName);
	}
	else{
		dbName = settings.value ( "database" ).toString();
		if(!dbName.isEmpty())
			setDatabaseName(dbName);
	}
  settings.endGroup();

  if ( ( type().contains ( "SQLITE" ) ) && ( !databaseName().isEmpty() ) ) {
    return true;
  }

  else if ( ( !hostName().isEmpty() ) || ( !type().isEmpty() ) || ( !databaseName().isEmpty() ) ) {
    return true;
  }

  return false;
}

bool ESqlDatabase::checkSettings()
{
  QSettings qSettings;
  qSettings.beginGroup ( "database" );
  QString myDriver = qSettings.value ( "type" ).toString();
  QString myHost = qSettings.value ( "host" ).toString();
  QString myDatabase = qSettings.value ( "database" ).toString();
  qSettings.endGroup();

  if ( ( myDriver.contains ( "SQLITE" ) ) && ( !myDatabase.isEmpty() ) ) {
    return true;
  }

  else if ( ( !myHost.isEmpty() ) || ( !myDriver.isEmpty() ) || ( !myDatabase.isEmpty() ) ) {
    return true;
  }

  return false;
}

ESqlDatabase ESqlDatabase::database(QString connectionName)
{
	return QSqlDatabase::database(connectionName, false);
}

void ESqlDatabase::setType(QString name)
{
	myType = name;
}

QString ESqlDatabase::type()
{
	return myType;
}

// ESqlDatabase ESqlDatabase::addDatabase(const QString& useType, const QString& connectionName)
// {
// 	ESqlDatabase db(useType);
// 	QSqlDatabasePrivate::addDatabase(db, connectionName);
// 	return db;
// }

QStringList ESqlDatabase::tableList() const
{
	QStringList result;
	
	ESqlDatabase db = ESqlDatabase::database(connectionName());
	bool closeDB = !db.isOpen();
	if(closeDB)
		db.open();
	QSqlQuery query(db);
	query.exec("SHOW FULL TABLES");
	while(query.next()){
		// only real tables are requested
		if(query.record().value(1).toString() != "BASE TABLE")
			continue;
		
		result << query.record().value(0).toString();
	}
	if(closeDB)
		db.close();
	return result;
}

QStringList ESqlDatabase::systemTableList() const
{
	QStringList result;
	result = QSqlDatabase::tables(QSql::SystemTables);
	return result;
}

QStringList ESqlDatabase::viewList() const
{
	QStringList result;
	
	ESqlDatabase db = ESqlDatabase::database(connectionName());
	bool closeDB = !db.isOpen();
	if(closeDB)
		db.open();
	QSqlQuery query(db);
	query.exec("SHOW FULL TABLES");
	while(query.next()){
		// only real tables are requested
		if(query.record().value(1).toString() != "VIEW")
			continue;
		
		result << query.record().value(0).toString();
	}
	if(closeDB)
		db.close();
	return result;
}

QStringList ESqlDatabase::allList() const
{
	QStringList result;
	result << tableList();
	result << systemTableList();
	result << viewList();
	return result;
}

void ESqlDatabase::init()
{
#ifdef KDE
	wallet = 0;
// 	wallet = KWallet::Wallet::openWallet(KWallet::Wallet::NetworkWallet(), 0);
// 	if(wallet){
// 		QSettings settings;
// 		QString myFolder = settings.value("appNameShort", "Meganizer").toString();
// 		if(!wallet->hasFolder(myFolder)){
// 			wallet->createFolder(myFolder);
// 		}
// 		if(wallet->setFolder(myFolder) != 0){
// 			wallet->closeWallet(myFolder, true);
// 		}
// 	}
#endif
}
