/*
    Copyright (C) 2010 - 2012 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/

#include "etableview.h"
#include "etableview.moc"

#include "edelegateforeignkey.h"

#include <QSettings>

ETableView::ETableView(QWidget* parent)
	: QTableView(parent)
{
	QSettings qSettings;
	setAlternatingRowColors(qSettings.value("alternatingRowColors").toBool());
}

void ETableView::setModel(QAbstractItemModel* model)
{
	deleteDelegates();
  QTableView::setModel(model);
}

void ETableView::reloadDelegates()
{
	QAbstractItemModel *mod = model();
	if(!mod)
		return;
	for(int i = 0; i < mod->columnCount(); i++){
		EDelegateForeignKey *delegate = qobject_cast<EDelegateForeignKey*>(itemDelegateForColumn(i));
		if(delegate){
			delegate->reloadForeignKeys();
		}
	}
}

void ETableView::deleteDelegates()
{
	// before we remove the model, we remove all delegates
	// and since delegates should not be shared,
	// we can delete them
	QAbstractItemModel *mod = model();
	if(!mod)
		return;
	
	for(int i = 0; i < mod->columnCount(); i++){
		QAbstractItemDelegate *delegate = itemDelegateForColumn(i);
		if(delegate){
			setItemDelegateForColumn(i, 0);
			delete delegate;
		}
	}
}

void ETableView::resizeAndHideColumns(QList<int> hideColumns, int expandingColumn)
{
	QAbstractItemModel *mod = model();
	if(!mod)
		return;
		
	// if no useful column index is given, we use the last column
	if(expandingColumn < 0 || expandingColumn >= mod->columnCount())
		expandingColumn = mod->columnCount() - 1;
		
	foreach(int index, hideColumns)
		hideColumn(index);
	
	int usedWidth = 0;
	for(int i = 0; i < mod->columnCount(); i++){
      if(!hideColumns.contains(i) && (i != expandingColumn)){
        resizeColumnToContents(i);
        usedWidth += columnWidth(i);
      }
    }
    // get width, which can be used for the title column
    if(usedWidth < width()){
      // some space left to grow title column
      setColumnWidth(expandingColumn, width() - usedWidth - 50);
    }	
}
