/*
    Copyright (C) 2010 - 2012 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/

#include "ewidget.h"
#include "ewidget.moc"

#include "eaction.h"

#include <QSettings>
#include <QApplication>
#include <QDockWidget>
#include <QAction>

EWidget::~EWidget()
{
	close();
	// delete added actions of managed dock widgets
	// (Those created within addManagedDockWidget())
	foreach(QPointer<QAction> dockAction, dockActions){
		dockActions.removeAll(dockAction);
		if(dockAction)
			delete dockAction;
	}
	// delete the managed dock widgets
	foreach(QPointer<QDockWidget> dockWidget, dockWidgets){
		dockWidgets.removeAll(dockWidget);
		if(dockWidget)
			delete dockWidget;
	}
}

void EWidget::setWindowTitle ( const QString &title )
{
  QWidget::setWindowTitle ( QString("%1 - %2").arg(QApplication::applicationName()).arg(title) );
}

QString EWidget::fileFilters(EngSaS::FileFilter reason)
{
	Q_UNUSED(reason);
	return QString();
}

bool EWidget::load(QFile* file, QString& error)
{
	Q_UNUSED(file);
	Q_UNUSED(error);
	return false;
}

bool EWidget::save(QFile* file, QString& error)
{
	Q_UNUSED(file);
	Q_UNUSED(error);
	return false;
}

bool EWidget::fileNew(QString& error)
{
	Q_UNUSED(error);
	return false;
}

bool EWidget::print(QString& error)
{
	Q_UNUSED(error);
	return false;
}

bool EWidget::undo(QString& error)
{
	Q_UNUSED(error);
	return false;
}

bool EWidget::redo(QString& error)
{
	Q_UNUSED(error);
	return false;
}

bool EWidget::cut(QString& error)
{
	Q_UNUSED(error);
	return false;
}

bool EWidget::copy(QString& error)
{
	Q_UNUSED(error);
	return false;
}

bool EWidget::paste(QString& error)
{
	Q_UNUSED(error);
	return false;
}

bool EWidget::reload(QString& error)
{
	Q_UNUSED(error);
	return false;
}

void EWidget::addManagedDockWidget(QDockWidget *widget, const Qt::DockWidgetArea &area,
																	 QString actionLabel, QString actionParent)
{
	if(!widget)
		return;
	dockWidgets << widget;
	emit addNewDockWidget(widget, area);
	if(!actionLabel.isEmpty() && !actionParent.isEmpty()){
		EAction *action = new EAction(actionLabel);
		action->setCheckable(true);
		emit addAction(action, actionParent);
		connect(widget, SIGNAL(visibilityChanged(bool)), action, SLOT(setChecked(bool)));
		connect(action, SIGNAL(triggered(bool)), widget, SLOT(setVisible(bool)));
		dockActions << action;
	}
}
