/*
    Copyright (C) 2010 - 2012 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/

#include "exmlitem.h"
#include "exmlitem.moc"

EXmlItem::EXmlItem(QString tagName, EXmlItem* parent) : QObject(NULL)
{
	setParent(parent);
	setTagName(tagName);
}

EXmlItem::~EXmlItem()
{
	qDeleteAll(childs);
}

bool EXmlItem::save(QDomElement &useElem)
{
	bool result = true;
	foreach(EXmlItem *item, childs){
		QDomElement tmpElem = useElem.ownerDocument().createElement("tmptag");
		if(!item->save(tmpElem) || tmpElem.isNull())
			result = false;
		useElem.appendChild(tmpElem);
	}
	return true;
}

int EXmlItem::rowCount() const
{
	return childs.size();
}

void EXmlItem::setParent(EXmlItem* parent)
{
	myParent = parent;
}

EXmlItem* EXmlItem::parent() const
{
	return myParent;
}

bool EXmlItem::addChild(int row, EXmlItem *item, bool emitLayoutChanges)
{
	if(!item)
		return false;
	if(emitLayoutChanges)
		emit layoutAboutToBeChanged();
	childs.insert(row, item);
	item->setParent(this);
	connect(item, SIGNAL(changed(EXmlItem*)), this, SLOT(changedByPass(EXmlItem*)));
	connect(item, SIGNAL(layoutAboutToBeChanged()), this, SLOT(layoutAboutToBeChangedByPass()));
	connect(item, SIGNAL(layoutChanged()), this, SLOT(layoutChangedByPass()));
	if(emitLayoutChanges)
		emit layoutChanged();
	return true;
}

bool EXmlItem::removeChild(int row)
{
	if(row < 0 || childs.size() <= row)
		return false;
	EXmlItem *item = childs[row];
	childs.removeAt(row);
	delete item;
	return true;
}

EXmlItem* EXmlItem::child(int row) const
{
	if(row < 0 || childs.size() <= row)
		return NULL;
	return childs[row];
}

int EXmlItem::row(EXmlItem* child) const
{
	int row = childs.indexOf(child); 
	if(row < 0)
		return -1;
	
	return row;
}

void EXmlItem::setTagName(QString tagName)
{
	myTagName = tagName.toLower();
}

QString EXmlItem::tagName() const
{
	return myTagName;
}

// QString EXmlItem::tagName(EXmlItem *(*callback)())
// {
// 	EXmlItem *item = callback();
// 	if(!item)
// 		return QString();
// 	QString res = item->tagName();
// 	delete item;
// 	return res;
// }

void EXmlItem::copyBase(EXmlItem* target) const
{
	if(!target)
		return;
	target->setTagName(tagName());
	
	// copy recursivly
	foreach(EXmlItem *child, childs)
		target->addChild(target->rowCount(), child->copy());
}
