/*
    Copyright (C) 2011 - 2012 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/


#include "efiledialogprivate.h"

#include <elibconfig.h>

#include <QSettings>
#include <QFileInfo>
#include <QDir>

EFileDialogPrivate::EFileDialogPrivate()
{

}

QString EFileDialogPrivate::getLastDirectory()
{
	QSettings settings;
	return settings.value("lastDir").toString();
}

void EFileDialogPrivate::saveLastDirectory(QString path)
{
	QFileInfo info(path);
	
	QSettings settings;
	settings.setValue("lastDir", info.absoluteDir().path());
}

QString EFileDialogPrivate::convertFilter(QString filter, QString *defaultFilter)
{
	// convert filter from Qt into KDE format if needed
#ifndef BINDINGS_KDE
	// we do not need to do anything with the filter
	Q_UNUSED(defaultFilter);
#else
	QStringList filters;
	filters = filter.split(";;");
	for(int i = 0; i < filters.size(); i++){
		bool isDefaultFilter = false;
		if(defaultFilter)
			if(filters[i] == defaultFilter)
				isDefaultFilter = true;
		int bracket = filters[i].indexOf("(");
		if(bracket < 0 || bracket > filters[i].size()){
			filters.removeAt(i);
		}
		else{
			QString name = filters[i].left(bracket).trimmed();
			bracket++;
			QString types = filters[i].mid(bracket, filters[i].length() - bracket - 1).trimmed();
			filters[i] = QString("%1|%2").arg(types).arg(name);
		}
		if(isDefaultFilter && defaultFilter){
			defaultFilter->clear();
			defaultFilter->append(filters[i]);
		}
	}
	filter = filters.join("\n");
#endif
	return filter;
}
