/*
    Copyright (C) 2011 - 2012 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/


#include "efileservicesettings.h"
#include "efileservicesettings.moc"

#include "efileserviceplugin.h"
#include "efileservice.h"

#include <QSettings>

EFileServiceSettings::EFileServiceSettings(QWidget* parent, Qt::WindowFlags flags)
	: ESettingsWidget(parent, flags)
{
	currentPlugin = NULL;
	
	setupUi(this);
	connect(fileServiceEdit, SIGNAL(currentIndexChanged(int)), this, SLOT(switchService()));
	
	QSettings settings;
	QString serviceId = settings.value("fileServiceId").toString();
	QList<EFileServicePlugin*> plugins = EFileService::services();
	foreach(EFileServicePlugin* plugin, plugins)
		mapServices.insert(plugin->name(), plugin);
	fileServiceEdit->addItems(mapServices.uniqueKeys());
	switchService();
}

bool EFileServiceSettings::saveSettings(QString& error)
{
	QSettings settings;
	if(!currentPlugin){
		settings.setValue("fileServiceId", "");
	}
	else{
		// maybe the service does not provide a ESettingsWidget,
		// therefore only try to save, if a widget is shown
		if(stackedWidget->count() > 1){
			ESettingsWidget *widget = qobject_cast<ESettingsWidget*>(stackedWidget->widget(1));
			if(!widget){
				error.append(tr("Could not store file service plugin settings."));
				return false;
			}
			if(!widget->saveSettings(error))
				return false;
		}
		settings.setValue("fileServiceId", currentPlugin->serviceId());
	}
	return true;
}

void EFileServiceSettings::switchService()
{
	ESettingsWidget *widget = NULL;
	if(stackedWidget->count() > 1){
		// cleanup last widget
		widget = qobject_cast<ESettingsWidget*>(stackedWidget->widget(1));
		if(!widget)
			return;
		QString errorString;
		if(!widget->saveSettings(errorString)){
			emit error(tr("Could not save service settings:\n%1").arg(errorString));
			return;
		}
		
		stackedWidget->setCurrentIndex(0);
		stackedWidget->removeWidget(widget);
		fileServiceBox->setTitle("");
	}
	
	currentPlugin = mapServices.value(fileServiceEdit->currentText());
	if(!currentPlugin)
		return;
	widget = currentPlugin->settingsWidget();
	if(widget){
		stackedWidget->addWidget(widget);
		stackedWidget->setCurrentIndex(1);
	}
	fileServiceBox->setTitle(currentPlugin->name());
}
