/*
		Copyright (C) 2013 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

		This library is free software; you can redistribute it and/or
		modify it under the terms of the GNU Lesser General Public
		License as published by the Free Software Foundation; either
		version 2.1 of the License, or (at your option) any later version.

		This library is distributed in the hope that it will be useful,
		but WITHOUT ANY WARRANTY; without even the implied warranty of
		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
		Lesser General Public License for more details.

		You should have received a copy of the GNU Lesser General Public
		License along with this library; if not, write to the Free Software
		Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
		MA 02110-1301  USA.
*/

#include "elogobjectprivate.h"
#include "elogobjectprivate.moc"

#include "ewidget.h"

#include <QDockWidget>
#include <QTextEdit>
#include <QTextStream>
#include <QMessageBox>
#include <QFile>

#include <iostream>

ELogObjectPrivate::ELogObjectPrivate(QObject *parent)
	: EObject(parent)
{
	logEdit = NULL;
}

ELogObjectPrivate::~ELogObjectPrivate()
{
	if(logEdit)
		delete logEdit;
	if(logFile.isOpen())
		logFile.close();
}

void ELogObjectPrivate::init(QString filePath)
{
	if(filePath.isEmpty())
		setupLogWidget();
	else{
		logFile.setFileName(filePath);
		if(!logFile.open(QFile::WriteOnly | QIODevice::Append)){
			setupLogWidget();
		}
	}
}

void ELogObjectPrivate::setup(EObject *object)
{
	if(!object)
		return;

	connect(object, SIGNAL(information(QString)), this, SLOT(setInformation(QString)));
	connect(object, SIGNAL(warning(QString)), this, SLOT(setWarning(QString)));
	connect(object, SIGNAL(error(QString)), this, SLOT(setError(QString)));
}

void ELogObjectPrivate::setup(EWidget *object)
{
	if(!object)
		return;

	connect(object, SIGNAL(information(QString)), this, SLOT(setInformation(QString)));
	connect(object, SIGNAL(warning(QString)), this, SLOT(setWarning(QString)));
	connect(object, SIGNAL(error(QString)), this, SLOT(setError(QString)));
}

void ELogObjectPrivate::addMessage(QtMsgType type, QString msg)
{
	QMutexLocker locker(&mutex);
	//in this function, you can write the message to any stream!
	switch (type) {
		case QtDebugMsg:
			appendMessage(msg, Qt::lightGray);
			break;
		case QtWarningMsg:
			setWarning(msg);
			break;
		case QtCriticalMsg:
			setError(msg);
			break;
		case QtFatalMsg:
			break;
	}
	if(type == QtFatalMsg){
		if(logFile.isOpen())
			appendMessage(msg, Qt::red);
		else{
			QTextStream stream(stderr);
			stream << msg;
			stream.flush();
#ifdef Q_OS_WIN32
			// on Windows, gui applications does not have a console,
			// therefore, the messagebox is additionally used here
			QMessageBox::critical(NULL, tr("Fatal error"), msg, QMessageBox::Ok);
#endif
		}
		abort();
	}
}

void ELogObjectPrivate::setInformation(QString msg)
{
	appendMessage(msg, Qt::black);
}

void ELogObjectPrivate::setWarning(QString msg)
{
	appendMessage(msg, Qt::darkYellow);
}

void ELogObjectPrivate::setError(QString msg)
{
	appendMessage(msg, Qt::red);
}

void ELogObjectPrivate::appendMessage(QString msg, QColor color)
{
	if(logFile.isOpen()){
		QTextStream stream(&logFile);
		stream << msg << endl;
		stream.flush();
	}
	else if(logEdit){
		logEdit->setTextColor(color);
		logEdit->append(msg.trimmed());
	}
}

void ELogObjectPrivate::setupLogWidget()
{
	logEdit = new QTextEdit();
	logEdit->setObjectName("logEdit");
	QDockWidget *dockWidget = new QDockWidget(tr("Log"));
	dockWidget->setObjectName("logDockWidget");
	dockWidget->setWidget(logEdit);
	addManagedDockWidget(dockWidget, Qt::BottomDockWidgetArea, tr("Show Log"), "viewMenu");
}
