/*
    Copyright (C) 2010 - 2012 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/

#ifndef EMIMEPROGRAM_H
#define EMIMEPROGRAM_H

#include "emimemodelitem.h"

class EMimeType;

#include <QDir>
#include <QString>
#include <QFileInfo>

/**
	* @internal
	* @brief An application, used by EMimeType.
	*/
class EMimeProgram : public EMimeModelItem{
	public:
		EMimeProgram(QString newFilePath = QString(), EMimeType *newParent = 0);

		EMimeType *parent() const;
		void setParent(EMimeType *newParent = 0);
		void setPath(QDir newPath);
		void setPath(QString newPath);
		/**
			* path to program
			* (e.g. /usr/bin is the path of /usr/bin/kate)
			* can be set via @see setPath(QDir newPath) and @see setPath(QString newPath)
			*/
		QDir path() const;
		/**
			* the filename of the program
			* (e.g. kate from /usr/bin/kate)
			*/
		void setProgram(QString newFile);
		/**
			* set's path and filename at once
			* (e.g. @p newFile = "/usr/bin/kate" for kate)
			*/
		void setProgram(QFileInfo newFile);
		QString program() const;
		/**
			* all informations about the program filename
			*/
		QFileInfo file() const;
		/**
			* set's path and filename at once
			* (e.g. @p newFilePath = "/usr/bin/kate" for kate)
			*/
		Q_DECL_DEPRECATED void setFilePath(QString newFilePath);
		/**
			* @brief Returns the complete path (e.g.: /usr/bin/kate).
			*/
		QString completePath() const;

		bool operator== (const EMimeProgram &other) const;
		bool operator!= (const EMimeProgram &other) const;

	private:
		QFileInfo myFile;
		EMimeType *myParent;
};

#endif // EMIMEPROGRAM_H
