/*
    Copyright (C) 2010 - 2012 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/

#ifndef EMIMETYPE_H
#define EMIMETYPE_H

#include "emimemodelitem.h"

class EMimeProgram;

#include <QString>
#include <QList>
#include <QFileInfo>

/**
	* @internal
	* @brief The representation of an MimeType, related applications and settings.
	*/
class EMimeType : public EMimeModelItem
{
  public:
    EMimeType ( QString newName = "" );
    ~EMimeType();

		inline static QString group(){ return "MimeTypes"; };
		
    void setName ( QString newName );
		/**
			* mime type name is for example application/pdf
			*/
    QString name() const;
    /**
      * suffix means *.suffix like bla/bla/bla/bli.pdf the suffix is pdf
			* suffix and prefix can not be set the same time, so this clears prefix
      */
    void setSuffix ( QString suffix );
    QString suffix() const;
    /**
      * prefix means something like protocol
			* clears suffix (@see setSuffix())
      */
    void setPrefix ( QString prefix );
    QString prefix() const;

    /**
      * sets the default application for this mimetype
			* @todo possiblePrograms().contains(Program*) isn't working!
      */
    void setDefaultProgram ( EMimeProgram *newProgram );
		void setDefaultProgram ( QString newProgram );
    EMimeProgram *defaultProgram() const;
    void setPossibleProgram ( QString possibleProgram );
    void setPossibleProgram ( QFileInfo possibleProgram );
    void setPossibleProgram ( EMimeProgram *possibleProgram );
    QList<EMimeProgram*> possiblePrograms() const;

    int rowCount() const;

    bool saveMimeType();
    bool loadMimeType ( QString newName = "" );

    bool operator== ( const EMimeType &compareType ) const;
    bool operator!= ( const EMimeType &compareType ) const;

  private:
    QString myName;
    QString mySuffix, myPrefix;
    EMimeProgram *myDefaultProgram;
    QList<EMimeProgram*> myPossiblePrograms;
};

#endif // EMIMETYPE_H
