/*
    Copyright (C) 2010 - 2012 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/

#include "equeryworker.h"
#include "equeryworker.moc"

#include "esqldatabase.h"

#include <QSqlError>
#include <QDateTime>
#include <QVariant>

EQueryWorker::EQueryWorker(QObject* parent): EObject(parent)
{
}

EQueryWorker::~EQueryWorker()
{
	if(!connectionName.isEmpty())
		ESqlDatabase::removeDatabase(connectionName);
}

void EQueryWorker::configureDB(QString dbname, QString username, QString password, QString host, QString driver)
{
	// make sure that really every thread has it's own db name
	connectionName = "QueryWorker-"+ QDateTime::currentDateTime().toString("yyyyMMddhhmmzzz");
	ESqlDatabase db = ESqlDatabase::addDatabase(driver, connectionName);
	db.setDatabaseName(dbname);
	db.setUserName(username);
	db.setPassword(password);
	db.setHostName(host);
}

void EQueryWorker::addToQueue(QObject* client, QVariant requestId, QString queryString, QString errorMsg, QList< QVariant > bindValues)
{
	ESqlDatabase db = ESqlDatabase::database(connectionName);
	abort = false;
	if(!db.open()){
		if(client){
			emit error(tr("Could not open the database connection.\nClient: %1\nRequestId: %2")
				.arg(client->objectName())
				.arg(requestId.toString())
			);
		}
		emit finished(client, requestId, false);
		db.close();
		return;
	}
	
	if(abort){
		emit finished(client, requestId, false);
		db.close();
		return;
	}
	
	// db is needed here, because otherwise the default connection would be used!
	QSqlQuery query(db);
	query.prepare(queryString);
	foreach(QVariant bind, bindValues)
		query.addBindValue(bind);
	
	if(!query.exec()){
		if(!errorMsg.isEmpty())
			emit error(errorMsg);
		emit finished(client, requestId, false);
		db.close();
		return;
	}
	
	if(abort){
		emit finished(client, requestId, false);
		db.close();
		return;
	}

	// if no data is present, the request was
	// successfull
	int max = query.size();
	if(max < 1){
		emit finished(client, requestId, true);
		db.close();
		return;
	}
	
	emit setRange(0, max);
	
	int i = 0;
	while(query.next()){
		if(abort){
			emit finished(client, requestId, false);
			db.close();
			return;
		}
		emit result(client, requestId, query.size(), query.record());
		i++;
		emit setValue(i);
	}
	
	db.close();
	emit finished(client, requestId, true);
}

void EQueryWorker::cancel()
{
	abort = true;
}
