/*
    Copyright (C) 2010 - 2012 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/

#ifndef EQUERYWORKER_H
#define EQUERYWORKER_H

#include "eobject.h"

#include <QVariant>
#include <QSqlQuery>
#include <QSqlRecord>

/**
	* @brief Executes the SQL-Statements given by the EQueryThread.
	*
	* This class is executed within EQueryThread::run() in a seperate thread.
	* Therefore all executed queries within this class are executed asychronously.
	*/
class EQueryWorker : public EObject
{
	Q_OBJECT
	public:
    EQueryWorker(QObject* parent = 0);
    ~EQueryWorker();
		
	public slots:
		void configureDB(QString dbname, QString username, QString password, QString host, QString driver);
		void addToQueue(QObject* client, QVariant requestId, QString queryString, QString errorMsg, QList< QVariant > bindValues);
		void cancel();

	private:
		bool abort;
		QString connectionName;
		
	signals:
		void result(QObject *client, QVariant requestId, int querySize, QSqlRecord record);
		void setRange(int min, int max);
		void setValue(int value);
		void finished(QObject *client, QVariant requestId, bool status);
};

#endif // EQUERYWORKER_H
