/*
    Copyright (C) 2010 - 2012 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/
#include "esearchexternalapp.h"
#include "esearchexternalapp.moc"
#include "ui_esearchexternalapp.h"

#include "emimeprogram.h"
#include "efiledialog.h"
#include "eopenfileexternal.h"
#include "esettings.h"

#include <QListWidgetItem>
#include <QFileInfo>
#include <QFileIconProvider>

#include <QDebug>

ESearchExternalApp::ESearchExternalApp ( QWidget *parent )
	: EDialog ( "eSearchExternalApp", parent ), m_ui ( new Ui::ESearchExternalApp )
{
	m_ui->setupUi ( this );

	connect ( m_ui->searchAppButton, SIGNAL ( clicked() ), this, SLOT ( searchApp() ) );
	connect ( m_ui->cancelButton, SIGNAL ( clicked() ), this, SLOT ( reject() ) );
	connect ( m_ui->acceptButton, SIGNAL ( clicked() ), this, SLOT ( accept() ) );
	connect ( m_ui->foundAppsView, SIGNAL ( currentTextChanged ( QString ) ), this, SLOT ( itemChanged ( QString ) ) );

	setHelp("keyword:External MimeTypes", m_ui->helpButton);
	loadGeometry();
}

ESearchExternalApp::~ESearchExternalApp()
{
	this->close();
	delete m_ui;
}

void ESearchExternalApp::init ( QList<EMimeProgram *>findAppsList )
{
	apps = findApps ( findAppsList );
	QFileIconProvider iconProvider;
	foreach ( EMimeProgram* app, apps ) {
		QListWidgetItem *item = new QListWidgetItem ( iconProvider.icon ( app->file() ), app->file().baseName(), m_ui->foundAppsView );
		m_ui->foundAppsView->addItem ( item );
	}
}

QString ESearchExternalApp::selectedApp()
{
	if ( ( curApp.isEmpty() ) && ( apps.size() > 0 ) )
		return apps[0]->file().absoluteFilePath();

	return curApp;
}

bool ESearchExternalApp::setAsDefault()
{
	return m_ui->defaultEdit->isChecked();
}

QList<QDir> ESearchExternalApp::defaultPaths()
{
	QList<QDir> paths;
#if defined(Q_OS_WIN32)
	paths.append ( QDir ( tr ( "c:/Program Files" ) ) );
	paths.append ( QDir ( tr ( "c:/Program Files (x86)" ) ) );
#else
	paths.append ( QDir ( "/usr/bin" ) );
	paths.append ( QDir ( "/bin" ) );
	paths.append ( QDir ( "/usr/local/bin" ) );
	paths.append ( QDir ( "/opt/bin" ) );
#endif
	return paths;
}

int ESearchExternalApp::exec()
{
	ESettings settings;
	settings.beginGroup ( EOpenFileExternal::group() );
	bool showAnytime = settings.value ( "showAnytime", false ).toBool();
	settings.endGroup();

	if ( ( !showAnytime ) && ( apps.size() == 1 ) ) {
		curApp = apps[0]->file().absoluteFilePath();
		return QDialog::Accepted;
	}

	return QDialog::exec();
}

void ESearchExternalApp::searchApp()
{
	QString dir;
	QList<QDir> dirs = defaultPaths();

	if ( dirs.size() > 0 )
		dir = dirs[0].absolutePath();

	QString app = EFileDialog::getOpenFileName ( this, tr ( "Search Applications" ), dir );

	if ( app.isEmpty() )
		return;

	EMimeProgram *application = new EMimeProgram ( app );

	apps.append ( application );

	QListWidgetItem *item = new QListWidgetItem ( QIcon(), application->file().baseName() );

	m_ui->foundAppsView->addItem ( item );

	m_ui->foundAppsView->setCurrentItem ( item );
}

void ESearchExternalApp::itemChanged ( QString currentText )
{
	foreach ( EMimeProgram *app, apps ) {
		if ( app->file().baseName() == currentText ) {
			curApp = app->file().absoluteFilePath();
			return;
		}
	}
	}

QList<EMimeProgram*> ESearchExternalApp::findApps ( QList<EMimeProgram*> appNames )
{
	QList<QDir> paths = defaultPaths();

	QList<EMimeProgram*> results;
	foreach ( EMimeProgram* app, appNames ) {
		if(app->file().isAbsolute()){
			results << app;
		}
		else{
			qDebug() << "Search paths used";
			foreach ( QDir path, paths ) {
				// extend the searchpaths with the path of the application
				path.setPath ( path.path() +"/"+ app->file().path() );
				QFileInfo info;
				info.setFile ( path , app->file().fileName() );

				if ( info.isExecutable() ) {
					app->setProgram(info);
					results << app;
				}
			}
		}
	}

	return results;
}
