/*
    Copyright (C) 2010 - 2012 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/

#ifndef ESEARCHEXTERNALAPP_H
#define ESEARCHEXTERNALAPP_H

#include "edialog.h"

class EMimeProgram;

#include <QDir>

namespace Ui
{
  class ESearchExternalApp;
}

/**
	* @internal
	* @brief A dialog, where the user can choose and add applications to open a specific mimetype.
	*
  * presents the user a dialog with a QListWidget where the user can choose
  * one of the apps from foundApps or he can add one manually
  */
class ESearchExternalApp : public EDialog
{
	Q_OBJECT
	public:
		/**
			*
			*/
		ESearchExternalApp ( QWidget *parent = 0 );
		~ESearchExternalApp();

		void init ( QList<EMimeProgram*> findAppsList );
		/**
			* @return the selected application
			*/
		QString selectedApp();
		/**
			* @return true if the user whishes to set the selectedApp() as the default app for this MimeType
			*/
		bool setAsDefault();
		/**
			* @return a List of directories where applicatios are searched
			*/
		static QList<QDir> defaultPaths();

	public slots:
		/**
			* if only one application is found, this one is used and the dialog is not shown at all
			* if more than one apps are found, the normal QDialog::exec() is called
			*/
		int exec();

	private slots:
		/**
			* opens the EFileDialog and let the user add a new application
			*/
		void searchApp();
		/**
			* sets the curApp to the selected one
			*/
		void itemChanged ( QString currentText );

	private:
		Ui::ESearchExternalApp *m_ui;
		QList<EMimeProgram*> apps;
		QString curApp;

		/**
			* searches all application's with name of appNames at standard locations
			* @param appNames a QStringList with names of applications,
			* @param appNames which should be looked for at the places
			*/
		QList<EMimeProgram*> findApps ( QList<EMimeProgram*> appNames );
};

#endif // ESEARCHEXTERNALAPP_H
