/*
		Copyright (C) 2010 - 2013 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/

#include "esqllogindialog.h"
#include "esqllogindialog.moc"
#include "ui_esqllogindialog.h"

ESqlLoginDialog::ESqlLoginDialog ( QWidget *parent )
	: EDialog ( "eSqlLogin", parent ), m_ui ( new Ui::ESqlLoginDialog )
{
	init();
}

ESqlLoginDialog::ESqlLoginDialog ( QString title, QWidget *parent )
	: EDialog ( "eSqlLogin", parent ), m_ui ( new Ui::ESqlLoginDialog )
{
	init();
	m_ui->connectionDetailsLabel->setText( title );
}

ESqlLoginDialog::~ESqlLoginDialog()
{
  close();
  delete m_ui;
}

void ESqlLoginDialog::setUsername(QString name)
{
	m_ui->usernameEdit->setText(name);
}

QString ESqlLoginDialog::username() const
{
	return m_ui->usernameEdit->text();
}

void ESqlLoginDialog::setPassword(QString password)
{
	m_ui->passwordEdit->setText(password);
}

QString ESqlLoginDialog::password() const
{
	return m_ui->passwordEdit->text();
}

void ESqlLoginDialog::displayError ( QString msg )
{
  m_ui->statusLabel->setText ( QString ( "<font color='red'>%1</font>" ).arg ( msg ) );
}

void ESqlLoginDialog::displayInformation ( QString msg )
{
  m_ui->statusLabel->setText ( QString ( "<font color='Green'>%1</font>" ).arg ( msg ) );
}

void ESqlLoginDialog::changeEvent ( QEvent *e )
{
	QDialog::changeEvent ( e );

	switch ( e->type() ) {
		case QEvent::LanguageChange:
			m_ui->retranslateUi ( this );
			break;
		default:
			break;
	}
}

void ESqlLoginDialog::usernameChanged()
{
	m_ui->passwordEdit->setText("");
#ifdef BINDINGS_KDE
	// try to get password for user
	if(!wallet)
		return;
	if(!wallet->isOpen())
		return;
	QMap<QString, QString> list;
	wallet->readPasswordList("*", list);
	foreach(QString key, list.uniqueKeys()){
		if(key == username()){
			password = list.value(key);
			m_ui->passwordEdit->setText(password);
		}
	}
#endif
}

void ESqlLoginDialog::init()
{
	setWindowModality ( Qt::WindowModal );
	m_ui->setupUi ( this );
	loadGeometry();

#ifdef BINDINGS_KDE
	wallet = KWallet::Wallet::openWallet(KWallet::Wallet::NetworkWallet(), winId());
#endif
	
	connect ( m_ui->loginButton, SIGNAL ( clicked() ), this, SLOT ( accept() ) );
	connect ( m_ui->cancelButton, SIGNAL ( clicked() ), this, SLOT ( reject() ) );
	connect ( m_ui->settingsButton, SIGNAL ( clicked() ), this, SLOT ( showSettingsByPass()) );
	connect ( m_ui->usernameEdit, SIGNAL (editingFinished()), this, SLOT ( usernameChanged()));
}
