/*
    Copyright (C) 2010 - 2012 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/

#include "version.h"

#include <QRegExp>

Version::Version()
{
	setMajor(0);
	setMinor(0);
	setPatch(0);
	setRc(0);
}

Version::Version(QString version)
{
	Version ver = Version::fromString(version);
	setMajor(ver.maj());
	setMinor(ver.min());
	setPatch(ver.patch());
	setRc(ver.rc());
}

Version::Version(int major, int minor, int patch, int rc)
{
	setMajor(major);
	setMinor(minor);
	setPatch(patch);
	setRc(rc);
}

bool Version::isValid() const
{
	if(maj() < 0 || min() < 0 || patch() < 0 || rc() < 0)
		return false;
	if(maj() > 0 || min() > 0 || patch() > 0)
		return true;
	return false;
}

QString Version::toString() const
{
	if(rc() > 0)
		return QString("%1.%2.%3-rc%4").arg(maj()).arg(min()).arg(patch()).arg(rc());
	else
		return QString("%1.%2.%3").arg(maj()).arg(min()).arg(patch());
}

Version Version::fromString(QString versionString)
{
	QRegExp regVer("([0-9]+)\\.([0-9]+)\\.([0-9]+)");
	if(regVer.indexIn(versionString) == -1)
		return Version();
	
	Version result;
	result.setMajor(regVer.cap(1).toInt());
	result.setMinor(regVer.cap(2).toInt());
	result.setPatch(regVer.cap(3).toInt());
	result.setRc(0);
	QRegExp regRc("-rc([0-9]+)");
	if(regRc.indexIn(versionString) > -1)
		result.setRc(regRc.cap(1).toInt());
	return result;
}

void Version::setMajor(int value)
{
	myMajor = value;
}

int Version::maj() const
{
	return myMajor;
}

void Version::setMinor(int value)
{
	myMinor = value;
}

int Version::min() const
{
	return myMinor;
}

void Version::setPatch(int value)
{
	myPatch = value;
}

int Version::patch() const
{
	return myPatch;
}

void Version::setRc(int value)
{
	myRc = value;
}

int Version::rc() const
{
	return myRc;
}

bool Version::operator==(const Version &rhs) const
{
	if(maj() != rhs.maj())
		return false;
	
	if(min() != rhs.min())
		return false;
	
	if(patch() != rhs.patch())
		return false;
	
	if(rc() != rhs.rc())
		return false;
	
	return true;
}

bool Version::operator!=(const Version &rhs) const
{
	return !operator==(rhs);
}

bool Version::operator<(const Version &rhs) const
{
	if(maj() < rhs.maj())
		return true;
	else if(maj() > rhs.maj())
		return false;
	
	if(min() < rhs.min())
		return true;
	else if(min() > rhs.min())
		return false;
	
	if(patch() < rhs.patch())
		return true;
	else if(patch() > rhs.patch())
		return false;
	
	// the test rc() == 0 must be done,
	// because of e.g.:
	// 0.3.2-rc2 is smaller than 0.3.2-rc0
	if(rc() < rhs.rc() && rc() > 0)
			return true;
	
	return false;
}

bool Version::operator<=(const Version &rhs) const
{
	if(operator<(rhs) || operator==(rhs))
		return true;
	return false;
}

bool Version::operator>(const Version &rhs) const
{
	return !operator<=(rhs);
}

bool Version::operator>=(const Version &rhs) const
{
	return !operator<(rhs);
}
