/*
    Copyright (C) 2010 - 2012 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/

#ifndef VERSION_H
#define VERSION_H

#include <QString>

/**
	* @brief Represents the version from an application.
	*
	* It can handle strings like "0.2.1-rc2" or "0.3.2".
	*/
class Version {
	public:
		Version();
		Version(QString version);
		Version(int major, int minor, int patch, int rc);
		bool isValid() const;
		QString toString() const;
		static Version fromString(QString versionString);
		
		void setMajor(int value);
		/**
			* @brief Returns the major version.
			*
			* Not named major, because conflicts with gnu_dev_major
			*/
		int maj() const;
		void setMinor(int value);
		/**
			* @brief Returns the minor version.
			*
			* Not named minor, because conflicts with gnu_dev_minor.
			*/
		int min() const;
		void setPatch(int value);
		int patch() const;
		void setRc(int value);
		int rc() const;
		
		bool operator== 	(const Version &rhs) const;
		bool operator!= 	(const Version &rhs) const;
		bool operator< 		(const Version &rhs) const;
		bool operator<= 	(const Version &rhs) const;
		bool operator> 		(const Version &rhs) const;
		bool operator>= 	(const Version &rhs) const;
		
	private:
		int myMajor, myMinor, myPatch, myRc;
};

#endif // VERSION_H
