/*
    Copyright (C) 2010 - 2012 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/

#include "eclosemodalwidgetthread.h"
#include "eclosemodalwidgetthread.moc"

#include "ecloser.h"

#include <QApplication>
#include <QTimer>
#include <QTime>
#include <QTest>
#include <QKeyEvent>

ECloseModalWidgetThread::ECloseModalWidgetThread(uint timeToCloseEvent)
{
	closer = NULL;
	finishedStartup = false;
	this->timeToCloseEvent = timeToCloseEvent;
	connect(this, SIGNAL(started()), this, SLOT(setStarted()));
}

ECloseModalWidgetThread::~ECloseModalWidgetThread()
{
	if(closer)
		delete closer;
}

void ECloseModalWidgetThread::stopThread()
{
	// stopping thread
	// cancel query first
	// stop eventloop
	quit();
	// wait 2000 seconds to stop the eventloop
	wait(2000);
}

void ECloseModalWidgetThread::run()
{
	closer = new ECloser();
	QTimer::singleShot(timeToCloseEvent, closer, SLOT(closeModalWidget()));
	exec();
}

bool ECloseModalWidgetThread::isStarted()
{
	return finishedStartup;
}

bool ECloseModalWidgetThread::waitForStart()
{
	if(isStarted())
		return true;

	// wait for thread to be started
	QTime dieTime = QTime::currentTime().addSecs(2);
	// Userevents should be excluded see http://lists.trolltech.com/qt-interest/2007-01/thread00133-0.html
	while( QTime::currentTime() < dieTime ){
		QCoreApplication::processEvents(QEventLoop::ExcludeUserInputEvents, 100);
		if(isStarted())
			return true;
	}

	if(isStarted())
		return true;
	return false;
}

void ECloseModalWidgetThread::setStarted()
{
	finishedStartup = true;
}
