/*
    Copyright (C) 2010 - 2012 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/

#include "testdesignerplugins.h"

#include <epluginloader.h>
#include <emodulelistwidgetitem.h>

#include <QTest>
#include <QDesignerCustomWidgetCollectionInterface>

void TestDesignerPlugins::testPlugins(QString dir)
{
	//load and test plugin
	EngSaS::ModuleListWidgetItems log;
	QList<QDesignerCustomWidgetCollectionInterface*> plugins = EPluginLoader<QDesignerCustomWidgetCollectionInterface>::loadPlugins(dir, log);
	if(plugins.size() == 0)
		QSKIP("Could not find designer plugin(s) at "+ QDir(dir).absolutePath().toLatin1(), SkipAll);

	foreach(QDesignerCustomWidgetCollectionInterface* plugin, plugins){
		QList<QDesignerCustomWidgetInterface*> widgets = plugin->customWidgets();
		foreach(QDesignerCustomWidgetInterface* widget, widgets)
			testPlugin(widget);
	}
	qDeleteAll(plugins);
}

void TestDesignerPlugins::testPlugin(QDesignerCustomWidgetInterface* plugin)
{
	QVERIFY(plugin);
	plugin->initialize(0);
	QVERIFY(plugin->isInitialized());
	QVERIFY(!plugin->name().isEmpty());
	QVERIFY(!plugin->group().isEmpty());
	QVERIFY(!plugin->toolTip().isEmpty());
	QVERIFY(!plugin->whatsThis().isEmpty());
	QVERIFY(!plugin->includeFile().isEmpty());
// 	QVERIFY(!plugin->icon().isNull());
	QVERIFY(!plugin->domXml().isEmpty());

	QWidget *widget = plugin->createWidget(0);
	QVERIFY(widget);
	delete widget;

	// test that the widget name is the same as the class name, but with other case
	QString xml = plugin->domXml();
	QRegExp regex("widget class=\"([^\"]+)\" name=\"([^\"]+)\"");
	QVERIFY(regex.indexIn(xml) > -1);
	QStringList results = regex.capturedTexts();
	QCOMPARE(results.size(), 3);
	QCOMPARE(results[1].toUpper(), results[2].toUpper());
	if(results[1] == results[2])
		QTest::qWarn("Plugin "+ plugin->name().toAscii() +" failed: "+ results[1].toAscii() +" == "+ results[2].toAscii());
	QVERIFY(results[1] != results[2]);
}
