/*
    Copyright (C) 2010 - 2012 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/

#include "testeparts.h"
#include "testeparts.moc"

#include <epart.h>
#include <ecmdlineoption.h>

class EAboutPerson;

#include <QTest>
#include <QResource>

void TestEParts::testEParts(EPart* theEPart, QStringList resourceFiles)
{
  QVERIFY(theEPart);

	// loading resources
	foreach(QString resource, resourceFiles)
		QVERIFY2(QResource::registerResource(resource), resource.toAscii());
	
  // testing basic plugin features
	testPlugin(theEPart);

  // version provided (svtl. mit svn)
  QString version = theEPart->versionString();
  QVERIFY(version != "");

  // check for version containing RCS revision
  QRegExp regex = QRegExp("[\\d]+\\.[\\d]+\\.[\\d]+(-[\\w]+)?");
  // QVERIFY(version);

  EngSaS::CmdLineOptions options = theEPart->cmdLineArgs();
  EngSaS::CmdLineOptions failing;
  // check for command line conflicts, e.g. epart using -h option
  for(int i=0; i < options.count(); i++){
    // checl on usage of help options
    if ((options[i]->alias() == "-h") || (options[i]->name() == "--help")) {
      failing.append(options[i]);
    // check on usage of version options
    } else if ((options[i]->alias() == "-v") || (options[i]->name() == "--version")) {
      failing.append(options[i]);
    }
  }
  QString msg = "";
  for (int i = 0; i < failing.count(); i++) {
    msg += failing[i]->helpLine() + "\n";
  }
  msg.chop(msg.count()-msg.lastIndexOf("\n"));
  QVERIFY2(failing.count() == 0, msg.toAscii());

  // help description
  QString hd = theEPart->helpDescription();
  QVERIFY(hd.count() > 0);

  // authors list
  QList<EAboutPerson*> persons = theEPart->aboutPersons();
  QVERIFY(persons.size() > 0);

  // (license)
}
