/*
    Copyright (C) 2012 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/

#include "echartlinetest.h"
#include "echartlinetest.moc"

#include <echartline.h>
#include <echartvalues.h>

#include <QTest>
#include <QPainter>

void EChartLineTest::testEmptyWidget()
{
	EChartLine widget;
	widget.show();
	// set the title to force repaint
	widget.setChartTitle("");
}

void EChartLineTest::testFullWidget()
{
	EChartLine widget;
	widget.setChartTitle("Test Title");
	widget.setXTitle("X Label");
	widget.setYTitle("Y Label");

	QList<QVariant> xValues;
	xValues << 1 << 2 << 3 << 4 << 5;
	QList<QVariant> yValues;
	yValues << "T 1" << "T 2" << "T 3" << "T 4" << "T 5";

	widget.setXTicks(xValues);
	widget.setYTicks(yValues);

	QList<EChartValues> valuesList;
	valuesList << EChartValues();
	valuesList[0].add(1, "T 3");
	valuesList[0].add(2, "T 1");
	valuesList[0].add(3, "T 5");
	valuesList[0].add(4, "T");
	valuesList[0].add(10, "T 4");

	valuesList << EChartValues();
	valuesList[1].add(1, "T 1");
	valuesList[1].add(2, "T 5");
	valuesList[1].add(3, "T 2");
	valuesList[1].add(4, "T 4");
	valuesList[1].add(5, "T 3");

	for(int i = 0; i < valuesList.size(); i++)
		widget.addDataSet(valuesList[i], QString("Line %1").arg(i));

	widget.show();

	QImage image(QSize(1000, 565), QImage::Format_RGB32);
	QPainter painter;
	painter.begin(&image);
	widget.exportChart(&painter, QRect(QPoint(0, 0), image.size()));
	painter.end();

	QCOMPARE(image, QImage(":/testcharts/echartline-testfullwidget.png"));

	QCOMPARE(widget.removeDataSet("Line 1"), true);
}

void EChartLineTest::testInterpolation()
{
	EChartLine widget;
	widget.setChartTitle("Test Title");
	widget.setXTitle("X Label");
	widget.setYTitle("Y Label");

	QList<QVariant> xValues;
	xValues << 1 << 2 << 3 << 4 << 5;
	QList<QVariant> yValues;
	yValues << 0 << 5 << 10 << 20;

	widget.setXTicks(xValues);
	widget.setYTicks(yValues);

	QList<EChartValues> valuesList;
	valuesList << EChartValues();
	valuesList[0].add(1, 2.5);
	valuesList[0].add(2.5, 5);
	valuesList[0].add(3, 7.5);
	valuesList[0].add(4.5, 10);
	valuesList[0].add(5.5, 0);

	for(int i = 0; i < valuesList.size(); i++)
		widget.addDataSet(valuesList[i], QString("Line %1").arg(i));

	QImage image(QSize(1000, 565), QImage::Format_RGB32);
	QPainter painter;
	painter.begin(&image);
	widget.exportChart(&painter, QRect(QPoint(0, 0), image.size()));
	painter.end();

	QCOMPARE(image, QImage(":/testcharts/echartline-testinterpolation.png"));

	widget.show();
}

QTEST_MAIN(EChartLineTest)
