/*
    Copyright (C) 2010 - 2012 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/

#include "eaboutmodeltest.h"
#include "eaboutmodeltest.moc"

#include <eaboutmodel.h>
#include <eaboutperson.h>
#include <modeltest.h>
#include "handlestrings.h"

#include <QTest>

void EAboutModelTest::testEAboutPerson()
{
	QString name = "test";
	QString given = "Person";
	QString mail = "example@example.org";
	EAboutPerson::Categories cats = EAboutPerson::Author | EAboutPerson::Translator;
	EAboutPerson person(name, given, cats);
	QCOMPARE(person.name(), name);
	QCOMPARE(person.givenName(), given);
	QCOMPARE(person.email(), QString());
	QCOMPARE(person.categories(), cats);
	
	EAboutPerson person1(name, given, mail, cats);
	QCOMPARE(person1.name(), name);
	QCOMPARE(person1.givenName(), given);
	QCOMPARE(person1.email(), mail);
	QCOMPARE(person1.categories(), cats);
}

void EAboutModelTest::testEAboutModel()
{
	EAboutModel *model = new EAboutModel(this);
	new ModelTest(model, this);
	
	EngSaS::AboutPersons persons;
	persons << new EAboutPerson("test", "Person", "example@example.org", EAboutPerson::Thanks);
	persons << new EAboutPerson("test1", "Person1", EAboutPerson::Author | EAboutPerson::Translator | EAboutPerson::Thanks);
	foreach(EAboutPerson *person, persons){
		QVERIFY(person);
		if(person->email().isEmpty())
			model->addPerson(person->name(), person->givenName(), person->categories());
		else
			model->addPerson(person->name(), person->givenName(), person->email(), person->categories());
	}
	model->addPerson(0);
	

	QCOMPARE(model->rowCount(), persons.size());
	for(int i = 0; i < persons.size(); i++){
		EAboutPerson *person = persons[i];
		QVERIFY(person);
		QCOMPARE(model->data(model->index(i, 0)).toString(), person->name());
		QCOMPARE(model->data(model->index(i, 1)).toString(), person->givenName());
		QCOMPARE(model->data(model->index(i, 2)).toString(), person->email());
	}
	
	QString html 	 = "<table width=100%>";
					html	+= "  <tr><th align=left>Author</th></tr><tr><th /></tr>";
					html	+= QString("  <tr><td>%1 %2 %3</td></tr>")
											.arg(persons[1]->givenName())
											.arg(persons[1]->name())
											.arg(persons[1]->email());
					html	+= "  <tr><th></th></tr>";
					html	+= "  <tr><th align=left>Translator</th></tr><tr><th /></tr>";
					html	+= QString("  <tr><td>%1 %2 %3</td></tr>")
											.arg(persons[1]->givenName())
											.arg(persons[1]->name())
											.arg(persons[1]->email());
					html	+= "  <tr><th></th></tr>";
					html	+= "  <tr><th align=left>Thanks</th></tr><tr><th /></tr>";
					html	+= QString("  <tr><td>%1 %2 %3</td></tr>")
											.arg(persons[0]->givenName())
											.arg(persons[0]->name())
											.arg(persons[0]->email());
					html	+= QString("  <tr><td>%1 %2 %3</td></tr>")
											.arg(persons[1]->givenName())
											.arg(persons[1]->name())
											.arg(persons[1]->email());
					html	+= "  <tr><th></th></tr>";
					html	+= "</table>";
	QCOMPARE(HandleStrings::strip(model->toHtml()), HandleStrings::strip(html));
}

QTEST_MAIN(EAboutModelTest)
