/*
    Copyright (C) 2010 - 2012 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/

#include "eactiontest.h"
#include "eactiontest.moc"

#include <eaction.h>

#include <QtTest>

void EActionTest::testEAction()
{
	QString label = "test";
	QString resourceStr = ":/libengsas/document-open";
	QString standardStr = "document-open";
	QIcon resource = QIcon(resourceStr);
#if QT_VERSION < 0x040600 || defined(Q_OS_WIN32) || defined(Q_OS_MAC)
	QIcon standard = resource;
#else
	QIcon standard = QIcon::fromTheme(standardStr);
#endif

	// test all constructors
	EAction *action = new EAction(label, this);
	QCOMPARE(action->text(), label);
	
	// because there is no icon, the text should be visible
	// Testing true, false, true, to simulate a full switch cycle
	action->setShowText(true);
	QCOMPARE(action->text(), label);
	action->setShowText(false);
	QCOMPARE(action->text(), label);
	action->setShowText(true);
	QCOMPARE(action->text(), label);
	
	delete action;
	
	action = new EAction(standardStr, resourceStr, label, this);
	QVERIFY(action->text().isEmpty());
	QCOMPARE(action->icon().pixmap(QSize(48, 48)).toImage(), standard.pixmap(QSize(48, 48)).toImage());

	// test actionWithText()
	QAction *actionWithText = action->actionWithText();
	QCOMPARE(actionWithText->text(), label);
	QCOMPARE(actionWithText->icon().pixmap(QSize(48, 48)).toImage(), standard.pixmap(QSize(48, 48)).toImage());

	// because there is an icon, the text should not be visible
	// Testing true, false, true, to simulate a full switch cycle
	action->setShowText(true);
	QCOMPARE(action->text(), label);
	QCOMPARE(actionWithText->text(), label);
	action->setShowText(false);
	QCOMPARE(action->text(), QString());
	QCOMPARE(actionWithText->text(), label);
	action->setShowText(true);
	QCOMPARE(action->text(), label);
	QCOMPARE(actionWithText->text(), label);

	action->setDisabled(true);
	QVERIFY(!action->isEnabled());
	QVERIFY(!actionWithText->isEnabled());
	
	action->setEnabled(true);
	QVERIFY(action->isEnabled());
	QVERIFY(actionWithText->isEnabled());

	QSignalSpy spy(action, SIGNAL(triggered()));
	actionWithText->trigger();
	QCOMPARE(spy.count(), 1);
	
	delete actionWithText;

	action = new EAction(standardStr, resourceStr, label, this, true);
	QCOMPARE(action->text(), label);
	QCOMPARE(action->icon().pixmap(QSize(48, 48)).toImage(), standard.pixmap(QSize(48, 48)).toImage());
	delete action;
}

QTEST_MAIN(EActionTest)
