/*
    Copyright (C) 2010 - 2012 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/

#include "ecmdlineoptiontest.h"
#include "ecmdlineoptiontest.moc"

#include <ecmdlineoption.h>

#include <QtTest>

void ECmdLineOptionTest::testBasicFunctions()
{
	QByteArray name = "test";
	QByteArray alias = "t";
	QString desc = "test description";
	QByteArray defaultVal = "defaultVal";
	ECmdLineOption opt(name, alias, desc, defaultVal);
	QCOMPARE(opt.isArgument(), false);
	QCOMPARE(opt.isOptional(), false);
	QCOMPARE(opt.requiresArgument(), false);
	QCOMPARE(opt.name(), "--"+ name);
	QCOMPARE(opt.alias(), "-"+ alias);
	QCOMPARE(opt.description(), desc);
	QCOMPARE(opt.defaultValue(), defaultVal);
	
	QString help = QString("-%1 --%2 %3 (Default: %4)")
									.arg(QString(alias))
									.arg(QString(name))
									.arg(desc)
									.arg(QString(defaultVal));
	QCOMPARE(opt.helpLine(), help);
	QString usage = QString("-%1").arg(QString(alias));
	QCOMPARE(opt.usage(), usage);
}

void ECmdLineOptionTest::testExtendedSetName_data()
{
	QTest::addColumn<QString>("setName");
	QTest::addColumn<QString>("setAlias");
	QTest::addColumn<QString>("name");
	QTest::addColumn<QString>("alias");
	QTest::addColumn<bool>("requiresArgument");
	QTest::addColumn<bool>("isArgument");
	QTest::addColumn<bool>("isOptional");
	QTest::addColumn<QString>("usage");
	QTest::addColumn<QString>("help");
	
	QTest::newRow("[optional]") << "[optional]" << "o"
															<< "--optional" << "-o"
															<< false << false << true
															<< "[-o]" << "[-o --optional]";
	QTest::newRow("[optional] <argument>") 	<< "[optional] <argument>" << "o"
																					<< "--optional" << "-o"
																					<< true << false << true
																					<< "[-o <argument>]" << "[-o --optional <argument>]";
	QTest::newRow("required") << "required" << ""
														<< "--required" << ""
														<< false << false << false
														<< "--required" << "--required";
	QTest::newRow("required <bla>") << "required  <bla>" << ""
																	<< "--required" << ""
																	<< true << false << false
																	<< "--required <argument>" << "--required <argument>";
	QTest::newRow("+required <argument>") << "+required <argument>" << "o"
																				<< "required" << ""
																				<< false << true << false
																				<< "required" << "required";
	QTest::newRow("+[optional] <argument>") << "+[optional] <argument>" << "o"
																					<< "optional" << ""
																					<< false << true << true
																					<< "[optional]" << "[optional]";
}

void ECmdLineOptionTest::testExtendedSetName()
{
	QFETCH(QString, setName);
	QFETCH(QString, setAlias);
	QFETCH(QString, name);
	QFETCH(QString, alias);
	QFETCH(bool, requiresArgument);
	QFETCH(bool, isArgument);
	QFETCH(bool, isOptional);
	QFETCH(QString, usage);
	QFETCH(QString, help);
	
	ECmdLineOption opt(setName.toUtf8(), setAlias.toUtf8());
	QCOMPARE(QString(opt.name()), name);
	QCOMPARE(QString(opt.alias()), alias);
	QCOMPARE(opt.requiresArgument(), requiresArgument);
	QCOMPARE(opt.isArgument(), isArgument);
	QCOMPARE(opt.isOptional(), isOptional);
	QCOMPARE(opt.usage().trimmed(), usage);
	QCOMPARE(opt.helpLine().trimmed(), help);
}

QTEST_MAIN(ECmdLineOptionTest)
