/*
    Copyright (C) 2010 - 2012 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/


#include "efileinfotest.h"
#include "efileinfotest.moc"

#include <efileinfo.h>

#include <QTest>

void EFileInfoTest::attributes()
{
	quint64 size = 0;
	EFileInfo info;
	QCOMPARE(info.isValid(), false);
	QCOMPARE(info.id(), QString());
	QCOMPARE(info.name(), QString());
	QCOMPARE(info.mimeType(), QString());
	QCOMPARE(info.size(), size);
	
	QString id = "testId";
	info.setId(id);
	QCOMPARE(info.id(), id);
	
	QString name = "filename.txt";
	info.setName(name);
	QCOMPARE(info.name(), name);
	QCOMPARE(info.isValid(), true);
	
	QString mimeType = "application/pdf";
	info.setMimeType(mimeType);
	QCOMPARE(info.mimeType(), mimeType);
	
	size = Q_UINT64_C(932838457459459);
	info.setSize(size);
	QCOMPARE(info.size(), size);
}

void EFileInfoTest::operators()
{
	EFileInfo info1, info2, info3, info4;
	info1.setId("file1");
	info2.setId("file2");
	info3.setId("file3");
	info4.setId(info1.id());
	
	QCOMPARE(info1, info4);
	QVERIFY(info1 == info1.id());
	
	QList<EFileInfo> infos;
	infos << info1 << info2 << info3;
	QCOMPARE(infos.indexOf(info2), 1);
	
	infos.removeOne(info2);
	QCOMPARE(infos.size(), 2);
	QCOMPARE(infos[0], info1);
	QCOMPARE(infos.indexOf(info2), -1);
	QCOMPARE(infos[1], info3);
}

QTEST_MAIN(EFileInfoTest)
