/*
    Copyright (C) 2010 - 2012 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/

#include "emainwindowtest.h"
#include "emainwindowtest.moc"

#include <emainwindow.h>
#include "testwidget.h"
#include <eaction.h>

#include <QTest>
#include <QApplication>
#include <QSignalSpy>
#include <QDockWidget>
#include <QToolBar>
#include <QAction>
#include <QMenu>
#include <QStatusBar>
#include <QMenuBar>

Q_DECLARE_METATYPE(EngSaS::AvailableActions);
Q_DECLARE_METATYPE(EngSaS::AvailableAction);

void EMainWindowTest::setWindowTitle()
{
	EMainWindow mw;
	QString title = "test";
	if(QApplication::applicationName().isEmpty())
		QApplication::setApplicationName(title);
	mw.setWindowTitle(title);
	title = QApplication::applicationName() +" - "+ title;
	QCOMPARE(mw.windowTitle(), title);

	// test also the set of organization name
	QCOMPARE(QApplication::organizationName(), QString("EngSaS"));
	QApplication::setOrganizationName(title);
	EMainWindow mw2;
	QCOMPARE(QApplication::organizationName(), title);
}

void EMainWindowTest::testActions_data()
{
	QTest::addColumn<EngSaS::AvailableActions>("testActions");

	QTest::newRow("Nothing") << EngSaS::AvailableActions();
	QTest::newRow("Open") << EngSaS::AvailableActions(EngSaS::FileOpen);
	QTest::newRow("New") << EngSaS::AvailableActions(EngSaS::FileNew);
	QTest::newRow("Save") << EngSaS::AvailableActions(EngSaS::FileSave);
	QTest::newRow("SaveAs") << EngSaS::AvailableActions(EngSaS::FileSaveAs);
	QTest::newRow("Print") << EngSaS::AvailableActions(EngSaS::FilePrint);
	QTest::newRow("Undo") << EngSaS::AvailableActions(EngSaS::EditUndo);
	QTest::newRow("Redo") << EngSaS::AvailableActions(EngSaS::EditRedo);
	QTest::newRow("Cut") << EngSaS::AvailableActions(EngSaS::EditCut);
	QTest::newRow("Copy") << EngSaS::AvailableActions(EngSaS::EditCopy);
	QTest::newRow("Paste") << EngSaS::AvailableActions(EngSaS::EditPaste);
	QTest::newRow("Reload") << EngSaS::AvailableActions(EngSaS::ViewReload);
	EngSaS::AvailableActions testActions = EngSaS::FileSaveAs | EngSaS::EditPaste | EngSaS::ViewReload;
	QTest::newRow("SaveAs Paste Reload") << testActions;
}

void EMainWindowTest::testActions()
{
	QFETCH(EngSaS::AvailableActions, testActions);
	TestWidget *tw = new TestWidget(testActions);
	connectESlots(tw);
	tw->emitSetCentralWidget();
	QCOMPARE(getCentralWidget(), tw);

	QMap<QAction*, EngSaS::AvailableAction> actionMap;
	actionMap.insert(fileOpenAction, EngSaS::FileOpen);
	actionMap.insert(fileNewAction, EngSaS::FileNew);
	actionMap.insert(fileSaveAction, EngSaS::FileSave);
	actionMap.insert(fileSaveAsAction, EngSaS::FileSaveAs);
	actionMap.insert(filePrintAction, EngSaS::FilePrint);
	actionMap.insert(editUndoAction, EngSaS::EditUndo);
	actionMap.insert(editRedoAction, EngSaS::EditRedo);
	actionMap.insert(editCutAction, EngSaS::EditCut);
	actionMap.insert(editCopyAction, EngSaS::EditCopy);
	actionMap.insert(editPasteAction, EngSaS::EditPaste);
	actionMap.insert(viewReloadAction, EngSaS::ViewReload);

	if(testActions.testFlag(EngSaS::FileSave))
		testActions |= EngSaS::FileSaveAs;
	
	// check status
	foreach(QAction *action, actionMap.uniqueKeys()){
		if(testActions.testFlag(actionMap.value(action)))
			QCOMPARE(action->isVisible(), true);
		else
			QCOMPARE(action->isVisible(), false);
	}
}

void EMainWindowTest::testSimpleSlots()
{
	QSKIP("Don't know how to handle them", SkipAll);
}

void EMainWindowTest::testAddMenu()
{
	TestWidget *tw = new TestWidget(0);
	connectESlots(tw);
	tw->emitSetCentralWidget();
	QCOMPARE(getCentralWidget(), tw);

	// add Menu
	tw->emitAddMenu(0, "", "");
	QCOMPARE(otherMenu->menuAction()->isVisible(), false);
	QMenu *menu = new QMenu();
	menu->setObjectName("testMenu");
	// into existing parent
	tw->emitAddMenu(menu, "viewMenu", "");
	QMenu *result = viewMenu->findChild<QMenu*>(menu->objectName());
	QCOMPARE(result, menu);
	delete menu; // delete it, because otherMenu should be empty again
	menu = new QMenu();
	// without parent
	menu->setObjectName("testMenu2");
	tw->emitAddMenu(menu, "", "");
	result = menubar->findChild<QMenu*>(menu->objectName());
	QCOMPARE(result, menu);
	// not existent parent
	menu->setObjectName("testMenu3");
	tw->emitAddMenu(menu, "asdasdasd", "");
	result = menubar->findChild<QMenu*>(menu->objectName());
	QCOMPARE(result, menu);
	QCOMPARE(otherMenu->menuAction()->isVisible(), false);
	QAction *action = new QAction(menu);
	tw->emitAddAction(action, menu->objectName(), "");
	QCOMPARE(otherMenu->menuAction()->isVisible(), true);
	delete menu;
	QEXPECT_FAIL("", "See TODO of EMainWindow related to auto hiding otherMenu", Continue);
	QCOMPARE(otherMenu->menuAction()->isVisible(), false);
	
	delete tw;
}

void EMainWindowTest::testAddAction()
{
	TestWidget *tw = new TestWidget(EngSaS::AvailableActions());
	connectESlots(tw);
	tw->emitSetCentralWidget();
	QCOMPARE(getCentralWidget(), tw);

	tw->emitAddAction(0, "", "");
	
	// add a submenu to test recursive findChildren function
	QMenu *subMenu = new QMenu();
	subMenu->setObjectName("subMenu");
	tw->emitAddMenu(subMenu, "fileMenu", "");
	
	// addAction
	QAction *action = new QAction("Test", 0);
	// with parent
	action->setObjectName("testAction1");
	tw->emitAddAction(action, "subMenu", "");
	QAction *result = subMenu->findChild<QAction*>(action->objectName());
	QCOMPARE(result, action);
	action->setObjectName("action2");
	// without parent
	tw->emitAddAction(action, "", "");
	result = otherMenu->findChild<QAction*>(action->objectName());
	QCOMPARE(result, action);
	QCOMPARE(otherMenu->menuAction()->isVisible(), true);
	delete action; // to check wether otherMenu is disabled again
	QEXPECT_FAIL("", "See TODO of EMainWindow related to auto hiding otherMenu", Continue);
	QCOMPARE(otherMenu->menuAction()->isVisible(), false);
	// with nono existent parent
	action = new QAction("Test", 0);
	action->setObjectName("action3");
	tw->emitAddAction(action, "asdasdasd", "");
	result = otherMenu->findChild<QAction*>(action->objectName());
	QCOMPARE(result, action);
	QCOMPARE(otherMenu->menuAction()->isVisible(), true);

	delete action;
	delete tw;
}

void EMainWindowTest::testAddToolBar()
{
	TestWidget *tw = new TestWidget(EngSaS::AvailableActions());
	connectESlots(tw);
	tw->emitSetCentralWidget();
	QCOMPARE(getCentralWidget(), tw);

	QToolBar *toolBar = new QToolBar("test", tw);
	tw->emitAddToolBar(toolBar, Qt::BottomToolBarArea);
	QCOMPARE(toolBarArea(toolBar), Qt::BottomToolBarArea);
	tw->emitAddToolBar(toolBar, Qt::TopToolBarArea);
	QCOMPARE(toolBarArea(toolBar), Qt::TopToolBarArea);
	
	delete tw;
}

void EMainWindowTest::testAddNewDockWidget()
{
	TestWidget *tw = new TestWidget(EngSaS::AvailableActions());
	connectESlots(tw);
	tw->emitSetCentralWidget();
	QCOMPARE(getCentralWidget(), tw);

	QDockWidget *widget = new QDockWidget("test", tw);
	tw->emitAddNewDockWidget(widget, Qt::BottomDockWidgetArea);
	QCOMPARE(dockWidgetArea(widget), Qt::BottomDockWidgetArea);

	delete tw;
}

void EMainWindowTest::testShowStatusWidget()
{
	TestWidget *tw = new TestWidget(EngSaS::AvailableActions());
	connectESlots(tw);
	tw->emitSetCentralWidget();
	QCOMPARE(getCentralWidget(), tw);

	QWidget *widget = new QWidget(tw);
	widget->setObjectName("testStatusWidget");
	tw->emitShowStatusWidget(widget);
	QWidget *result = statusbar->findChild<QWidget*>(widget->objectName());
	QCOMPARE(result, widget);

	delete tw;
}

void EMainWindowTest::fileNew()
{
	TestWidget *tw = new TestWidget(EngSaS::FileNew);
	connectESlots(tw);
	tw->emitSetCentralWidget();
	QCOMPARE(getCentralWidget(), tw);

	QSKIP("Can not use fileNewAction here, don't know why", SkipSingle);
	QSignalSpy spy(tw, SIGNAL(fileNewCalled()));
	fileNewAction->trigger();
	QCOMPARE(spy.count(), 1);
	
	delete tw;
}

void EMainWindowTest::testFileOpen()
{
	QSKIP("Don't know how to handle them", SkipAll);
}

void EMainWindowTest::testFileSave()
{
	QSKIP("Don't know how to handle them", SkipAll);
}

void EMainWindowTest::testFileSaveAs()
{
	QSKIP("Don't know how to handle them", SkipAll);
}

void EMainWindowTest::testFilePrint()
{
	TestWidget *tw = new TestWidget(EngSaS::FilePrint);
	connectESlots(tw);
	tw->emitSetCentralWidget();
	QCOMPARE(getCentralWidget(), tw);

	QSignalSpy spy(tw, SIGNAL(printCalled()));
	filePrintAction->trigger();
	QCOMPARE(spy.count(), 1);

	delete tw;
}

void EMainWindowTest::testFileExit()
{
	EMainWindow *mw = new EMainWindow();
	mw->show();
	QCOMPARE(mw->isVisible(), true);
	fileExitAction->trigger();
	QCOMPARE(mw->isHidden(), true);
	delete mw;
}

void EMainWindowTest::testEditCut()
{
	TestWidget *tw = new TestWidget(EngSaS::EditCut);
	connectESlots(tw);
	tw->emitSetCentralWidget();
	QCOMPARE(getCentralWidget(), tw);

	QSignalSpy spy(tw, SIGNAL(cutCalled()));
	editCutAction->trigger();
	QCOMPARE(spy.count(), 1);

	delete tw;
}

void EMainWindowTest::testEditCopy()
{
	TestWidget *tw = new TestWidget(EngSaS::EditCopy);
	connectESlots(tw);
	tw->emitSetCentralWidget();
	QCOMPARE(getCentralWidget(), tw);

	QSignalSpy spy(tw, SIGNAL(copyCalled()));
	editCopyAction->trigger();
	QCOMPARE(spy.count(), 1);

	delete tw;
}

void EMainWindowTest::testEditPaste()
{
	TestWidget *tw = new TestWidget(EngSaS::EditPaste);
	connectESlots(tw);
	tw->emitSetCentralWidget();
	QCOMPARE(getCentralWidget(), tw);

	QSignalSpy spy(tw, SIGNAL(pasteCalled()));
	editPasteAction->trigger();
	QCOMPARE(spy.count(), 1);

	delete tw;
}

void EMainWindowTest::testEditUndo()
{
	TestWidget *tw = new TestWidget(EngSaS::EditUndo);
	connectESlots(tw);
	tw->emitSetCentralWidget();
	QCOMPARE(getCentralWidget(), tw);

	QSignalSpy spy(tw, SIGNAL(undoCalled()));
	editUndoAction->trigger();
	QCOMPARE(spy.count(), 1);

	delete tw;
}

void EMainWindowTest::testEditRedo()
{
	TestWidget *tw = new TestWidget(EngSaS::EditRedo);
	connectESlots(tw);
	tw->emitSetCentralWidget();
	QCOMPARE(getCentralWidget(), tw);

	QSignalSpy spy(tw, SIGNAL(redoCalled()));
	editRedoAction->trigger();
	QCOMPARE(spy.count(), 1);

	delete tw;
}

void EMainWindowTest::testViewReload()
{
	TestWidget *tw = new TestWidget(EngSaS::ViewReload);
	connectESlots(tw);
	tw->emitSetCentralWidget();
	QCOMPARE(getCentralWidget(), tw);

	QSignalSpy spy(tw, SIGNAL(reloadCalled()));
	viewReloadAction->trigger();
	QCOMPARE(spy.count(), 1);

	delete tw;
}

QTEST_MAIN(EMainWindowTest)
