/*
    Copyright (C) 2010 - 2012 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/

#include "eobjecttest.h"
#include "eobjecttest.moc"

#include <eobject.h>

#include <QTest>
#include <QSignalSpy>
#include <QFile>

Q_DECLARE_METATYPE(QMenu*);
Q_DECLARE_METATYPE(QAction*);
Q_DECLARE_METATYPE(QToolBar*);
Q_DECLARE_METATYPE(QWidget*);
Q_DECLARE_METATYPE(QDockWidget*);
Q_DECLARE_METATYPE(Qt::DockWidgetArea);
Q_DECLARE_METATYPE(Qt::ToolBarArea);

void EObjectTest::initTestCase()
{
	qRegisterMetaType<QMenu*>();
	qRegisterMetaType<QAction*>();
	qRegisterMetaType<QToolBar*>();
	qRegisterMetaType<QDockWidget*>();
	qRegisterMetaType<Qt::DockWidgetArea>();
	qRegisterMetaType<Qt::ToolBarArea>();
}

void EObjectTest::connectByPass()
{
	EObject *parent = new EObject(this);
	EObject *child = new EObject(parent);

	// testing different connection variants
	EObject::connectByPass(parent, child);
	delete child;
	child = new EObject(parent);
	parent->connectByPass(child);

	// testing by passes
	parent->connectByPass(this);

	QString msg = "test";

	// error signal
	QSignalSpy spy(parent, SIGNAL(error(QString)));
	emit error(msg);
	QCOMPARE(spy.count(), 1);
	QCOMPARE(spy.takeFirst()[0].toString(), msg);

	// warning signal
	QSignalSpy spy1(parent, SIGNAL(warning(QString)));
	emit warning(msg);
	QCOMPARE(spy1.count(), 1);
	QCOMPARE(spy1.takeFirst()[0].toString(), msg);

	// information signal
	QSignalSpy spy2(parent, SIGNAL(information(QString)));
	emit information(msg);
	QCOMPARE(spy2.count(), 1);
	QCOMPARE(spy2.takeFirst()[0].toString(), msg);

	// addMenu signal
	QMenu *testMenu = new QMenu();
	QSignalSpy spy3(parent, SIGNAL(addMenu(QMenu*, QString, QString)));
	emit addMenu(testMenu, msg);
	QCOMPARE(spy3.count(), 1);
	QList<QVariant> arguments = spy3.takeFirst();
	QCOMPARE(arguments.at(0).value<QMenu*>(), testMenu);
	QCOMPARE(arguments.at(1).toString(), msg);
	delete testMenu;

	// addAction signal
	QAction *testAction = new QAction("", 0);
	QSignalSpy spy4(parent, SIGNAL(addAction(QAction*, QString, QString)));
	emit addAction(testAction, msg);
	QCOMPARE(spy4.count(), 1);
	arguments = spy4.takeFirst();
	QCOMPARE(arguments.at(0).value<QAction*>(), testAction);
	QCOMPARE(arguments.at(1).toString(), msg);
	delete testAction;

	// addToolBar signal
	QToolBar *testToolBar = new QToolBar();
	QSignalSpy spy5(parent, SIGNAL(addToolBar(QToolBar*, Qt::ToolBarArea)));
	emit addToolBar(testToolBar);
	QCOMPARE(spy5.count(), 1);
	arguments = spy5.takeFirst();
	QCOMPARE(arguments.at(0).value<QToolBar*>(), testToolBar);
	delete testToolBar;

	// addNewDockWidget signal
	QDockWidget *testDockWidget = new QDockWidget();
	QSignalSpy spy6(parent, SIGNAL(addNewDockWidget(QDockWidget*, Qt::DockWidgetArea)));
	emit addNewDockWidget(testDockWidget, Qt::BottomDockWidgetArea);
	QCOMPARE(spy6.count(), 1);
	arguments = spy6.takeFirst();
	QCOMPARE(arguments.at(0).value<QDockWidget*>(), testDockWidget);
	QCOMPARE(arguments.at(1).value<Qt::DockWidgetArea>(), Qt::BottomDockWidgetArea);
	delete testDockWidget;

	// setCentralWidget signal
	QWidget *testWidget = new QWidget();
	QSignalSpy spy7(parent, SIGNAL(setCentralWidget(QWidget*)));
	emit setCentralWidget(testWidget);
	QCOMPARE(spy7.count(), 1);
	arguments = spy7.takeFirst();
	QCOMPARE(arguments.at(0).value<QWidget*>(), testWidget);
	delete testWidget;

	// showSettings signal
	QSignalSpy spy8(parent, SIGNAL(showSettings()));
	emit showSettings();
	QCOMPARE(spy8.count(), 1);

	// showHelp signal
	QSignalSpy spy9(parent, SIGNAL(showHelp(QString)));
	emit showHelp(msg);
	QCOMPARE(spy9.count(), 1);
	arguments = spy9.takeFirst();
	QCOMPARE(arguments.at(0).toString(), msg);

	// showStatusWidget signal
	QWidget *statusWidget = new QWidget();
	QSignalSpy spy10(parent, SIGNAL(showStatusWidget(QWidget*)));
	emit showStatusWidget(statusWidget);
	QCOMPARE(spy10.count(), 1);
	arguments = spy10.takeFirst();
	QCOMPARE(arguments.at(0).value<QWidget*>(), statusWidget);

	delete child;
	delete parent;
}

void EObjectTest::reloadSettings(){
	EObject object;
	object.reloadSettings();
}

QTEST_MAIN(EObjectTest)
