/*
    Copyright (C) 2010 - 2012 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/

#include "epushbuttontest.h"
#include "epushbuttontest.moc"

#include <epushbutton.h>

#include <QtTest>

void EPushButtonTest::testEPushButton()
{
	QString label = "test";
	QString resourceStr = ":/libengsas/document-open";
	QString standardStr = "document-open";
	QIcon resource = QIcon(resourceStr);
// if QIcon::fromTheme is not available and on OS without theme support
// we must use the standard item
#if QT_VERSION < 0x040600 || defined(Q_OS_WIN32) || defined(Q_OS_MAC)
	QIcon standard = resource;
#else
	QIcon standard = QIcon::fromTheme(standardStr);
#endif
	
	EPushButton *button = new EPushButton();
	QCOMPARE(button->text(), QString());
	delete button;

	button = new EPushButton(label);
	QCOMPARE(button->text(), label);

	// because there is no icon, the text should be visible
	// Testing true, false, true, to simulate a full switch cycle
	button->setShowText(true);
	QCOMPARE(button->text(), label);
	button->setShowText(false);
	QCOMPARE(button->text(), label);
	button->setShowText(true);
	QCOMPARE(button->text(), label);
	
	delete button;

	button = new EPushButton(standardStr, resourceStr, label);
	QCOMPARE(button->text(), QString());
	QCOMPARE(button->icon().pixmap(QSize(48, 48)).toImage(), standard.pixmap(QSize(48, 48)).toImage());

	// because there is an icon, the text should not be visible
	// Testing true, false, true, to simulate a full switch cycle
	button->setShowText(true);
	QCOMPARE(button->text(), label);
	button->setShowText(false);
	QCOMPARE(button->text(), QString());
	button->setShowText(true);
	QCOMPARE(button->text(), label);
	
	delete button;
}

QTEST_MAIN(EPushButtonTest)
