/*
    Copyright (C) 2010 - 2012 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/

#include "esettingsmodeltest.h"
#include "esettingsmodeltest.moc"

#include <esettingsmodel.h>
#include <esettingsmodelitem.h>
#include <modeltest.h>

#include <QtTest>

void ESettingsModelTest::testESettingsModelItem()
{
	ESettingsModelItem *item = new ESettingsModelItem(QString(), 0, 0);
	QCOMPARE(item->name(), QString());
	QVERIFY(!item->widget());
	QVERIFY(!item->parent());
	QCOMPARE(item->rowCount(), 0);
	QString test;
	QCOMPARE(item->save(test), true);
	delete item;

	QString name = "test";
	ESettingsModelItem *parent = new ESettingsModelItem(name, 0, 0);
	item = new ESettingsModelItem(name, 0, parent);
	QCOMPARE(item->name(), name);
	QVERIFY(!item->widget());
	QCOMPARE(item->parent(), parent);

	ESettingsModelItem *child = new ESettingsModelItem(name, 0, 0);
	item->addChild(parent);
	QCOMPARE(parent->parent(), item);
	item->addChild(parent);
	item->addChild(child);
	QCOMPARE(item->rowCount(), 2);
	QCOMPARE(item->childRow(0), -1);
	QCOMPARE(item->childRow(child), 1);
	QVERIFY(!item->child(-1));
	QVERIFY(!item->child(item->rowCount()));
	QCOMPARE(item->child(item->childRow(child)), child);
	delete item;
}

void ESettingsModelTest::testESettingsModel()
{
	ESettingsModel *model = new ESettingsModel(this);
	ModelTest *modelTest = new ModelTest(model, this);
	Q_UNUSED(modelTest);

	QString nl1 = "test1";
	QString nl11 = "test11";
	QString nl12 = "test12";
	QString nl121 = "test121";
	QString nl2 = "test2";
	ESettingsModelItem *l1 = new ESettingsModelItem(nl1, 0, 0);
	l1->addChild(new ESettingsModelItem(nl11, 0, 0));
	ESettingsModelItem *l12 = new ESettingsModelItem(nl12, 0, 0);
	l1->addChild(l12);
	l12->addChild(new ESettingsModelItem(nl121, 0, 0));

	QVERIFY(model->addItem(l1));
	QVERIFY(model->addItem(new ESettingsModelItem(nl2, 0, 0)));

	QModelIndex il1 = model->index(0, 0);
	QModelIndex il11 = model->index(0, 0, il1);
	QModelIndex il12 = model->index(1, 0, il1);
	QModelIndex il121 = model->index(0, 0, il12);
	QModelIndex il2 = model->index(1, 0);

	QVERIFY(il1.isValid());
	QVERIFY(il11.isValid());
	QVERIFY(il12.isValid());
	QVERIFY(il121.isValid());
	QVERIFY(il2.isValid());

	QCOMPARE(model->data(il1).toString(), nl1);
	QCOMPARE(model->data(il11).toString(), nl11);
	QCOMPARE(model->data(il12).toString(), nl12);
	QCOMPARE(model->data(il121).toString(), nl121);
	QCOMPARE(model->data(il2).toString(), nl2);
}

QTEST_MAIN(ESettingsModelTest)
