/*
    Copyright (C) 2010 - 2012 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/

#include "testitem.h"
#include "testitem.moc"

#include "testsubitem.h"

#include <exmlparseritem.h>

#include <QStringList>

TestItem::TestItem(EXmlModel *model, EXmlItem *parent)
	: TestModelBaseItem("testitem", parent)
{
	myModel = model;
}

TestSubItem *TestItem::child(int row) const
{
	EXmlItem *item = TestModelBaseItem::child(row);
	if(!item)
		return NULL;
	return qobject_cast<TestSubItem*>(item);
}

bool TestItem::appendChild(QString label, QString value, bool emitLayoutChanges)
{
	TestSubItem *item = new TestSubItem(label, myModel, this);
	item->setValue(value);
	if(!addChild(rowCount(), item, emitLayoutChanges)){
		delete item;
		return false;
	}
	return true;
}

void TestItem::clone(const TestItem *source)
{
	if(!source)
		return;

	setLabel(source->label());
	for(int i = 0; i < source->rowCount(); i++){
		TestSubItem *attr = source->child(i);
		appendChild(attr->label(), attr->value());
	}
}

void TestItem::setId(QString id)
{
	TestModelBaseItem::setId(id);
	for(int i = 0; i < rowCount(); i++){
		TestSubItem *attr = qobject_cast<TestSubItem*>(child(i));
#if QT_VERSION >= 0x040800
		if(Q_LIKELY(attr))
#else
		if(attr)
#endif
			attr->setId(QString("%1-%2").arg(id).arg(i));
	}
}

bool TestItem::load(const QDomElement element, QMap<QString, EXmlParserItem *> parser)
{
	// basic checking, if given element is valid
	if(element.tagName().toLower() != tagName())
		return false;

	setId(element.attribute("id"));

	// process child elements
	QDomElement child = element.firstChildElement();
	while(!child.isNull()){
		if(!loadBasics(child)){
			// all other should be handled by others
			QStringList keys = parser.uniqueKeys();
			if(!keys.contains(child.tagName().toLower()))
				return false;
			if(!addChild(rowCount(), parser.value(child.tagName().toLower())->load(child, this, parser), false))
				return false;
		}
		child = child.nextSiblingElement();
	}

	return true;
}

bool TestItem::save(QDomElement &useElem)
{
	// we recieve segfaults, when using null elements
	// therefore we check for it
	if(useElem.isNull())
		return false;

	return TestModelBaseItem::save(useElem);
}

EXmlItem *TestItem::copy () const
{
	TestItem *target = new TestItem(NULL);
	target->clone(this);
	return target;
}
