/*
    Copyright (C) 2010 - 2012 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/

#ifndef TESTITEM_H
#define TESTITEM_H

#include "testmodelbaseitem.h"

class TestSubItem;

class EXmlModel;

/**
	* @brief One possible solution, which consists of several TestSubItem.
	*
	* Something like:
	*
	* - TestItem: All inclusive holiday
	*   - TestSubItem: Target - Somewhere
	*   - TestSubItem: Food inclusive
	* - TestItem: customized tours
	*   - TestSubItem: Target - Somewhere else
	*   - TestSubItem: Food not inclusive
	*
	* @todo implement EXmlItem API correctly. Especially the clone and copy functions.
	*/
class TestItem : public TestModelBaseItem
{
	Q_OBJECT
	public:
		TestItem(EXmlModel *model, EXmlItem *parent = NULL);

		inline EXmlModel *model() const{ return myModel; }

		/**
			* @brief Returns the child from @p row.
			*
			* If invalid row is passed, NULL is returned.
			* To add new childs use appendChild() and to remove childs
			* use removeChild()
			*/
		TestSubItem *child(int row) const;
		/**
			* @brief Adds a new child after all already listed childs.
			*
			* To access it use child() or to remove it, call removeChild().
			* @see EXmlItem::addChild();
			*/
		bool appendChild(QString label, QString value, bool emitLayoutChanges = true);
		/**
			* @brief Copies the data of @p source into current option.
			*/
		void clone(const TestItem *source);
		void setId(QString id);

		// EXmlItem API
		static inline EXmlItem *create(EXmlModel *model){ return new TestItem(model); }
		bool load(const QDomElement element, QMap< QString, EXmlParserItem * > parser);
		bool save(QDomElement &useElem);
		EXmlItem *copy() const;

	private:
		EXmlModel *myModel;
};

#endif // TESTITEM_H
