/*
    Copyright (C) 2010 - 2012 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/

#include "testmodelbaseitem.h"
#include "testmodelbaseitem.moc"

#include <exmlparseritem.h>

#include <QStringList>

TestModelBaseItem::TestModelBaseItem(QString tagName, EXmlItem *parent)
	: EXmlItem(tagName, parent)
{
	if(tagName.isEmpty())
		setTagName("TestModelBaseItem");
}

bool TestModelBaseItem::isValid() const
{
	if(label().isEmpty())
		return false;

	return true;
}

QString TestModelBaseItem::label() const
{
	return myLabel;
}

void TestModelBaseItem::setLabel(QString label)
{
	myLabel = label.left(120);
	emit changed(this);
}

QString TestModelBaseItem::comment() const
{
	return myComment;
}

void TestModelBaseItem::setComment(QString comment)
{
	myComment = comment.left(1000);
	emit changed(this);
}

QString TestModelBaseItem::id() const
{
	return myId;
}

void TestModelBaseItem::setId(QString id)
{
	myId = id;
}

bool TestModelBaseItem::load(const QDomElement element, QMap<QString, EXmlParserItem *> parser)
{
	// basic checking, if given element is valid
	if(element.tagName().toLower() != tagName())
		return false;

	// process child elements
	QDomElement child = element.firstChildElement();
	while(!child.isNull()){
		// processing the child elements, which are handled within this class
		if(!loadBasics(child)){
			// all other should be handled by others
			QStringList keys = parser.uniqueKeys();
			if(!keys.contains(child.tagName().toLower()))
				return false;

			if(!addChild(rowCount(), parser.value(child.tagName().toLower())->load(child, this, parser), false))
				return false;
		}
		child = child.nextSiblingElement();
	}

	return true;
}

bool TestModelBaseItem::save(QDomElement &useElem)
{
	// we recieve segfaults, when using null elements
	// therefore we check for it
	if(useElem.isNull())
		return false;

	useElem.setTagName(tagName());

	useElem.setAttribute("id", id());

	QDomDocument doc = useElem.ownerDocument();
	// adding label
	QDomElement tmp = doc.createElement("label");
	tmp.appendChild(doc.createTextNode(label()));
	useElem.appendChild(tmp);
	// adding comment
	tmp = doc.createElement("comment");
	tmp.appendChild(doc.createTextNode(comment()));
	useElem.appendChild(tmp);

	return EXmlItem::save(useElem);
}

QVariant TestModelBaseItem::data (int column, int role) const
{
	if((role != Qt::DisplayRole) && (role != Qt::EditRole))
		return QVariant();

	switch(column){
		case 0:
			return label();
		case 1:
			return comment();
		default:
			return QVariant();
	}
}

EXmlItem *TestModelBaseItem::copy () const
{
	TestModelBaseItem *res = dynamic_cast<TestModelBaseItem*>(create(NULL));
	if(!res)
		return NULL;
	copyBase(res);
	res->setLabel(label());
	res->setComment(comment());
	return res;
}

bool TestModelBaseItem::loadBasics(const QDomElement &element)
{
	if(element.isNull())
		return false;

	if(element.tagName().toLower() == "label"){
		setLabel(element.text());
		return true;
	}
	if(element.tagName().toLower() == "comment"){
		setComment(element.text());
		return true;
	}
	return false;
}
