/*
    Copyright (C) 2010 - 2012 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/

#include "testsubitem.h"
#include "testsubitem.moc"

#include "testitem.h"

#include <exmlparseritem.h>

#include <QStringList>

TestSubItem::TestSubItem(QString label, EXmlModel *model, TestItem *parent)
	: TestModelBaseItem("testsubitem", parent)
{
	setLabel(label);
	myModel = model;
}

bool TestSubItem::isValid() const
{
	if(!TestModelBaseItem::isValid())
		return false;
	if(value().isEmpty())
		return false;
	return true;
}

TestItem* TestSubItem::testItem() const
{
	return dynamic_cast<TestItem*>(EXmlItem::parent());
}

EXmlModel *TestSubItem::model() const
{
	return myModel;
}

void TestSubItem::setValue(QString value)
{
	myValue = value.left(120);
}

QString TestSubItem::value() const
{
	return myValue;
}

bool TestSubItem::load(const QDomElement element, QMap<QString, EXmlParserItem *> parser)
{
	// basic checking, if given element is valid
	if(element.tagName().toLower() != tagName())
		return false;

	setId(element.attribute("id"));

	// process child elements
	QDomElement child = element.firstChildElement();
	while(!child.isNull()){
		// processing the child elements, which are handled within this class
		if(!loadBasics(child)){
			if(child.tagName().toLower() == "value"){
				setValue(child.text());
			}
			else{
				// all other should be handled by others
				QStringList keys = parser.uniqueKeys();
				if(!keys.contains(child.tagName().toLower()))
					return false;

				if(!addChild(rowCount(), parser.value(child.tagName().toLower())->load(child, this, parser), false))
					return false;
			}
		}
		child = child.nextSiblingElement();
	}

	return true;
}

bool TestSubItem::save(QDomElement &useElem)
{
	// we recieve segfaults, when using null elements
	// therefore we check for it
	if(useElem.isNull())
		return false;

	QDomDocument doc = useElem.ownerDocument();
	// adding label
	QDomElement tmp = doc.createElement("value");
	tmp.appendChild(doc.createTextNode(value()));
	useElem.appendChild(tmp);

	return TestModelBaseItem::save(useElem);
}

QVariant TestSubItem::data (int column, int role) const
{
	if((role != Qt::DisplayRole) && (role != Qt::EditRole))
		return QVariant();

	switch(column){
		case 0:
			return label();
		case 1:
			return value();
		default:
			return QVariant();
	}
}

EXmlItem *TestSubItem::copy () const
{
	TestSubItem *res = dynamic_cast<TestSubItem*>(create(NULL));
	if(!res)
		return NULL;
	copyBase(res);
	res->setValue(value());
	return res;
}
