/*
    Copyright (C) 2010 - 2012 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/

#ifndef TESTSUBITEM_H
#define TESTSUBITEM_H

#include "testmodelbaseitem.h"

class TestItem;

class EXmlModel;

/**
	* @brief One element of the TestItem.
	*/
class TestSubItem : public TestModelBaseItem
{
	Q_OBJECT
	public:
		TestSubItem(QString label, EXmlModel *model, TestItem *parent);

		bool isValid() const;

		/**
			* @brief Returns the parent, where it belongs to.
			*/
		TestItem *testItem() const;
		EXmlModel *model() const;

		void setValue(QString value);
		/**
			* @brief Returns the value of this attribute.
			*
			* The maximum length of this string is 120 chars.
			*
			* For example all Optins within one model, should have
			* the same list of attributes, but all have different values.
			* E.g.:
			* - Option: All inclusive holiday
			*   - OptionAttribute: Target (Label) - Somewhere (Value)
			*   - OptionAttribute: Food inclusive (Label) - Yes (value)
			*/
		QString value() const;

		// EXmlItem API
		static inline EXmlItem *create(EXmlModel *model){ return new TestSubItem("", model, NULL); }
		bool load(const QDomElement element, QMap< QString, EXmlParserItem * > parser);
		bool save(QDomElement &useElem);
		QVariant data (int column, int role=Qt::DisplayRole) const;
		EXmlItem *copy() const;

	private:
		QString myValue;
		EXmlModel *myModel;

};

#endif // TESTSUBITEM_H
