/*
    Copyright (C) 2010 - 2012 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/

#include "esqlfieldtest.h"
#include "esqlfieldtest.moc"

#include <esqlfield.h>

#include <QtTest>

void ESqlFieldTest::testSetFunctions()
{
	// test constructors
	ESqlField field;
	QCOMPARE(field.dataType(), ESqlField::NoType);
	QCOMPARE(field.isValid(), false);
	QCOMPARE(field.isPrimaryKey(), false);
	QCOMPARE(field.autoIncrement(), false);
	QCOMPARE(field.notNull(), false);
	QCOMPARE(field.length(), 0u);
	QCOMPARE(field.isUnsigned(), false);
	QCOMPARE(field.isBinary(), false);
	QCOMPARE(field.zeroFill(), false);

	QString name = "test";
	ESqlField field1(name, ESqlField::Integer, true, true);
	QCOMPARE(field1.name(), name);
	QCOMPARE(field1.dataType(), ESqlField::Integer);
	QCOMPARE(field1.notNull(), true);
	QCOMPARE(field1.autoIncrement(), true);

	field1.setDefaultValue(0);
	field1.setIsBinary(true);
	field1.setIsPrimaryKey(true);
	field1.setIsUnsigned(true);
	field1.setLength(3);
	field1.setZeroFill(true);
	QCOMPARE(field1.defaultValue().toInt(), 0);
	QCOMPARE(field1.isBinary(), true);
	QCOMPARE(field1.isUnsigned(), true);
	QCOMPARE(field1.length(), 3u);
	QCOMPARE(field1.zeroFill(), true);
}

void ESqlFieldTest::testCopy()
{
	QString name = "test";
	ESqlField field(name, ESqlField::Text, true, true);
	field.setDefaultValue(0);
	field.setIsBinary(true);
	field.setIsPrimaryKey(true);
	field.setIsUnsigned(true);
	field.setLength(3);
	field.setZeroFill(true);
	
	ESqlField *field2 = field.copy();
	QCOMPARE(field2->name(), name);
	QCOMPARE(field2->dataType(), ESqlField::Text);
	QCOMPARE(field2->notNull(), true);
	QCOMPARE(field2->autoIncrement(), true);
	QCOMPARE(field2->defaultValue().toInt(), 0);
	QCOMPARE(field2->isBinary(), true);
	QCOMPARE(field2->isUnsigned(), true);
	QCOMPARE(field2->length(), 3u);
	QCOMPARE(field2->zeroFill(), true);
	
}

QTEST_MAIN(ESqlFieldTest)
