/*
    Copyright (C) 2010 - 2012 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/

#include "esqltabletest.h"
#include "esqltabletest.moc"

#include <esqltable.h>
#include "handlestrings.h"

#include <QtTest>

void ESqlTableTest::testSetFunctions()
{
	ESqlTable *table = new ESqlTable();
	QCOMPARE(table->name(), QString());
	QCOMPARE(table->isValid(), false);
	delete table;
	QString name = "test";
	table = new ESqlTable(name);
	QCOMPARE(table->name(), name);
	QCOMPARE(table->isValid(), false);
	
	table->appendFieldId();
	QList<ESqlField*> fields = table->fields();
	QCOMPARE(table->isValid(), true);
	QCOMPARE(fields.size(), 1);
	QCOMPARE(fields[0]->name(), QString("id"));
	QCOMPARE(fields[0]->dataType(), ESqlField::Integer);
	QCOMPARE(fields[0]->notNull(), true);
	QCOMPARE(fields[0]->autoIncrement(), true);
	
	table->appendField(name, ESqlField::Varchar, true, true);
	fields = table->fields();
	QCOMPARE(fields.size(), 2);
	QCOMPARE(table->isValid(), false);
	QCOMPARE(fields[1]->dataType(), ESqlField::Varchar);
	QCOMPARE(fields[1]->name(), name);
	QCOMPARE(fields[1]->notNull(), true);
	QCOMPARE(fields[1]->autoIncrement(), true);
	fields[1]->setLength(10);
	QCOMPARE(table->isValid(), true);
	
	table->appendField(fields[1]->copy());
	fields = table->fields();
	QCOMPARE(fields.size(), 3);
	QCOMPARE(fields[2]->dataType(), ESqlField::Varchar);
	QCOMPARE(fields[2]->name(), name);
	QCOMPARE(fields[2]->notNull(), true);
	QCOMPARE(fields[2]->autoIncrement(), true);
	
	QStringList values;
	values << "" << name << name;
	table->appendValues(values);
	QCOMPARE(table->values().size(), 1);
	QCOMPARE(table->values()[0][0], QString());
	QCOMPARE(table->values()[0][1], name);
	QCOMPARE(table->values()[0][2], name);
	
	delete table;
}

void ESqlTableTest::testCreateStatement()
{
	ESqlTable *table = new ESqlTable("target");
	ESqlTable *source = new ESqlTable("source");
	
	source->appendFieldId();
	table->appendFieldId();
	
	table->addForeignKey("testField1", "source");
	table->addForeignKey(table->appendField("testField2", ESqlField::Integer, true), source);
	table->addForeignKey(source, "testField3");
	QCOMPARE(table->fields().size(), 3);
	QString dbName = "testDB";
	QString statement = QString("CREATE TABLE IF NOT EXISTS `%1`.`%2` (\n"
		" `id` INT UNSIGNED  NOT NULL  AUTO_INCREMENT ,\n"
		" `testField2` INT NOT NULL ,\n"
		" `testField3` INT UNSIGNED  NOT NULL ,	PRIMARY KEY(`id`),\n"
		" CONSTRAINT `fk_%2_testField1`\n"
		"  FOREIGN KEY (`testField1`)\n"
		"  REFERENCES `%1`.`%3`(`id`)\n"
		"  ON DELETE NO ACTION   ON UPDATE NO ACTION,\n"
		" CONSTRAINT `fk_%2_testField2`\n"
		"  FOREIGN KEY (`testField2`)\n"
		"  REFERENCES `%1`.`%3`(`id`)\n"
		"  ON DELETE NO ACTION   ON UPDATE NO ACTION,\n"
		" CONSTRAINT `fk_%2_testField3`\n"
		"  FOREIGN KEY (`testField3`)\n"
		"  REFERENCES `%1`.`%3`(`id`)\n"
		"  ON DELETE NO ACTION   ON UPDATE NO ACTION )")
		.arg(dbName).arg(table->name()).arg(source->name());

	QCOMPARE(HandleStrings::strip(table->createStatement(dbName, EngSaS::SQL::Other)),
					 HandleStrings::strip(statement));
}

QTEST_MAIN(ESqlTableTest)
