/*
    Copyright (C) 2010 - 2012 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/

#include "versiontest.h"
#include "versiontest.moc"

#include "internal/version.h"

#include <QTest>

void VersionTest::basics()
{
	Version ver1;
	QCOMPARE(ver1.maj(), 0);
	QCOMPARE(ver1.min(), 0);
	QCOMPARE(ver1.patch(), 0);
	QCOMPARE(ver1.rc(), 0);
	
	Version ver2(1, 2, 3, 4);
	QCOMPARE(ver2.maj(), 1);
	QCOMPARE(ver2.min(), 2);
	QCOMPARE(ver2.patch(), 3);
	QCOMPARE(ver2.rc(), 4);
	
	Version ver3("1.2.3-rc4");
	QCOMPARE(ver3.maj(), 1);
	QCOMPARE(ver3.min(), 2);
	QCOMPARE(ver3.patch(), 3);
	QCOMPARE(ver3.rc(), 4);
}

void VersionTest::isValid()
{
	Version ver(-1, 1, 1, 1);
	QCOMPARE(ver.isValid(), false);
	ver = Version(1, -1, 1, 1);
	QCOMPARE(ver.isValid(), false);
	ver = Version(1, 1, -1, 1);
	QCOMPARE(ver.isValid(), false);
	ver = Version(1, 1, 1, -1);
	QCOMPARE(ver.isValid(), false);
	
	ver = Version();
	QCOMPARE(ver.isValid(), false);
	
	ver = Version(1, 0, 0, 0);
	QCOMPARE(ver.isValid(), true);
	ver = Version(0, 1, 0, 0);
	QCOMPARE(ver.isValid(), true);
	ver = Version(0, 0, 1, 0);
	QCOMPARE(ver.isValid(), true);
	ver = Version(0, 0, 0, 1);
	QCOMPARE(ver.isValid(), false);
}

void VersionTest::toString()
{
	Version ver(1, 2, 3, 4);
	QCOMPARE(ver.toString(), QString("1.2.3-rc4"));
	ver = Version(1, 2, 3, 0);
	QCOMPARE(ver.toString(), QString("1.2.3"));
}

void VersionTest::operators()
{
	Version ref(1, 1, 1, 0);
	Version greater(1, 1, 2, 0);
	Version smaller(1, 1, 0, 0);
	Version smaller1(1, 1, 1, 1);
	Version equal(1, 1, 1, 0);
	
	QVERIFY(ref < greater);
	QVERIFY(ref <= greater);	
	QVERIFY(ref > smaller);
	QVERIFY(ref >= smaller);
	QVERIFY(ref > smaller1);
	QVERIFY(ref >= smaller1);
	QVERIFY(ref == equal);
	QVERIFY(ref <= equal);
	QVERIFY(ref >= equal);
}

QTEST_MAIN(VersionTest)
